@@ -30,6 +30,9 @@
 
 #include "gmime-disposition.h"
 
+#include <glib/gmessages.h>
+#include <glib/gmem.h>
+#include <glib/gstrfuncs.h>
 
 static int
 param_equal (gconstpointer v, gconstpointer v2)
