@@ -37,7 +37,7 @@ typedef enum {
 	GMIME_FILTER_BASIC_QP_ENC,
 	GMIME_FILTER_BASIC_QP_DEC,
 	GMIME_FILTER_BASIC_UU_ENC,
-	GMIME_FILTER_BASIC_UU_DEC
+	GMIME_FILTER_BASIC_UU_DEC,
 } GMimeFilterBasicType;
 
 typedef struct _GMimeFilterBasic {
