@@ -972,7 +972,7 @@ multipart_get_body (GMimeMultipart *multipart, gboolean want_plain, gboolean *is
 	const char *content;
 	char *body = NULL;
 	GList *subpart;
-	size_t len;
+	guint len;
 	
 	subpart = multipart->subparts;
 	while (subpart) {
@@ -1042,7 +1042,7 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 	const GMimeContentType *type;
 	const char *content;
 	char *body = NULL;
-	size_t len = 0;
+	guint len = 0;
 	
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	g_return_val_if_fail (is_html != NULL, NULL);
