@@ -27,7 +27,7 @@
 
 #include <string.h>
 #include <sys/types.h>
-#include <regex.h>
+#include <pcreposix.h>
 #include <ctype.h>
 
 #include "gmime-parser.h"
