@@ -1174,7 +1174,7 @@ g_mime_part_get_content_object (const GMimePart *mime_part)
  * and sets %len to the length of the buffer.
  **/
 const char *
-g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
+g_mime_part_get_content (const GMimePart *mime_part, guint *len)
 {
 	const char *retval = NULL;
 	GMimeStream *stream;
