@@ -153,6 +153,10 @@ pan_action_do (PanAction action)
 			article_action_selected_save_as ();
 			break;
 
+		case ACTION_MANUAL_DECODE:
+			article_action_selected_manual_decode ();
+			break;
+
 		case ACTION_NET_ONLINE:
 			break;
 
