@@ -356,12 +356,45 @@ article_action_selected_save_attachments (void)
 typedef struct
 {
 	Group * group;
-	GSList * slists_of_articles;
+	GSList * list;
 }
 SelectedSaveAsStruct;
 
 static void
-add_article_grouping_to_gslist (Article * article, gpointer user_data)
+add_article_to_gslist (Article * article, gpointer user_data)
+{
+	SelectedSaveAsStruct * data = (SelectedSaveAsStruct*) user_data;
+
+	/* get the first group */
+	if (data->group == NULL)
+		data->group = article->group;
+
+	/* append this article and its children to the gslist of articles */
+	data->list = g_slist_append (data->list, article);
+	if (article->parts > 1)
+		data->list = g_slist_concat (data->list,  g_slist_copy(article->threads));
+}
+
+void
+article_action_selected_manual_decode (void)
+{
+	SelectedSaveAsStruct data;
+	data.group = NULL;
+	data.list = NULL;
+
+	/* build the list of articles */
+	header_pane_foreach_selected (add_article_to_gslist, &data);
+
+	/* if any articles are selected, save them */
+	if (data.list != NULL) {
+		GSList * slist_of_slists = g_slist_append (NULL, data.list);
+		manual_decode_attachment_as (slist_of_slists, data.group->download_dir);
+		/* save_attach_as owns slist_of_slists now */
+	}
+}
+
+static void
+add_article_gslist_to_gslist (Article * article, gpointer user_data)
 {
 	GSList * l = NULL;
 	SelectedSaveAsStruct * data = (SelectedSaveAsStruct*) user_data;
@@ -370,13 +403,13 @@ add_article_grouping_to_gslist (Article * article, gpointer user_data)
 	if (data->group == NULL)
 		data->group = article->group;
 
-	/* build an array of this node and all its children. */
+	/* build a new gslist of this node and its children */
 	if (article->parts > 1)
 		l = g_slist_copy (article->threads);
 	l = g_slist_prepend (l, article);
 
-	/* add this to the gslist of articles */
-	data->slists_of_articles = g_slist_prepend (data->slists_of_articles, l);
+	/* add the new gslist to the gslist-of-gslists-of-articles */
+	data->list = g_slist_prepend (data->list, l);
 }
 
 void
@@ -384,16 +417,16 @@ article_action_selected_save_as (void)
 {
 	SelectedSaveAsStruct data;
 	data.group = NULL;
-	data.slists_of_articles = NULL;
+	data.list = NULL;
 
 	/* build the list of articles */
-	header_pane_foreach_selected (add_article_grouping_to_gslist, &data);
+	header_pane_foreach_selected (add_article_gslist_to_gslist, &data);
 
 	/* if any articles are selected, save them */
-	if (data.slists_of_articles != NULL) {
-		data.slists_of_articles = g_slist_reverse (data.slists_of_articles);
-		save_attachment_as (data.slists_of_articles, data.group->download_dir);
-		/* save_attach_as owns slists_of_articles now */
+	if (data.list != NULL) {
+		data.list = g_slist_reverse (data.list);
+		save_attachment_as (data.list, data.group->download_dir);
+		/* save_attach_as owns data.list now */
 	}
 }
 
