@@ -43,10 +43,6 @@ static GtkWidget *find_dialog;
 static void
 find_dialog_destroy_cb (void)
 {
-	pan_lock();
-	gtk_widget_destroy (find_dialog);
-	pan_unlock();
-
 	find_dialog = NULL;
 }
 
@@ -80,35 +76,36 @@ test_phrase (GPatternSpec * pattern, const char * phrase, gboolean case_sensitiv
  * Return 0 if equal
  */
 static gboolean
-compare_article (const Article              * article,
-                 const header_compare_data  * header)
+compare_article (const Article * article, gpointer compare_data_gpointer)
 {
+	const header_compare_data * header = (const header_compare_data*) compare_data_gpointer;
+
 	/* if it ain't an article, it ain't a match */
 	if (!article)
-		return -1;
+		return 0;
 
 	/* make sure the unread state (if user specified) matches... */
 	if (header->unread_only && article_is_read (article))
-		return -1;
+		return 0;
 
 	/* make sure the author (if user specified one) matches.. */
 	if (header->author != NULL) {
 		char buf[512] = { '\0' };
 		article_get_author_str (article, buf, sizeof(buf));
 		if (!test_phrase (header->author, buf, header->case_sensitive))
-			return -1;
+			return 0;
 	}
 
 	/* make sure the subject (if user specified one) matches.. */
 	if (header->subject && !test_phrase (header->subject, article_get_subject(article), header->case_sensitive))
-		return -1;
+		return 0;
 
 	/* make sure the message-id (if user specified one) matches.. */
 	if (header->msgid && !test_phrase (header->msgid, article_get_message_id(article), header->case_sensitive))
-		return -1;
+		return 0;
 
 	/* hooray, a match */
-	return 0;
+	return 1;
 }
 
 static GPatternSpec*
@@ -146,14 +143,8 @@ find_article (const char   * author,
               gboolean       unread_only,
               gboolean       reverse)
 {
-	GtkCTreeNode * sel;
-	GtkCTreeNode * tmp;
-	GtkCTree* tree = GTK_CTREE(Pan.article_ctree);
 	header_compare_data header_data;
 
-	/* sanity checks */
-	g_return_if_fail (articlelist_get_group()!=NULL);
-
 	/* set up search function struct */
 	header_data.author = create_pattern (author, case_sensitive);
 	header_data.subject = create_pattern (subject, case_sensitive);
@@ -161,43 +152,11 @@ find_article (const char   * author,
 	header_data.unread_only = unread_only;
 	header_data.case_sensitive = case_sensitive;
 
-	/* search */
-	sel = articlelist_get_selected_node();
-	if (!sel)
-		sel = gtk_ctree_node_nth (tree, 0);
-	tmp = sel;
-	for ( ;; )
-	{
-		const Article* article = NULL;
-
-		/* next node */
-		tmp = reverse
-			? articlelist_node_prev (tmp, FALSE)
-			: articlelist_node_next (tmp);
-		if (!tmp) { /* wrap */
-			int n = reverse
-				? GTK_CLIST(tree)->rows-1
-				: 0;
-			tmp = gtk_ctree_node_nth (tree, n);
-			if (tmp==sel)
-				break;
-			tmp = reverse
-				? articlelist_node_prev (tmp, FALSE)
-				: articlelist_node_next (tmp);
-		}
-		if (tmp==sel)
-			break;
-
-		/* compare */
-		article = (const Article*)
-			gtk_ctree_node_get_row_data (tree, tmp);
-		if (!compare_article (article, &header_data))
-			break;
-	}
-
-	/* select the node.. */
-	if (tmp!=sel)
-		articlelist_set_selected_nodes_nolock (&tmp, 1);
+	/* select the article */
+	if (reverse)
+		header_pane_select_prev_if (compare_article, &header_data);
+	else
+		header_pane_select_next_if (compare_article, &header_data);
 
 	/* cleanup */
 	if (header_data.author)
@@ -289,10 +248,6 @@ find_dialog_response_cb (GtkDialog     * dialog,
 void
 articlelist_find_next_cb (void)
 {
-	/* user may have hit "find again" without loading a group first */
-	if (!articlelist_get_group())
-		return;
-
 	/* user may have hit "find again" without hitting "find" first */
 	if (!find_article__subject && !find_article__author && !find_article__message_id) {
 		articlelist_find_text_cb();
@@ -315,13 +270,12 @@ void
 articlelist_find_text_cb (void)
 {
 	GtkWidget * t;
-	GtkWidget * w;
 	GtkWidget * author_entry;
 	GtkWidget * subject_entry;
 	GtkWidget * message_id_entry;
 	GtkWidget * case_sensitive_checkbutton;
 	GtkWidget * unread_only_checkbutton;
-	guint       row;
+	guint       row = 0;
 
 	/* There can be only one! (find window) */
 	if (find_dialog != NULL)
@@ -331,7 +285,7 @@ articlelist_find_text_cb (void)
 		return;
 	}
 
-	find_dialog = gtk_dialog_new_with_buttons (_("Pan: Find Message"),
+	find_dialog = gtk_dialog_new_with_buttons (_("Pan: Find Article"),
 	                                           GTK_WINDOW(Pan.window),
 	                                           GTK_DIALOG_DESTROY_WITH_PARENT,
 	                                           GTK_STOCK_GO_BACK, PAN_RESPONSE_FIND_PREV,
@@ -339,60 +293,28 @@ articlelist_find_text_cb (void)
 	                                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 	                                           NULL);
 
-	row = 0;
-	t = gtk_table_new (6, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (t), 18);
-	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
-
-	w = gtk_label_new_with_mnemonic (_("_Subject:"));
-	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
-	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
-	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+1);
+	t = pan_hig_workarea_create ();
+	pan_hig_workarea_add_section_spacer (t, row, 3);
 
 	subject_entry = gtk_entry_new ( );
-	gtk_label_set_mnemonic_widget (GTK_LABEL(w), subject_entry);
-	gtk_object_set_data (GTK_OBJECT(find_dialog), "subject", subject_entry);
-	gtk_table_attach_defaults (GTK_TABLE(t), subject_entry, 1, 2, row, row+1);
-	++row;
-
-	w = gtk_label_new_with_mnemonic (_("_Author:"));
-	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
-	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
-	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+1);
+	g_object_set_data (G_OBJECT(find_dialog), "subject", subject_entry);
+	pan_hig_workarea_add_row (t, &row, _("_Subject:"), subject_entry, NULL);
 
 	author_entry = gtk_entry_new ( );
-	gtk_label_set_mnemonic_widget (GTK_LABEL(w), author_entry);
-	gtk_object_set_data (GTK_OBJECT(find_dialog), "author", author_entry);
-	gtk_table_attach_defaults (GTK_TABLE(t), author_entry, 1, 2, row, row+1);
-	++row;
-
-	w = gtk_label_new_with_mnemonic (_("_Message-ID:"));
-	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
-	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
-	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+1);
+	g_object_set_data (G_OBJECT(find_dialog), "author", author_entry);
+	pan_hig_workarea_add_row (t, &row, _("_Author:"), author_entry, NULL);
 
 	message_id_entry = gtk_entry_new ();
-	gtk_label_set_mnemonic_widget (GTK_LABEL(w), message_id_entry);
-	gtk_object_set_data ( GTK_OBJECT(find_dialog), "message-id", message_id_entry );
-	gtk_table_attach_defaults (GTK_TABLE(t), message_id_entry, 1, 2, row, row+1);
-	++row;
+	g_object_set_data (G_OBJECT(find_dialog), "message-id", message_id_entry);
+	pan_hig_workarea_add_row (t, &row, _("_Message-ID:"), message_id_entry, NULL);
 
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 12u);
-	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 2, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
 
-	case_sensitive_checkbutton = gtk_check_button_new_with_mnemonic (_("Case Sensiti_ve"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(case_sensitive_checkbutton), FALSE);
-	gtk_object_set_data (GTK_OBJECT(find_dialog), "case", case_sensitive_checkbutton);
-	gtk_table_attach_defaults (GTK_TABLE(t), case_sensitive_checkbutton, 0, 2, row, row+1);
-	++row;
+	case_sensitive_checkbutton = pan_hig_workarea_add_wide_checkbutton (t, &row, _("Case Sensiti_ve"), FALSE);
+	g_object_set_data (G_OBJECT(find_dialog), "case", case_sensitive_checkbutton);
 
-	unread_only_checkbutton = gtk_check_button_new_with_mnemonic (_("_Unread Only"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(unread_only_checkbutton), FALSE);
-	gtk_object_set_data (GTK_OBJECT(find_dialog), "unread", unread_only_checkbutton );
-	gtk_table_attach_defaults (GTK_TABLE(t), unread_only_checkbutton, 0, 2, row, row+1);
-	++row;
+	unread_only_checkbutton = pan_hig_workarea_add_wide_checkbutton (t, &row, _("_Unread Only"), FALSE);
+	g_object_set_data (G_OBJECT(find_dialog), "unread", unread_only_checkbutton);
 	
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(find_dialog)->vbox), t, FALSE, FALSE, 0);
 	
