@@ -78,7 +78,6 @@ extern gpointer   create_articlelist_ctree               (void);
 extern void articlelist_set_threaded                     (gboolean threaded_on);
 
 extern void articlelist_set_group                        (Group*);
-extern void articlelist_refresh                          (void);
 extern void articlelist_read_selected                    (void);
 extern void articlelist_activate_selected                (void);
 
@@ -93,6 +92,7 @@ extern void articlelist_select_all_nolock                (void);
 extern void articlelist_deselect_all_nolock              (void);
 extern void articlelist_add_replies_to_selection_nolock  (void);
 extern void articlelist_add_thread_to_selection_nolock   (void);
+extern void articlelist_add_set_to_selection_nolock      (void);
 
 extern void articlelist_selected_unflag_for_dl_nolock    (void);
 extern void articlelist_selected_flag_for_dl_nolock      (void);
