@@ -287,9 +287,9 @@ extern void cred_hash2(unsigned char *out,unsigned char *in,unsigned char *key);
 extern void cred_hash3(unsigned char *out,unsigned char *in,unsigned char *key, int forw);
 extern void SamOEMhash( unsigned char *data, unsigned char *key, int val);
 
-void SMBencrypt(uchar *passwd, uchar *c8, uchar *p24);
-void SMBNTencrypt(uchar *passwd, uchar *c8, uchar *p24);
-extern void mdfour(unsigned char *out, unsigned char *in, int n);
+void SMBencrypt(const uchar *passwd, uchar *c8, uchar *p24);
+void SMBNTencrypt(const uchar *passwd, uchar *c8, uchar *p24);
+extern void mdfour(unsigned char *out, const unsigned char *in, int n);
 
 
 /*
@@ -865,7 +865,7 @@ extern void SMBOWFencrypt(uchar passwd[16], uchar *c8, uchar p24[24]);
  */
 
 void
-SMBencrypt(uchar *passwd, uchar *c8, uchar *p24)
+SMBencrypt(const uchar *passwd, uchar *c8, uchar *p24)
   {
   uchar p14[15], p21[21];
   
@@ -902,7 +902,7 @@ static int _my_wcslen(int16 *str)
  * format.
  */
  
-static int _my_mbstowcs(int16 *dst, uchar *src, int len)
+static int _my_mbstowcs(int16 *dst, const uchar *src, int len)
 {
 	int i;
 	int16 val;
@@ -923,7 +923,7 @@ static int _my_mbstowcs(int16 *dst, uchar *src, int len)
  */
  
 static void
-E_md4hash(uchar *passwd, uchar *p16)
+E_md4hash(const uchar *passwd, uchar *p16)
 {
 	int len;
 	int16 wpwd[129];
@@ -957,7 +957,7 @@ void SMBOWFencrypt(uchar passwd[16], uchar *c8, uchar p24[24])
 
 /* Does the NT MD4 hash then des encryption. */
  
-void SMBNTencrypt(uchar *passwd, uchar *c8, uchar *p24)
+void SMBNTencrypt(const uchar *passwd, uchar *c8, uchar *p24)
 {
 	uchar p21[21];
  
@@ -1114,7 +1114,7 @@ static void mdfour64(uint32 *M)
 		X[j] = 0;
 }
 
-static void copy64(uint32 *M, unsigned char *in)
+static void copy64(uint32 *M, const unsigned char *in)
 {
 	int i;
 
@@ -1132,7 +1132,7 @@ static void copy4(unsigned char *out,uint32 x)
 }
 
 /* produce a md4 message digest from data of length n bytes */
-void mdfour(unsigned char *out, unsigned char *in, int n)
+void mdfour(unsigned char *out, const unsigned char *in, int n)
 {
 	unsigned char buf[128];
 	uint32 M[16];
@@ -1216,15 +1216,15 @@ else \
 
 #define AddString(ptr, header, string) \
 { \
-char *p = string; \
+const char *p = string; \
 int len = 0; \
 if (p) len = strlen(p); \
-AddBytes(ptr, header, ((unsigned char*)p), len); \
+AddBytes(ptr, header, ((const unsigned char*)p), len); \
 }
 
 #define AddUnicodeString(ptr, header, string) \
 { \
-char *p = string; \
+const char *p = string; \
 unsigned char *b = NULL; \
 int len = 0; \
 if (p) \
@@ -1257,7 +1257,7 @@ static char *unicodeToString(char *p, size_t len)
   return buf;
   }
 
-static unsigned char *strToUnicode(char *p)
+static unsigned char *strToUnicode(const char *p)
   {
   static unsigned char buf[1024];
   size_t l = strlen(p);
@@ -1275,7 +1275,7 @@ static unsigned char *strToUnicode(char *p)
   }
 
 
-void spa_build_auth_request (SPAAuthRequest *request, char *user, char *domain)
+void spa_build_auth_request (SPAAuthRequest *request, const char *user, const char *domain)
   {
     char *u = strdup(user);
     char *p = strchr(u,'@');
@@ -1296,7 +1296,7 @@ void spa_build_auth_request (SPAAuthRequest *request, char *user, char *domain)
     free(u);
   }
 
-void spa_build_auth_response (SPAAuthChallenge *challenge, SPAAuthResponse *response, char *user, char *password)
+void spa_build_auth_response (SPAAuthChallenge *challenge, SPAAuthResponse *response, const char *user, const char *password)
   {
     uint8 lmRespData[24];
     uint8 ntRespData[24];
