@@ -27,6 +27,7 @@
 
 #include <pan/base/acache.h>
 #include <pan/base/article.h>
+#include <pan/base/article-thread.h>
 #include <pan/base/debug.h>
 #include <pan/base/gnksa.h>
 #include <pan/base/group.h>
@@ -658,7 +659,7 @@ article_forall_in_threads (Article           ** articles,
 	guint i;
 	GHashTable * hash;
 	GPtrArray * array;
-	debug_enter ("articlelist_get_unique_threads");
+	debug_enter ("article_forall_in_threads");
 
 	/* get the thread for each article passed in */
 	hash = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -686,6 +687,7 @@ article_forall_in_threads (Article           ** articles,
 	/* cleanup */
 	g_ptr_array_free (array, TRUE);
 	g_hash_table_destroy (hash);
+	debug_enter ("article_forall_in_threads");
 }
 
 /***
@@ -844,6 +846,109 @@ article_get_subject (const Article * a)
 	return pstring_is_set (&a->subject) ? a->subject.str : "";
 }
 
+
+static double
+string_likeness (const PString * str1, const PString * str2)
+{
+	double retval;
+
+	if (pstring_strchr (str1, ' ') == NULL) /* only one word, so count common characters */
+	{
+		int i;
+		int common_chars = 0;
+
+		for (i=0; i<str1->len; ++i)
+			if (pstring_strchr (str2, str1->str[i]) != NULL)
+				++common_chars;
+
+		retval = (double)common_chars / str1->len;
+	}
+	else /* more than one word, so count common words */
+	{
+		int str1_words = 0;
+		int common_words = 0;
+		PString march = *str1;
+		PString token;
+
+		/* if the word is also in str2, increment common_words */
+		while (pstring_next_token_shallow (&march, ' ', &march, &token)) {
+			if (token.len) {
+				const char * pch = pstring_strstr (str2, &token);
+				if (pch!=NULL
+				    && (pch==str2->str || pch[-1]==' ')
+				    && (pch+token.len==str2->str+str2->len || pch[token.len]==' '))
+					++common_words;
+				++str1_words;
+
+			}
+		}
+
+		retval = (double)common_words / str1_words;
+	}
+
+	return retval;
+}
+
+gboolean
+article_subjects_are_similar (const Article  * article_1,
+                              const Article  * article_2)
+{
+	int i;
+	double min_closeness;
+	PString subj_1 = PSTRING_INIT;
+	PString subj_2 = PSTRING_INIT;
+	static const char * const common_substrings [] = { "mp3", "gif", "jpg", "jpeg", "yEnc" };
+
+	/* make our own copies of the strings so that we can mutilate them */
+	pstring_copy_a (&subj_1, &article_1->subject);
+	pstring_copy_a (&subj_2, &article_2->subject);
+
+	/* white-out the common substrings that most often skew string_likeness too high */
+	for (i=0; i!=G_N_ELEMENTS(common_substrings); ++i)
+	{
+		const char * needle = common_substrings[i];
+		const int needle_len = strlen (needle);
+		char * pch;
+
+		if ((pch = pan_strnstr_len (subj_1.str, subj_1.len, needle, needle_len)))
+			memset (pch, 0, needle_len);
+		if ((pch = pan_strnstr_len (subj_2.str, subj_2.len, needle, needle_len)))
+			memset (pch, 0, needle_len);
+	}
+
+	/* white-out non-alpha characters */
+	{
+		char * march;
+		char * end;
+
+		for (march=subj_1.str, end=subj_1.str+subj_1.len; march!=end; ++march)
+			if (!isalpha(*march))
+				*march = ' ';
+		for (march=subj_2.str, end=subj_2.str+subj_2.len; march!=end; ++march)
+			if (!isalpha(*march))
+				*march = ' ';
+	}
+
+	/* decide how picky we want to be.
+	   - The shorter the string, the more alike they have to be.
+	   - longer strings typically include long unique filenames. */
+	{
+		const int min_len = MIN (article_1->subject.len, article_2->subject.len);
+		const gboolean is_short_string = min_len <= 20;
+		const gboolean is_long_string = min_len >= 30;
+
+		if (is_short_string)
+			min_closeness = 0.6;
+		else if (is_long_string)
+			min_closeness = 0.5;
+		else
+			min_closeness = 0.55;
+	}
+
+	return string_likeness (&subj_1, &subj_2)  >= min_closeness;
+}
+
+
 void
 article_set_message_id (Article        * a,
 	                const PString  * message_id)
