@@ -34,18 +34,12 @@ int acache_max_megs = 10;
 gboolean acache_flush_on_exit = FALSE;
 gboolean break_thread_when_subject_changes = FALSE;
 
-static gboolean _quote_chars_impl[UCHAR_MAX];
-const gboolean * const _quote_chars = _quote_chars_impl;
-
 void
 base_prefs_init (const char    * new_download_dir,
-                 const char    * new_quote_chars,
                  int             new_acache_max_megs,
                  gboolean        new_acache_flush_on_exit,
                  gboolean        break_thread)
 {
-	int i;
-
 	replace_gstr (&download_dir, g_strdup(new_download_dir));
 	break_thread_when_subject_changes = break_thread;
 
@@ -55,12 +49,6 @@ base_prefs_init (const char    * new_download_dir,
 
 	default_incoming_name_real = _("Unknown");
 	default_incoming_name_addr = _("unknown@spammesenseless.com");
-
-	/* set _quote_chars */
-	for (i=0; i<UCHAR_MAX; ++i)
-		_quote_chars_impl[i] = FALSE;
-	for (i=0; new_quote_chars!=NULL && new_quote_chars[i]!='\0'; ++i)
-		_quote_chars_impl[(guchar)new_quote_chars[i]] = TRUE;
 }
 
 const char*
@@ -71,12 +59,9 @@ get_data_dir (void)
 	if (path == NULL)
 	{
 		const char * pan_home = g_getenv ("PAN_HOME");
-		const char * user_home = g_get_home_dir ();
+		const char * user_home = pan_get_home_dir ();
 
-		if (user_home == NULL)
-			user_home = "";
-
-		if (path==NULL && is_nonempty_string (pan_home))
+		if (path==NULL)
 			path = g_strdup (pan_home);
 
 		if (path==NULL) {
