@@ -23,6 +23,12 @@
 #include <stdio.h>
 #include <time.h>
 
+#include <glib/gstring.h>
+#include <glib/ghash.h>
+#include <glib/gthread.h>
+#include <glib/gstrfuncs.h>
+#include <glib/gmem.h>
+
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
 
