@@ -143,8 +143,8 @@ read_group_data (Server        * server,
 
 		for (i=0; i<qty; ++i)
 		{
-			const gchar * newsrc;
-			const gchar * purged;
+			const char * newsrc;
+			const char * purged;
 			Group * g;
 			glong l;
 
@@ -203,9 +203,12 @@ read_group_data (Server        * server,
 
 			g->flags = (gint8)   get_next_token_int   (march_idx, '\n', &march_idx);
 
-			if (version>=8) {
+			if (version>=8)
 				g->filter_show = get_next_token_ulong (march_idx, '\n', &march_idx);
-			}
+			else
+				g->filter_show = -1;
+			if (g->filter_show!=FILTER_SHOW_MATCHES && g->filter_show!=FILTER_SHOW_SUBTHREADS && g->filter_show!=FILTER_SHOW_THREAD)
+				g->filter_show = FILTER_SHOW_MATCHES;
 
 			/* the filter bitfields changed between versions 12 and 13 ...
 			    reset the fields for file format versions < 13 */
@@ -265,11 +268,9 @@ read_group_data (Server        * server,
 				skip_next_token (march_idx, '\n', &march_idx);
 			}
 
-			if (is_nonempty_string(newsrc))
-				newsrc_init (group_get_newsrc(g), newsrc, g->article_low, g->article_high);
+			group_set_newsrc_read_string (g, newsrc);
 
-			if (is_nonempty_string(purged))
-				newsrc_init (group_get_purged(g), purged, g->article_low, g->article_high);
+			group_set_purged_string (g, purged);
 
 			if (group_is_valid (g))
 				g_ptr_array_add (groups, g);
@@ -357,7 +358,7 @@ file_grouplist_load (Server * server, ServerGroupsType group_type, StatusItem *
 	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (group_type!=0);
 
-	if (!pan_strcmp (server->name, INTERNAL_SERVER_NAME))
+	if (pstring_equal (&server->name, &INTERNAL_SERVER_NAME))
 		return;
 
 	g_get_current_time (&start);
@@ -403,6 +404,20 @@ file_grouplist_load (Server * server, ServerGroupsType group_type, StatusItem *
 ****
 ***/
 
+static long
+write_to_file (FILE * out_fp, long * pos, const char * str)
+{
+	long retval;
+
+	if (!is_nonempty_string (str))
+		retval = -1;
+	else {
+		retval = *pos;
+		*pos += fwrite (str, 1, strlen(str)+1, out_fp);
+	}
+
+	return retval;
+}
 
 static void
 write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * status)
@@ -424,24 +439,18 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 	for (i=0; i!=groups->len; ++i)
 	{
 		Group * group = GROUP(g_ptr_array_index (groups, i));
-
-		gchar * pch = group->_newsrc==NULL
-			? NULL
-			: newsrc_get_read_str (group_get_newsrc(group));
-
-		gchar * purged = group->_purged==NULL
-			? NULL
-			: newsrc_get_read_str (group_get_purged(group));
+		char * pch = group_get_newsrc_read_string (group);
+		char * purged = group_get_purged_string (group);
 
 		/* write the group name */
-		const long name_idx = get_string_offset (NULL, NULL, group->name, dat_fp, &pos);
-		const long desc_idx = get_string_offset (NULL, NULL, group->description, dat_fp, &pos);
-		const long path_idx = get_string_offset (NULL, NULL, group->download_dir, dat_fp, &pos);
-		const long char_idx = get_string_offset (NULL, NULL, group->default_charset, dat_fp, &pos);
-		const long news_idx = get_string_offset (NULL, NULL, pch?pch:"", dat_fp, &pos);
-		const long dead_idx = get_string_offset (NULL, NULL, purged?purged:"", dat_fp, &pos);
-		const long filt_idx = get_string_offset (NULL, NULL, group->filter_name, dat_fp, &pos);
-		const long iden_idx = get_string_offset (NULL, NULL, group->identity_name, dat_fp, &pos);
+		const long name_idx = write_to_file (dat_fp, &pos, group->name.str);
+		const long desc_idx = write_to_file (dat_fp, &pos, group->description);
+		const long path_idx = write_to_file (dat_fp, &pos, group->download_dir);
+		const long char_idx = write_to_file (dat_fp, &pos, group->default_charset);
+		const long news_idx = write_to_file (dat_fp, &pos, pch);
+		const long dead_idx = write_to_file (dat_fp, &pos, purged);
+		const long filt_idx = write_to_file (dat_fp, &pos, group->filter_name);
+		const long iden_idx = write_to_file (dat_fp, &pos, group->identity_name);
 
 		/* write the non-string fields. */
 		fprintf (idx_fp,
@@ -471,10 +480,8 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 		}
 
 		/* cleanup */
-		if (pch != NULL)
-			g_free (pch);
-		if (purged != NULL)
-			g_free (purged);
+		g_free (pch);
+		g_free (purged);
 	}
 
 	debug_exit ("write_groups");
@@ -541,7 +548,7 @@ save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, St
 void
 file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem * status)
 {
-	gint save_qty = 0;
+	int save_qty = 0;
 	GTimeVal start;
 	debug_enter ("file_grouplist_save");
 
@@ -549,8 +556,9 @@ file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem *
 	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail ((group_type|SERVER_GROUPS_SUBSCRIBED)||(group_type|SERVER_GROUPS_UNSUBSCRIBED));
 
-	if (!pan_strcmp (server->name, INTERNAL_SERVER_NAME))
+	if (pstring_equal (&server->name, &INTERNAL_SERVER_NAME))
 		return;
+
 	/**
 	***  Save the Groups
 	**/
@@ -596,41 +604,49 @@ file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem *
 ***/
 
 void
-file_grouplist_server_name_changed  (const char * old_name,
-                                     const char * new_name)
+file_grouplist_server_name_changed  (const PString * old_name,
+                                     const PString * new_name)
 {
 	char old_path [PATH_MAX];
 	char new_path [PATH_MAX];
 
 	/* sanity clause */
-	g_return_if_fail (is_nonempty_string (old_name));
-	g_return_if_fail (is_nonempty_string (new_name));
+	g_return_if_fail (pstring_is_set (old_name));
+	g_return_if_fail (pstring_is_set (new_name));
 
 	/* rename the files: sub idx */
-	g_snprintf (old_path, sizeof(old_path), "%s%c%s_sub.idx", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	g_snprintf (old_path, sizeof(old_path), "%s%c%*.*s_sub.idx", get_data_dir(), G_DIR_SEPARATOR,
+	            old_name->len, old_name->len, old_name->str);
 	if (pan_file_exists (old_path)) {
-		g_snprintf (new_path, sizeof(new_path), "%s%c%s_sub.idx", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		g_snprintf (new_path, sizeof(new_path), "%s%c%*.*s_sub.idx", get_data_dir(), G_DIR_SEPARATOR,
+		            new_name->len, new_name->len, new_name->str);
 		pan_file_rename (old_path, new_path);
 	}
 
 	/* rename the files: sub dat */
-	g_snprintf (old_path, sizeof(old_path), "%s%c%s_sub.dat", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	g_snprintf (old_path, sizeof(old_path), "%s%c%*.*s_sub.dat", get_data_dir(), G_DIR_SEPARATOR,
+	            old_name->len, old_name->len, old_name->str);
 	if (pan_file_exists (old_path)) {
-		g_snprintf (new_path, sizeof(new_path), "%s%c%s_sub.dat", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		g_snprintf (new_path, sizeof(new_path), "%s%c%*.*s_sub.dat", get_data_dir(), G_DIR_SEPARATOR,
+		            new_name->len, new_name->len, new_name->str);
 		pan_file_rename (old_path, new_path);
 	}
 
 	/* rename the files: unsub idx */
-	g_snprintf (old_path, sizeof(old_path), "%s%c%s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	g_snprintf (old_path, sizeof(old_path), "%s%c%*.*s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR,
+	            old_name->len, old_name->len, old_name->str);
 	if (pan_file_exists (old_path)) {
-		g_snprintf (new_path, sizeof(new_path), "%s%c%s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		g_snprintf (new_path, sizeof(new_path), "%s%c%*.*s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR,
+		            new_name->len, new_name->len, new_name->str);
 		pan_file_rename (old_path, new_path);
 	}
 
 	/* rename the files: unsub dat */
-	g_snprintf (old_path, sizeof(old_path), "%s%c%s_unsub.dat", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	g_snprintf (old_path, sizeof(old_path), "%s%c%*.*s_unsub.dat", get_data_dir(), G_DIR_SEPARATOR,
+	            old_name->len, old_name->len, old_name->str);
 	if (pan_file_exists (old_path)) {
-		g_snprintf (new_path, sizeof(new_path), "%s%c%s_unsub.dat", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		g_snprintf (new_path, sizeof(new_path), "%s%c%*.*s_unsub.dat", get_data_dir(), G_DIR_SEPARATOR,
+		            new_name->len, new_name->len, new_name->str);
 		pan_file_rename (old_path, new_path);
 	}
 }
