@@ -76,8 +76,7 @@ gnksa_init (void)
 		                                  && ch!='>' && ch!='@' && ch!=',' && ch!=';'
 		                                  && ch!=':' && ch!='\\' && ch!='"' && ch!='.'
 		                                  && ch!='[' && ch!=']';
-		_quoted_chars[ch] = isgraph(ch) && ch!='"' && ch!='(' && ch!=')'
-		                                && ch!='<' && ch!='>' && ch!='\\';
+		_quoted_chars[ch] = isgraph(ch) && ch!='"' && ch!='<' && ch!='>' && ch!='\\';
 		_tag_chars[ch] = isgraph(ch) && ch!='!' && ch!='(' && ch!=')' && ch!='<'
 		                             && ch!='>' && ch!='@' && ch!=',' && ch!=';'
 		                             && ch!=':' && ch!='\\' && ch!='"' && ch!='['
@@ -107,7 +106,7 @@ static char _unquoted_chars[UCHAR_MAX] = {
 
 static char _quoted_chars[UCHAR_MAX] = {
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,1,1,1,1,
-	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
 	1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
 	1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -152,7 +151,7 @@ static gboolean
 read_space (const char * start, const char ** end)
 {
 	/* 1*( <HT (ASCII 9)> / <blank (ASCII 32)> */
-	const gchar * pch = start;
+	const char * pch = start;
 	while (*pch=='\t' || *pch==' ') ++pch;
 	if (pch != start) {
 		*end = pch;
@@ -348,8 +347,8 @@ gnksa_check_domain (const char * domain, guint domain_len)
 	int label_qty;
 	const char * top;
 	const char * pch;
-	const gchar * label = NULL;
-	gint label_len = 0;
+	const char * label = NULL;
+	int label_len = 0;
 
 	debug1 (DEBUG_GNKSA, "gnksa_check_domain: [%s]", domain);
 
@@ -419,9 +418,9 @@ gnksa_check_domain (const char * domain, guint domain_len)
 static int
 gnksa_check_localpart (const char * localpart)
 {
-	gint word_len = 0;
-	const gchar * pch;
-	const gchar * word = NULL;
+	int word_len = 0;
+	const char * pch;
+	const char * word = NULL;
 
 	/* make sure it's not empty... */
 	if (!is_nonempty_string(localpart))
@@ -431,7 +430,7 @@ gnksa_check_localpart (const char * localpart)
 	pch = localpart;
 	while (get_next_token_run (pch, '.', &pch, &word, &word_len))
 	{
-		gint i;
+		int i;
 
 		if (word_len < 1)
 			return GNKSA_ZERO_LENGTH_LOCAL_WORD;
@@ -448,12 +447,12 @@ gnksa_check_localpart (const char * localpart)
 ******
 *****/
 
-static gint
-gnksa_check_address (const gchar * address, gint address_len)
+static int
+gnksa_check_address (const char * address, int address_len)
 {
-	gint retval = GNKSA_OK;
-	gchar * work;
-	gchar * begin;
+	int retval = GNKSA_OK;
+	char * work;
+	char * begin;
 
 	/* get rid of vacuous case */
 	if (!is_nonempty_string(address))
@@ -490,18 +489,18 @@ gnksa_check_address (const gchar * address, gint address_len)
 *****/
 
 static int
-gnksa_split_from (const gchar  * from,
-                  gchar        * addr_buf,
-                  gint           addr_max,
-                  gchar        * name_buf,
-                  gint           name_max,
-		  gint         * addrtype,
-		  gboolean       strict)
+gnksa_split_from (const char  * from,
+                  char        * addr_buf,
+                  int           addr_max,
+                  char        * name_buf,
+                  int           name_max,
+		  int         * addrtype,
+		  gboolean      strict)
 {
-	gchar * begin, * addr_buf_rangle;
-	gchar * end;
-	gchar * work;
-	gchar * lparen;
+	char * begin, * addr_buf_rangle;
+	char * end;
+	char * work;
+	char * lparen;
 
 	*addr_buf = '\0';
 	*name_buf = '\0';
@@ -545,18 +544,17 @@ gnksa_split_from (const gchar  * from,
 	else if ((lparen = strchr (work,'(')) != NULL) /* charles@rebelbase.com (Charles Kerr) */
 	{
 		size_t len;
-		gchar * real;
-		const gchar * end;
+		char * real;
+		const char * end;
 
 		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
 
 		/* address part */
-		len = MIN(addr_max, lparen-work);
-		g_strlcpy (addr_buf, work, len);
+		pan_strncpy_len (addr_buf, addr_max, work, lparen-work);
 		g_strstrip (addr_buf);
 
 		if (strict) {
-			gint val = gnksa_check_address (addr_buf, strlen(addr_buf));
+			int val = gnksa_check_address (addr_buf, strlen(addr_buf));
 			if (val)
 				return val;
 		}
@@ -573,7 +571,7 @@ gnksa_split_from (const gchar  * from,
 		if (strict && (!read_paren_phrase(lparen+1,&end) || end==NULL))
 			return GNKSA_ILLEGAL_PAREN_PHRASE;
 
-		g_strlcpy (name_buf, real, name_max);
+		pan_strncpy_len (name_buf, name_max, real, len);
 	}
 	else if (strchr(work,'@') != NULL) /* charles@rebelbase.com */
 	{
@@ -647,9 +645,9 @@ gnksa_split_from (const gchar  * from,
 
 int
 gnksa_do_check_from (const char   * from,
-                     gchar        * addr,
+                     char        * addr,
                      guint          addr_max,
-                     gchar        * name,
+                     char        * name,
                      guint          name_max,
 		     gboolean       strict)
 {
@@ -683,9 +681,9 @@ gnksa_do_check_from (const char   * from,
 *****/
 
 void
-gnksa_strip_realname (gchar * realname)
+gnksa_strip_realname (char * realname)
 {
-	gchar * pch;
+	char * pch;
 
 	g_return_if_fail (is_nonempty_string(realname));
 
@@ -710,8 +708,8 @@ int
 gnksa_check_from (const char* from, gboolean strict)
 {
 	int retval;
-	gchar addr[512];
-	gchar name[512];
+	char addr[512];
+	char name[512];
 
 	retval = gnksa_do_check_from (from,
 	                              addr, sizeof(addr),
@@ -741,11 +739,11 @@ gnksa_check_from (const char* from, gboolean strict)
  * to work properly.
  */
 static int
-gnksa_check_message_id (const char* message_id, gint message_id_len)
+gnksa_check_message_id (const char* message_id, int message_id_len)
 {
 	int val;
-	gchar * tmp;
-	gchar * pch;
+	char * tmp;
+	char * pch;
 
 	/* make a temporary copy that we can poke */
        	pan_strndup_alloca (tmp, message_id, message_id_len);
@@ -842,8 +840,8 @@ remove_broken_message_ids_from_references (const char * references, GString * se
  * @param untrimmed "references" string
  * @return newly-allocated trimmed "references" string
  */
-gchar*
-gnksa_trim_references_to_len (const gchar* refs, int cutoff)
+char*
+gnksa_trim_references_to_len (const char* refs, int cutoff)
 {
 	char * retval;
 	GString * fixed;
@@ -938,11 +936,11 @@ gnksa_generate_message_id (const char * domain_name)
 	return g_string_free (msgid, FALSE);
 }
 
-gchar*
-gnksa_generate_message_id_from_email_addr (const gchar * addr)
+char*
+gnksa_generate_message_id_from_email_addr (const char * addr)
 {
-	const gchar * pch;
-	gchar * message_id;
+	const char * pch;
+	char * message_id;
 
 	/* find the domain in the email address */
 	pch = NULL;
@@ -960,10 +958,10 @@ gnksa_generate_message_id_from_email_addr (const gchar * addr)
 
 	/* make a copy of the rhs of the email address and trim out the '>' */
 	if (1) {
-		gchar * domain = pan_strdup_alloca (pch);
+		char * domain = pan_strdup_alloca (pch);
 		pch = strchr (domain, '>');
 		if (pch != NULL)
-			*((gchar*)pch) = '\0';
+			*((char*)pch) = '\0';
 		g_strstrip (domain);
 		message_id = gnksa_generate_message_id (domain);
 	}
@@ -973,18 +971,18 @@ gnksa_generate_message_id_from_email_addr (const gchar * addr)
 }
 
 
-static gchar*
-gnksa_trim_references (const gchar* refs)
+static char*
+gnksa_trim_references (const char* refs)
 {
 	const int GNKSA_CUTOFF = 986; /* GNKSA rule 7: 998 chars - 12 for "References: " */
 	return gnksa_trim_references_to_len (refs, GNKSA_CUTOFF);
 }
 
-gchar*
-gnksa_generate_references (const gchar * references,
-                           const gchar * message_id)
+char*
+gnksa_generate_references (const char * references,
+                           const char * message_id)
 {
-	gchar * retval = NULL;
+	char * retval = NULL;
 
 	if (is_nonempty_string (message_id))
 	{
@@ -1066,8 +1064,9 @@ pan_is_signature_delimiter (const char * line, const int line_len)
 	return SIG_NONE;
 }
 
-gboolean
-pan_remove_signature (char * body)
+SigType
+pan_find_signature_delimiter  (const char     * body,
+                               char          ** setme_delimiter)
 {
 	const static int SIG_THRESHOLD = 6;
 	const char * march = body;
@@ -1077,6 +1076,9 @@ pan_remove_signature (char * body)
 	int lines_below = 0;
 	int line_len = 0;
 
+	if (setme_delimiter != NULL)
+		*setme_delimiter = NULL;
+
 	/*iterate through the text, line by line*/
 	while (get_next_token_run (march, '\n', &march, (const char**)&line, &line_len))
 	{
@@ -1094,12 +1096,13 @@ pan_remove_signature (char * body)
 				++lines_below;
 		}
 	}
-	if (sig_type == SIG_NONE) return FALSE;
-	if (sig_point == NULL) return FALSE;
+	if (sig_type == SIG_NONE) return SIG_NONE;
+	if (sig_point == NULL) return SIG_NONE;
 	if (sig_type == SIG_STANDARD)
 	{
-		*sig_point = '\0';
-		return TRUE;
+		if (setme_delimiter != NULL)
+			*setme_delimiter = sig_point;
+		return SIG_STANDARD;
 	}
 	
 	
@@ -1108,9 +1111,23 @@ pan_remove_signature (char * body)
 	 */
 	if (sig_type == SIG_NONSTANDARD && lines_below <= SIG_THRESHOLD )
 	{
-		*sig_point = '\0';
-		return TRUE;
+		if (setme_delimiter != NULL)
+			*setme_delimiter = sig_point;
+		return SIG_NONSTANDARD;
 	}
 	
-	return FALSE;
+	return SIG_NONE;
+}
+
+
+gboolean
+pan_remove_signature (char * body)
+{
+	char * sig_point = NULL;
+	const gboolean has_signature = pan_find_signature_delimiter (body, &sig_point) != SIG_NONE;
+
+	if (has_signature)
+		*sig_point = '\0';
+
+	return has_signature;
 }
