@@ -1001,94 +1001,99 @@ remove_articles_from_thread (Article    ** articles,
 void
 group_remove_articles (Group * group, Article ** articles, guint article_qty)
 {
-	guint i;
-	int removed_read;
-	GPtrArray * tmp_remove;
-	GPtrArray * removed;
-	GHashTable * ours;
 	debug_enter ("group_remove_articles");
 
 	/* sanity clause */
 	g_return_if_fail (group_is_valid (group));
 	g_return_if_fail (group->_articles_refcount>=0);
-	g_return_if_fail (articles!=NULL);
-	if (article_qty == 0) return;
-	g_return_if_fail (article_qty > 0);
-	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
-	g_return_if_fail (articles[0]->group == group);
 
-	/* let everyone know that articles are going to be removed */
-	tmp_remove = g_ptr_array_new ();
-	pan_g_ptr_array_assign (tmp_remove, (gpointer*)articles, article_qty);
-	sort_articles ((Article**)tmp_remove->pdata, tmp_remove->len, ARTICLE_SORT_MSG_ID, TRUE);
-	pan_callback_call (group_get_articles_removed_callback(), group, tmp_remove);
-
-	/**
-	***  Try to remove the articles
-	**/
-
-	g_mutex_lock (group_get_article_mutex (group));
-	ours = group_get_articles (group);
-	removed_read = 0;
-	removed = g_ptr_array_sized_new (article_qty);
-	for (i=0; i!=tmp_remove->len; ++i)
+	if (article_qty > 0)
 	{
-		const Article * a = ARTICLE (g_ptr_array_index(tmp_remove,i));
-		const PString * message_id = &a->message_id;
-		Article * our_a = ARTICLE (g_hash_table_lookup (ours, message_id));
+		guint i;
+		int removed_read;
+		GPtrArray * tmp_remove;
+		GPtrArray * removed;
+		GHashTable * ours;
 
-		if (our_a != NULL)
+		g_return_if_fail (articles!=NULL);
+		g_return_if_fail (article_qty > 0);
+		g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
+		g_return_if_fail (articles[0]->group == group);
+
+		/* let everyone know that articles are going to be removed */
+		tmp_remove = g_ptr_array_new ();
+		pan_g_ptr_array_assign (tmp_remove, (gpointer*)articles, article_qty);
+		sort_articles ((Article**)tmp_remove->pdata, tmp_remove->len, ARTICLE_SORT_MSG_ID, TRUE);
+		pan_callback_call (group_get_articles_removed_callback(), group, tmp_remove);
+
+		/**
+		***  Try to remove the articles
+		**/
+
+		g_mutex_lock (group_get_article_mutex (group));
+		ours = group_get_articles (group);
+		removed_read = 0;
+		removed = g_ptr_array_sized_new (article_qty);
+		for (i=0; i!=tmp_remove->len; ++i)
 		{
-			g_hash_table_remove (ours, message_id);
+			const Article * a = ARTICLE (g_ptr_array_index(tmp_remove,i));
+			const PString * message_id = &a->message_id;
+			Article * our_a = ARTICLE (g_hash_table_lookup (ours, message_id));
 
-			/* add to the 'removed' array */
-			g_ptr_array_add (removed, our_a);
+			if (our_a != NULL)
+			{
+				g_hash_table_remove (ours, message_id);
 
-			/* count how many removed articles were read */
-			if (article_is_read (our_a))
-				++removed_read;
+				/* add to the 'removed' array */
+				g_ptr_array_add (removed, our_a);
+
+				/* count how many removed articles were read */
+				if (article_is_read (our_a))
+					++removed_read;
+			}
 		}
+
+		/**
+		***  Rethread the affected articles
+		**/
+
+		if (removed->len)
+		{
+			/* add removed articles to the 'deleted' list */
+			if (group->_deleted_articles == NULL)
+				group->_deleted_articles = g_ptr_array_new ();
+			pan_g_ptr_array_append (group->_deleted_articles, removed->pdata, removed->len);
+
+			/* rethread the affected branches */
+			article_forall_in_threads ((Article**)removed->pdata, removed->len, GET_WHOLE_THREAD,
+						   remove_articles_from_thread, removed);
+
+			group_set_articles_dirty (group);
+		}
+
+		/**
+		***  Unlock the lock
+		**/
+
+		g_mutex_unlock (group_get_article_mutex (group));
+
+		/**
+		***  Update the stats
+		**/
+
+		if (removed->len != 0) {
+			guint change_type = 0u;
+			change_type |= group_set_article_qty_impl (group, group->article_qty - removed->len);
+			change_type |= group_set_article_read_qty_impl (group, group->article_read_qty - removed_read);
+			if (change_type)
+				fire_groups_changed (&group, 1, change_type);
+		}
+
+		/* cleanup */
+		g_ptr_array_free (removed, TRUE);
+		g_ptr_array_free (tmp_remove, TRUE);
 	}
 
-	/**
-	***  Rethread the affected articles
-	**/
-
-	if (removed->len)
-	{
-		/* add removed articles to the 'deleted' list */
-		if (group->_deleted_articles == NULL)
-			group->_deleted_articles = g_ptr_array_new ();
-		pan_g_ptr_array_append (group->_deleted_articles, removed->pdata, removed->len);
-
-		/* rethread the affected branches */
-		article_forall_in_threads ((Article**)removed->pdata, removed->len, GET_WHOLE_THREAD,
-		                           remove_articles_from_thread, removed);
-
-		group_set_articles_dirty (group);
-	}
-
-	/**
-	***  Unlock the lock
-	**/
-
-	g_mutex_unlock (group_get_article_mutex (group));
-
-	/**
-	***  Update the stats
-	**/
-
-	if (removed->len != 0) {
-		guint change_type = 0u;
-		change_type |= group_set_article_qty_impl (group, group->article_qty - removed->len);
-		change_type |= group_set_article_read_qty_impl (group, group->article_read_qty - removed_read);
-		if (change_type)
-			fire_groups_changed (&group, 1, change_type);
-	}
-
-	/* cleanup */
-	g_ptr_array_free (removed, TRUE);
-	g_ptr_array_free (tmp_remove, TRUE);
 	debug_exit ("group_remove_articles");
 }
 
