@@ -38,7 +38,7 @@ pan_message_identifier_init (MessageIdentifier * mid, MessageIdentifierClass * k
 {
 	mid->line_qty = 0ul;
 	mid->byte_qty = 0ul;
-	mid->message_id = NULL;
+	mid->message_id = PSTRING_INIT;
 	mid->readable_name = NULL;
 	mid->message_sources = g_ptr_array_new ();
 }
@@ -52,16 +52,15 @@ pan_message_identifier_finalize (GObject * object)
 	for (i=0; i<mid->message_sources->len; ++i)
 	{
 		MessageSource * source = (MessageSource*) g_ptr_array_index (mid->message_sources, i);
-		g_free (source->server_name);
-		g_free (source->group_name);
+		pstring_clear (&source->server_name);
+		pstring_clear (&source->group_name);
 		g_free (source);
 	}
 	g_ptr_array_free (mid->message_sources, TRUE);
 	g_free (mid->readable_name);
-	g_free (mid->message_id);
+	pstring_clear (&mid->message_id);
 
 	mid->message_sources = NULL;
-	mid->message_id = NULL;
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -112,7 +111,7 @@ message_identifier_new (const char * message_id)
 	MessageIdentifier * mid;
 
 	mid = g_object_new (PAN_TYPE_MESSAGE_IDENTIFIER, NULL, NULL);
-	mid->message_id = g_strdup (message_id);
+	pstring_set (&mid->message_id, message_id, strlen(message_id));
 
 	return mid;
 }
@@ -124,7 +123,8 @@ message_identifier_new_from_article (const Article * article)
 
 	g_return_val_if_fail (article_is_valid(article), NULL);
 
-	mid = message_identifier_new (article_get_message_id(article));
+	mid = g_object_new (PAN_TYPE_MESSAGE_IDENTIFIER, NULL, NULL);
+	pstring_copy (&mid->message_id, &article->message_id);
 	message_identifier_set_readable_name (mid, article_get_subject(article));
 	message_identifier_add_source_from_article (mid, article);
 
@@ -133,21 +133,23 @@ message_identifier_new_from_article (const Article * article)
 
 static MessageSource*
 message_identifier_get_source_for_group (MessageIdentifier   * mid,
-                                         const char          * server_name,
-                                         const char          * group_name)
+                                         const PString       * server_name,
+                                         const PString       * group_name)
 {
 	guint i;
 	MessageSource * source = NULL;
 
 	/* sanity clause */
 	g_return_val_if_fail (PAN_IS_MESSAGE_IDENTIFIER(mid), NULL);
-	g_return_val_if_fail (is_nonempty_string (server_name), NULL);
-	g_return_val_if_fail (is_nonempty_string (group_name), NULL);
+	g_return_val_if_fail (pstring_is_set (server_name), NULL);
+	g_return_val_if_fail (pstring_is_set (group_name), NULL);
 
+	/* find the source matching this server+group */
 	for (i=0; source==NULL && i<mid->message_sources->len; ++i) {
 		MessageSource * it = (MessageSource*) g_ptr_array_index (mid->message_sources, i);
-		if (!strcmp(server_name,it->server_name) && !strcmp (group_name, it->group_name))
+		if (pstring_equal (server_name, &it->server_name) && pstring_equal (group_name, &it->group_name)) {
 			source = it;
+		}
 	}
 
 	return source;
@@ -155,21 +157,23 @@ message_identifier_get_source_for_group (MessageIdentifier   * mid,
 
 void
 message_identifier_add_source (MessageIdentifier  * mid,
-	                       const char         * server,
-	                       const char         * group,
+	                       const PString      * server,
+	                       const PString      * group,
 	                       gulong               number)
 {
 	MessageSource * source;
 
 	/* sanity clause */
 	g_return_if_fail (PAN_IS_MESSAGE_IDENTIFIER(mid));
-	g_return_if_fail (is_nonempty_string (server));
-	g_return_if_fail (is_nonempty_string (group));
+	g_return_if_fail (pstring_is_set (server));
+	g_return_if_fail (pstring_is_set (group));
 	g_return_if_fail (message_identifier_get_source_for_group (mid, server, group) == NULL);
 
 	source = g_new (MessageSource, 1);
-	source->server_name = g_strdup (server);
-	source->group_name = g_strdup (group);
+	source->server_name = PSTRING_INIT;
+	source->group_name = PSTRING_INIT;
+	pstring_copy (&source->server_name, server);
+	pstring_copy (&source->group_name, group);
 	source->number = number;
 	g_ptr_array_add (mid->message_sources, source);
 }
@@ -177,9 +181,7 @@ message_identifier_add_source (MessageIdentifier  * mid,
 static void
 add_source_from_article_xreffunc (Server * server, Group * group, gulong number, gpointer user_data)
 {
-	MessageIdentifier * mid = MESSAGE_IDENTIFIER(user_data);
-
-	message_identifier_add_source (mid, server->name, group->name, number);
+	message_identifier_add_source (MESSAGE_IDENTIFIER(user_data), &server->name, &group->name, number);
 }
 
 void
@@ -191,7 +193,7 @@ message_identifier_add_source_from_article  (MessageIdentifier  * mid,
 
 	mid->line_qty = article->linecount;
 	mid->byte_qty = article->byte_qty;
-	message_identifier_add_source (mid, article->group->server->name, article->group->name, article->number);
+	message_identifier_add_source (mid, &article->group->server->name, &article->group->name, article->number);
 	article_xref_foreach (article, add_source_from_article_xreffunc, mid, SERVER_GROUPS_ALL, TRUE);
 }
 
@@ -199,13 +201,13 @@ gboolean
 message_identifier_is_valid (const MessageIdentifier * mid)
 {
 	g_return_val_if_fail (PAN_IS_MESSAGE_IDENTIFIER(mid), FALSE);
-	g_return_val_if_fail (is_nonempty_string(mid->message_id), FALSE);
+	g_return_val_if_fail (pstring_is_set (&mid->message_id), FALSE);
 
 	return TRUE;
 }
 
 gboolean
-message_identifiers_are_valid (const MessageIdentifier ** mids, int qty)
+message_identifiers_are_valid (const MessageIdentifier * const * mids, int qty)
 {
 	int i;
 	g_return_val_if_fail (qty >= 1, FALSE);
@@ -217,29 +219,29 @@ message_identifiers_are_valid (const MessageIdentifier ** mids, int qty)
 
 MessageSource*
 message_identifier_get_source_for_server   (MessageIdentifier  * mid,
-                                            const char         * server)
+                                            const PString      * server)
 {
 	int i;
 	MessageSource * retval = NULL;
 
 	/* sanity clause */
 	g_return_val_if_fail (PAN_IS_MESSAGE_IDENTIFIER(mid), NULL);
-	g_return_val_if_fail (is_nonempty_string(server), NULL);
+	g_return_val_if_fail (pstring_is_set(server), NULL);
 
 	/* return the first for the matching that server */
 	for (i=0; !retval && i<mid->message_sources->len; ++i) {
 		MessageSource * source = (MessageSource*) g_ptr_array_index (mid->message_sources, i);
-		if (!pan_strcmp (server, source->server_name))
+		if (pstring_equal (server, &source->server_name))
 			retval = source;
 	}
 
 	return retval;
 }
 
-const char*
+const PString*
 message_identifier_get_primary_group (MessageIdentifier * mid)
 {
-	const char * retval = NULL;
+	const PString* retval = NULL;
 
 	/* sanity clause */
 	g_return_val_if_fail (PAN_IS_MESSAGE_IDENTIFIER(mid), NULL);
@@ -247,7 +249,7 @@ message_identifier_get_primary_group (MessageIdentifier * mid)
 	/* find the first group name */
 	if (mid->message_sources->len) {
 		MessageSource * source = (MessageSource*) g_ptr_array_index (mid->message_sources, 0);
-		retval = source->group_name;
+		retval = &source->group_name;
 	}
 
 	return retval;
@@ -274,7 +276,7 @@ group2art_delete_ghfunc (gpointer key, gpointer val, gpointer user_data)
 }
 
 void
-message_identifiers_delete (const MessageIdentifier ** mids,
+message_identifiers_delete (const MessageIdentifier * const * mids,
                             int                        mid_qty,
 			    ServerGroupsType           type)
 {
@@ -301,12 +303,12 @@ message_identifiers_delete (const MessageIdentifier ** mids,
 			Group * group;
 
 			/* find the source's server */
-			server = serverlist_get_named_server (source->server_name);
+			server = serverlist_get_named_server (&source->server_name);
 			if (server == NULL)
 				continue;
 
 			/* find the source's group */
-			group = server_get_named_group_in_type (server, source->group_name, type);
+			group = server_get_named_group_in_type (server, &source->group_name, type);
 			if (group == NULL)
 				continue;
 
@@ -315,10 +317,10 @@ message_identifiers_delete (const MessageIdentifier ** mids,
 			 * the newsrc directly and not need to fire an event.
 			 * Otherwise, the articles are loaded, so we need to
 			 * use the Article objects. */
-			if (!group_ref_articles_if_loaded (group)) {
-				newsrc_mark_article (group_get_purged(group), source->number, TRUE);
-			} else {
-				Article * a = group_get_article_by_message_id (group, mid->message_id);
+			if (!group_ref_articles_if_loaded (group))
+				group_mark_article_purged (group, source->number);
+			else {
+				Article * a = group_get_article_by_message_id (group, &mid->message_id);
 				if (a != NULL) {
 					/* The Article container we use is a Hash, to weed out duplicates. */
 					GHashTable * art_hash = (GHashTable*) g_hash_table_lookup (group2art, group);
@@ -363,7 +365,7 @@ group2art_mark_read_ghfunc (gpointer key, gpointer val, gpointer user_data)
 }
 
 void
-message_identifiers_mark_read (const MessageIdentifier ** mids,
+message_identifiers_mark_read (const MessageIdentifier * const * mids,
                                int                        mid_qty,
                                gboolean                   read,
 			       ServerGroupsType           type)
@@ -391,12 +393,12 @@ message_identifiers_mark_read (const MessageIdentifier ** mids,
 			Group * group;
 
 			/* find the source's server */
-			server = serverlist_get_named_server (source->server_name);
+			server = serverlist_get_named_server (&source->server_name);
 			if (server == NULL)
 				continue;
 
 			/* find the source's group */
-			group = server_get_named_group_in_type (server, source->group_name, type);
+			group = server_get_named_group_in_type (server, &source->group_name, type);
 			if (group == NULL)
 				continue;
 
@@ -405,10 +407,10 @@ message_identifiers_mark_read (const MessageIdentifier ** mids,
 			 * the newsrc directly and not need to fire an event.
 			 * Otherwise, the articles are loaded, so we need to
 			 * use the Article objects. */
-			if (!group_ref_articles_if_loaded (group)) {
-				newsrc_mark_article (group_get_newsrc(group), source->number, read);
-			} else {
-				Article * a = group_get_article_by_message_id (group, mid->message_id);
+			if (!group_ref_articles_if_loaded (group))
+				group_mark_article_read (group, source->number, read);
+			else {
+				Article * a = group_get_article_by_message_id (group, &mid->message_id);
 				if (a != NULL) {
 					/* The Article container we use is a Hash, to weed out duplicates. */
 					GHashTable * art_hash = (GHashTable*) g_hash_table_lookup (group2art, group);
@@ -447,3 +449,19 @@ message_identifier_set_readable_name (MessageIdentifier  * mid,
 
 	replace_gstr (&mid->readable_name, g_strdup(name));
 }
+
+const PString**
+message_identifiers_get_id_array (const MessageIdentifier * const * mids,
+                                  int                               mid_qty)
+{
+	int i;
+	const PString ** retval = NULL;
+
+	g_return_val_if_fail (message_identifiers_are_valid (mids, mid_qty), NULL);
+
+	retval = g_new0 (const PString*, mid_qty);
+	for (i=0; i<mid_qty; ++i)
+		retval[i] = &mids[i]->message_id;
+
+	return retval;
+}
