@@ -22,8 +22,9 @@
 
 #include <glib/gtypes.h>
 #include <glib-object.h>
-#include <pan/base/server.h>
 #include <pan/base/article.h>
+#include <pan/base/pstring.h>
+#include <pan/base/server.h>
 
 #define PAN_TYPE_MESSAGE_IDENTIFIER             (pan_message_identifier_get_type ())
 #define MESSAGE_IDENTIFIER(obj)                 (G_TYPE_CHECK_INSTANCE_CAST ((obj), PAN_TYPE_MESSAGE_IDENTIFIER, MessageIdentifier))
@@ -38,7 +39,7 @@ struct _MessageIdentifier
 {
 	GObject parent_object;
 
-	char * message_id;
+	PString message_id;
 	char * readable_name;
 	unsigned long line_qty;
 	unsigned long byte_qty;
@@ -47,8 +48,8 @@ struct _MessageIdentifier
 
 typedef struct
 {
-	char * server_name;
-	char * group_name;
+	PString server_name;
+	PString group_name;
 	unsigned long number;
 }
 MessageSource;
@@ -62,25 +63,25 @@ GType pan_message_identifier_get_type (void);
 
 gboolean             message_identifier_is_valid                   (const MessageIdentifier   * mid);
 
-gboolean             message_identifiers_are_valid                 (const MessageIdentifier  ** mids,
-                                                                    int                         mid_qty);
+gboolean             message_identifiers_are_valid                 (const MessageIdentifier * const * mids,
+                                                                    int                               mid_qty);
 
 MessageIdentifier*   message_identifier_new                        (const char         * message_id);
 
 MessageIdentifier*   message_identifier_new_from_article           (const Article      * article);
 
 void                 message_identifier_add_source                 (MessageIdentifier  * mid,
-                                                                    const char         * server,
-                                                                    const char         * group,
+                                                                    const PString      * server,
+                                                                    const PString      * group,
                                                                     gulong               number);
 
 void                 message_identifier_add_source_from_article    (MessageIdentifier  * mid,
                                                                     const Article      * article);
 
 MessageSource*       message_identifier_get_source_for_server      (MessageIdentifier  * mid,
-                                                                    const char         * server);
+                                                                    const PString      * server);
 
-const char*          message_identifier_get_primary_group          (MessageIdentifier  * mid);
+const PString*       message_identifier_get_primary_group          (MessageIdentifier  * mid);
 
 const char*          message_identifier_get_readable_name          (const MessageIdentifier * mid);
 
@@ -91,13 +92,16 @@ void                 message_identifier_set_readable_name          (MessageIdent
 ***
 **/
 
-void                 message_identifiers_mark_read                 (const MessageIdentifier ** mids,
-                                                                    int                        mid_qty,
-                                                                    gboolean                   read,
-                                                                    ServerGroupsType           type);
+void                 message_identifiers_mark_read                 (const MessageIdentifier * const * mids,
+                                                                    int                               mid_qty,
+                                                                    gboolean                          read,
+                                                                    ServerGroupsType                  type);
 
-void                 message_identifiers_delete                    (const MessageIdentifier ** mids,
-                                                                    int                        mid_qty,
-                                                                    ServerGroupsType           type);
+void                 message_identifiers_delete                    (const MessageIdentifier * const * mids,
+                                                                    int                               mid_qty,
+                                                                    ServerGroupsType                  type);
+
+const PString**      message_identifiers_get_id_array              (const MessageIdentifier * const * mids,
+                                                                    int                               mid_qty);
 
 #endif
