@@ -58,13 +58,20 @@ Newsrc*   newsrc_new              (const char         * read_str,
  * Reset an existing newsrc object.
  * @param newsrc the newsrc object to be updated.
  * @param read_str same as read_str in newsrc_new.
- * @param group_low the low article number in the group.
- * @param group_high the high article number in the group.
+ * @param group_low the low article number in the group.  Numbers below this value in read_str will be trimmed.
+ * @param group_high the high article number in the group.  Numbers above this value in read_str will be trimmed.
  */
-void      newsrc_init             (Newsrc             * newsrc,
-                                   const char         * read_str,
-                                   gulong               group_low,
-                                   gulong               group_high);
+void      newsrc_init                 (Newsrc             * newsrc,
+                                       const char         * read_str,
+                                       gulong               group_low,
+                                       gulong               group_high);
+
+/**
+ * Equivalent to newsrc_init where group_low and group_high
+ * are defined from the boundaries of range_str.
+ */
+void      newsrc_init_from_range_str  (Newsrc             * newsrc,
+                                       const char         * range_str);
 
 /**
 ***  Client functions
@@ -123,17 +130,21 @@ void      newsrc_import_line      (Newsrc             * newsrc,
                                    gboolean           * setme_subscribed,
                                    const char         * import_string);
 
+gboolean  newsrc_parse_raw_line   (const char     * raw_line,
+                                   GString        * setme_group_name_or_NULL,
+                                   gboolean       * setme_subscribed_or_NULL,
+                                   GString        * setme_range_string_or_NULL);
+                                     
+
 /**
  * Generate a line suitable for writing to a .newsrc file.
  * @param newsrc the newsrc object whose article-read info is to be used
  * @param group_name the name to be written to the output string
  * @param subscribed whether or not the group is subscribed.
- * @param appendme the GString to which the .newsrc line should be appended.
  */
-void      newsrc_export_line      (const Newsrc       * newsrc,
+char*      newsrc_export_line     (const Newsrc       * newsrc,
                                    const char         * group_name,
-                                   gboolean             subscribed,
-                                   GString            * appendme);
+                                   gboolean             subscribed);
 
 /**
 ***  Housekeeping
