@@ -189,6 +189,48 @@ pan_strcmp_len  (const char * str_a,
         return retval;
 }
 
+gchar *
+pan_strnstr_len (const char * haystack,
+                 int          haystack_len,
+                 const char * needle,
+                 int          needle_len)
+{
+	g_return_val_if_fail (haystack != NULL, NULL);
+	g_return_val_if_fail (needle != NULL, NULL);
+                                                                                                                                           
+	if (haystack_len<0 || needle_len<0)
+		return strstr (haystack, needle);
+	else
+	{
+		const char *p = haystack;
+		const char *end;
+
+		if (needle_len == 0)
+			return (char*) haystack;
+
+		if (haystack_len < needle_len)
+			return NULL;
+
+		end = haystack + haystack_len - needle_len;
+
+		while (*p && p <= end)
+		{
+			int i;
+			for (i=0; i<needle_len; ++i)
+				if (p[i] != needle[i])
+					goto next;
+																	   
+			return (char *)p;
+																	   
+		next:
+			++p;
+		}
+
+		return NULL;
+	}
+} 
+
+
 /**
  * Replaces the search string inside original with the replace string.
  * This should be safe with overlapping elements of text, though I haven't
@@ -953,6 +995,30 @@ g_mime_charset_strndup (const char     * to_charset,
 	return retval;
 }
 
+static char*
+pan_clean_utf8 (const char * in,
+                int          in_len)
+{
+	GString * str = g_string_new (NULL);
+	const char *end;
+	char * retval;
+
+	while (g_utf8_validate (in, in_len, &end) == FALSE) {
+		if (end > in)
+			g_string_append_len (str, in, end-in);
+		g_string_append_c (str, '?');
+		if (in_len != -1) 
+			in_len -= (1+end-in);
+		in = end+1;
+	}
+
+	g_string_append_len (str, in, in_len);
+	retval = str->str;
+	g_string_free (str, FALSE);
+
+	return retval;
+}
+
 char*
 pan_header_to_utf8 (const char   * header,
                     int            header_len,
@@ -979,11 +1045,25 @@ pan_header_to_utf8 (const char   * header,
 		 !g_utf8_validate (header, header_len, NULL)) {
 		if (is_nonempty_string(charset)) {
 			retval = g_mime_charset_strndup ("UTF-8", charset, header, header_len);
+			if (!g_utf8_validate (retval, -1, NULL)) {
+				g_free (retval);
+				retval = NULL;
+			}
 		}
 		if (retval == NULL) {
 			const char * local_charset = get_charset_from_locale ();
 			retval = g_mime_charset_strndup ("UTF-8", local_charset, header, header_len);
+
+			if (!g_utf8_validate (retval, -1, NULL)) {
+				g_free (retval);
+				retval = NULL;
+			}
 		}
+
+		if (retval == NULL)
+			retval = pan_clean_utf8 (header, header_len);
+		else
+			replace_gstr (&retval, pan_clean_utf8 (retval, -1));
 	}
 
 	/* fallback */
