@@ -155,6 +155,12 @@ const char*  get_next_token_run (const char   * pch,
                                  const char  ** setme_start,
                                  int          * setme_len);
 
+char* pan_strnstr_len (const char * haystack,
+                       int          haystack_len,
+                       const char * needle,
+                       int          needle_len);
+
+
 char* pan_str_escape   (const char * encode_for_xml);
 
 char* pan_str_unescape (const char * decode_xml);
