@@ -19,18 +19,16 @@
 
 #include <config.h>
 
-#include <glib.h>
+#include <glib/gmem.h>
+#include <glib/gtypes.h>
+#include <glib/gthread.h>
+#include <glib/gmessages.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/pan-object.h>
 
 static glong obj_count = 0;
 
-/* #define PAN_OBJECT_BOOKKEEPING */
-#ifdef PAN_OBJECT_BOOKKEEPING
-static GPtrArray * live = NULL;
-#endif
-
 static GStaticMutex ref_lock = G_STATIC_MUTEX_INIT;
 
 void
@@ -82,11 +80,6 @@ pan_object_constructor (PanObject* o, PanObjectDestructor destructor)
 	g_return_if_fail (o!=NULL);
 
 	++obj_count;
-#ifdef PAN_OBJECT_BOOKKEEPING
-	if (live == NULL)
-		live = g_ptr_array_new ();
-	g_ptr_array_add (live, o);
-#endif
 	debug2 (DEBUG_PAN_OBJECT, "pan_object_constructor: %p (%ld PanObjects alive)", o, obj_count);
 
 	o->ref_count = (gint16)1;
@@ -95,30 +88,12 @@ pan_object_constructor (PanObject* o, PanObjectDestructor destructor)
 void
 pan_object_destructor (PanObject *o)
 {
-#ifdef PAN_OBJECT_BOOKKEEPING
-	if (live != NULL)
-		g_ptr_array_remove (live, o);
-#endif
 	--obj_count;
 	debug2 (DEBUG_PAN_OBJECT, "pan_object_destructor: %p (%ld PanObjects alive)", o, obj_count);
 
 	g_return_if_fail (o!=NULL);
 }
 
-void
-pan_object_bookkeeping (void)
-{
-#ifdef PAN_OBJECT_BOOKKEEPING
-	if (live != NULL)
-	{
-		guint i;
-		for (i=0; i!=live->len; ++i)
-			g_message ("PanObject still alive: %p", g_ptr_array_index(live,i));
-		g_ptr_array_free (live, TRUE);
-	}
-#endif
-}
-
 gboolean
 pan_object_is_valid (const PanObject * o)
 {
