@@ -22,7 +22,7 @@
 
 #define PAN_OBJECT(a) ((PanObject*)a)
 
-#include <glib.h>
+#include <glib/gtypes.h>
 
 typedef struct _PanObject PanObject;
 
@@ -78,6 +78,4 @@ void      pan_object_ref              (PanObject*);
 
 void      pan_object_unref            (PanObject*);
 
-void      pan_object_bookkeeping      (void);
-
 #endif
