@@ -35,12 +35,12 @@
 #include <pan/base/serverlist.h>
 #include <pan/base/util-file.h>
 
+#define SERVER_NAME_KEY "/Pan/Servers/Names"
 static GSList * serverlist = NULL;
-
 static Server * _active = NULL;
 
-const gchar * PAN_SENT;
-const gchar * PAN_SENDLATER;
+PString PAN_SENT;
+PString PAN_SENDLATER;
 
 /**
 ***  Private Utilities
@@ -59,59 +59,54 @@ server_name_compare (gconstpointer a, gconstpointer b)
 **/
 
 static Server*
-server_fetch (const char *name)
+server_fetch (const PString * name_in)
 {
+	PString name = PSTRING_INIT;
 	Server * server = NULL;
-	gboolean has_section;
-	char * buf;
+	char section_name[PATH_MAX];
 	debug_enter ("server_fetch");
 
-	/* find out if we've got a pan_config section for this server */
-	buf = g_strdup_printf ("/Pan/%s/", name);
-	pan_config_push_prefix (buf);
-	has_section = pan_config_has_section (buf);
-	replace_gstr (&buf, NULL);
+	/* sanity clause */
+	g_return_val_if_fail (pstring_is_set (name_in), NULL);
+	name = pstring_shallow (name_in->str, name_in->len);
+	name = pstring_strstrip_shallow (&name);
+	g_return_val_if_fail (pstring_is_set (&name), NULL);
 
-	if (has_section)
+	/* find the section */
+	g_snprintf (section_name, sizeof(section_name), "/Pan/%*.*s", name.len, name.len, name.str);
+	if (pan_config_has_section (section_name))
 	{
 		server = server_new ();
-		server->address = pan_config_get_string ("Address");
-		if (server->address == NULL)
+		pstring_copy (&server->name, &name);
+
+		/* initialize the server from pan_config */
+		pan_config_push_prefix (section_name);
+		server->address = pstring_shallow (pan_config_get_string ("Address", NULL), -1);
+		server->port = pan_config_get_int ("Port", 119);
+		server->max_connections = pan_config_get_int ("Max_Connections", 2);
+		server->idle_secs_before_timeout = pan_config_get_int ("Idle_Timeout_Secs", 180);
+		server->last_newgroup_list_time = pan_config_get_ulong ("Last_Grouplist_Update", 0ul);
+		server->use_newsrc = pan_config_get_bool ("Use_Newsrc", FALSE);
+		server->newsrc_filename = pstring_shallow (pan_config_get_string ("Newsrc_Filename", NULL), -1);
+		server->need_auth = pan_config_get_bool ("Auth_Required", FALSE);
+		server->username = pstring_shallow (pan_config_get_string ("Username", NULL), -1);
+		server->password = pstring_shallow (pan_config_get_string ("Password", NULL), -1);
+		pan_config_pop_prefix ();
+
+		/* if server is OK, then keep it.   Otherwise unref and return NULL */
+		if (server_is_valid (server) && pstring_is_set (&server->name) && pstring_is_set (&server->address))
+		{
+			/* make sure that server's directory exists */
+			char path[PATH_MAX];
+			g_snprintf (path, sizeof(path), "%s" G_DIR_SEPARATOR_S "%*.*s", get_data_dir(), name.len, name.len, name.str);
+			pan_file_ensure_path_exists (path);
+		}
+		else
 		{
 			pan_object_unref (PAN_OBJECT(server));
 			server = NULL;
 		}
-		else
-		{
-			char * path;
-			char * pch;
-			
-			/* make sure the server's config directory exists */
-		       	path = g_strdup_printf ("%s%c%s", get_data_dir(), G_DIR_SEPARATOR, name);
-			pan_file_ensure_path_exists (path);
-
-			/* load the server from pan_config */
-			server->port = pan_config_get_int ("Port=119");
-			server_set_name (server, name);
-			server->use_newsrc = pan_config_get_bool ("Use_Newsrc=0");
-			server->newsrc_filename = pan_config_get_string ("Newsrc_Filename");
-			server->need_auth = pan_config_get_int ("Auth_Required=0");
-			server->username = pan_config_get_string ("Username");
-			server->password = pan_config_get_string ("Password");
-			server->max_connections = pan_config_get_int ("Max_Connections=2");
-			server->idle_secs_before_timeout = pan_config_get_int ("Idle_Timeout_Secs=180");
-			/* no good way to get a ulong from pan_config? */
-			server->last_newgroup_list_time = 1;
-			pch = pan_config_get_string ("Last_Grouplist_Update");
-			if (is_nonempty_string(pch))
-				server->last_newgroup_list_time = strtoul (pch, NULL, 10);
-
-			/* cleanup */
-			g_free (path);
-			g_free (pch);
-		}
 	}
-	pan_config_pop_prefix ();
 
 	debug_exit ("server_fetch");
 	return server;
@@ -129,24 +124,22 @@ server_list_load (void)
 	debug_enter ("server_list_load");
 
 	/* build all the servers in the list */
-       	p = pan_config_get_string ("/Pan/Servers/Names"); /*commas*/
+       	p = pan_config_get_string (SERVER_NAME_KEY, NULL); /*commas*/
 	if (p != NULL)
 	{
 		const char * walk = p;
-		GString * str = g_string_new (NULL);
+		PString pstr;
 
 		/* create a server object for each server in the list */
-		while (get_next_token_g_str (walk, ',', &walk, str))
-		{
-			Server * server = server_fetch (str->str);
-			if (server != NULL)
-				l = g_slist_insert_sorted (l,
-				                           server,
-				                           server_name_compare);
+		while (get_next_token_pstring (walk, ',', &walk, &pstr)) {
+			if (pstr.len) {
+				Server * server = server_fetch (&pstr);
+				if (server!=NULL && (g_slist_find_custom (l, server, server_name_compare)==NULL))
+					l = g_slist_insert_sorted (l, server, server_name_compare);
+			}
 		}
 
 		/* cleanup */
-		g_string_free (str, TRUE);
 		g_free (p);
 	}
 
@@ -209,33 +202,33 @@ serverlist_set_active_server (Server * server)
 
 	/* import the first time the server is activated */
 	if (server->use_newsrc && !server->newsrc_imported) {
-		newsrc_import (server, server->newsrc_filename, FALSE);
+		newsrc_import (server, &server->newsrc_filename, FALSE);
 		server->newsrc_imported = TRUE;
 	}
 
 	/* activate it */
 	_active = server;
-	if (_active!=NULL && is_nonempty_string(_active->name))
-		pan_config_set_string ("/Pan/State/Server", _active->name);
+	if (_active!=NULL && pstring_is_set(&_active->name))
+		pan_config_set_string ("/Pan/State/Server", _active->name.str);
 	else
 		pan_config_clean_key ("/Pan/State/Server");
 	pan_callback_call (serverlist_get_server_activated_callback(), _active, NULL);
 }
 
 Server*
-serverlist_get_named_server (const char* servername)
+serverlist_get_named_server (const PString* servername)
 {
 	GSList * l;
 	Server * retval = NULL;
 	debug_enter ("server_get_by_name");
 
-	g_return_val_if_fail (is_nonempty_string(servername), NULL);
+	g_return_val_if_fail (pstring_is_set (servername), NULL);
 
 	server_init_servers ();
 
 	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
 		Server * s = SERVER(l->data);
-		if (!pan_strcmp (server_get_name (s), servername))
+		if (pstring_equal (servername, &s->name))
 			retval = s;
 	}
 
@@ -244,15 +237,15 @@ serverlist_get_named_server (const char* servername)
 }
 
 Group*
-serverlist_get_named_folder (const gchar * foldername)
+serverlist_get_named_folder (const PString * foldername)
 {
 	Server * server;
 		 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(foldername), NULL);
+	g_return_val_if_fail (pstring_is_set (foldername), NULL);
 			 
 	/* get the folder server... */
-	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
+	server = serverlist_get_named_server (&INTERNAL_SERVER_NAME);
 	g_return_val_if_fail (server_is_valid (server), NULL);
 					 
 	/* get the folder... */
@@ -269,15 +262,17 @@ get_default_server (void)
 	g_return_val_if_fail (serverlist!=NULL, NULL);
 
 	/* look in the gnome config for a default */
-	name = pan_config_get_string ("/Pan/State/Server");
-	if (is_nonempty_string (name))
-		retval = serverlist_get_named_server (name);
+	name = pan_config_get_string ("/Pan/State/Server", NULL);
+	if (is_nonempty_string (name)) {
+		const PString pname = pstring_shallow (name, -1);
+		retval = serverlist_get_named_server (&pname);
+	}
 	g_free (name);
 
 	/* otherwise just use the first server */
 	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
 		Server * s = SERVER(l->data);
-		if (strcmp (server_get_name (s), INTERNAL_SERVER_NAME))
+		if (!pstring_equal (&s->name, &INTERNAL_SERVER_NAME))
 			retval = s;
 	}
 
@@ -306,17 +301,29 @@ serverlist_init (void)
 }
 
 void
-serverlist_add_server (Server * server)
+serverlist_add_server (Server * server, GError ** err)
 {
-	g_return_if_fail (server_is_valid (server));
+	g_return_if_fail (err==NULL || *err==NULL);
 
 	server_init_servers ();
 
+	if (!server_is_valid (server)) {
+		if (err != NULL)
+			*err = g_error_new (g_quark_from_string("serverlist"), 1, _("Server is corrupt"));
+		return;
+	}
+
+	if (serverlist_get_named_server (&server->name) != NULL) {
+		if (err != NULL)
+			*err = g_error_new (g_quark_from_string("serverlist"), 2, _("A Server with that name already exists."));
+		return;
+	}
+
 	serverlist = g_slist_insert_sorted (serverlist,
 	                                    server,
 	                                    server_name_compare);
 
-	if (pan_strcmp (INTERNAL_SERVER_NAME, server->name))
+	if (!pstring_equal (&server->name, &INTERNAL_SERVER_NAME))
 		serverlist_save ();
 }
 
@@ -334,60 +341,62 @@ serverlist_remove_server (Server * server)
 void
 serverlist_save (void)
 {
-	GString * servers = g_string_new (NULL);
-	GSList *l = NULL;
-	gchar * p;
-	char buf[PATH_MAX];
-
-	/* Out with the old */
-	p = pan_config_get_string ("/Pan/Servers/Names");
-	if (p != NULL) {
-		int str_len = 0;
-		const char * str_begin = NULL;
-		const char * walk = p;
-		while (get_next_token_run (walk, ',', &walk, &str_begin, &str_len)) {
-			char * line = g_newa (char, str_len+1);
-			memcpy (line, str_begin, str_len);
-			line[str_len] = '\0';
-
-			g_snprintf (buf, sizeof(buf), "/Pan/%s", line);
-			pan_config_clean_section (buf);
-		}
-		g_free (p);
-	}
-
-	/* In with the new */
-	for (l=serverlist; l!=NULL; l=l->next)
+	/* out with the old */
 	{
-		Server *s = SERVER(l->data);
-		const char * sname = server_get_name (s);
+		char * p = pan_config_get_string (SERVER_NAME_KEY, NULL);
+		if (p != NULL) {
+			GString * tmp = g_string_new (NULL);
+			PString pstr = PSTRING_INIT;
+			const char * walk = p;
+			while (get_next_token_pstring (walk, ',', &walk, &pstr)) {
+				if (pstr.len) {
+					g_string_assign (tmp, "/Pan/");
+					g_string_append_len (tmp, pstr.str, pstr.len);
+					pan_config_clean_section (tmp->str);
+				}
+			}
+			g_string_free (tmp, TRUE);
+			g_free (p);
+		}
+	}
 
-		/* don't need to load/save this one */
-		if (!pan_strcmp (INTERNAL_SERVER_NAME, sname))
-			continue;
+	/* in with the new */
+	{
+		GSList * l = NULL;
+		GString * servers = g_string_new (NULL);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s", sname);
-		pan_config_push_prefix (buf);
-		g_snprintf (buf, sizeof(buf), "%lu", s->last_newgroup_list_time);
-		pan_config_set_string ("Last_Grouplist_Update", buf);
-		pan_config_set_string ("Address", s->address);
-		pan_config_set_int ("Port", s->port);
-		pan_config_set_bool ("Use_Newsrc", s->use_newsrc);
-		pan_config_set_string ("Newsrc_Filename", s->newsrc_filename);
-		pan_config_set_int ("Auth_Required", s->need_auth);
-		pan_config_set_string ("Username", s->username);
-		pan_config_set_string ("Password", s->password);
-		pan_config_set_int ("Max_Connections", s->max_connections);
-		pan_config_set_int ("Idle_Timeout_Secs", s->idle_secs_before_timeout);
-		pan_config_pop_prefix ();
+		for (l=serverlist; l!=NULL; l=l->next)
+		{
+			Server * s = SERVER(l->data);
 
-		if (servers->len)
-			g_string_append_c (servers, ',');
-		g_string_append (servers, sname);
+			if (server_is_valid (s) && pstring_is_set(&s->name) && !pstring_equal(&INTERNAL_SERVER_NAME, &s->name))
+			{
+				char section_name[PATH_MAX];
+				g_snprintf (section_name, sizeof(section_name), "/Pan/%*.*s", s->name.len, s->name.len, s->name.str);
+
+				pan_config_push_prefix (section_name);
+				pan_config_set_string ("Address", s->address.str);
+				pan_config_set_int ("Port", s->port);
+				pan_config_set_int ("Max_Connections", s->max_connections);
+				pan_config_set_int ("Idle_Timeout_Secs", s->idle_secs_before_timeout);
+				pan_config_set_ulong ("Last_Grouplist_Update", s->last_newgroup_list_time);
+				pan_config_set_bool ("Use_Newsrc", s->use_newsrc);
+				pan_config_set_string ("Newsrc_Filename", s->newsrc_filename.str);
+				pan_config_set_bool ("Auth_Required", s->need_auth);
+				pan_config_set_string ("Username", s->username.str);
+				pan_config_set_string ("Password", s->password.str);
+				pan_config_pop_prefix ();
+
+				if (servers->len)
+					g_string_append_c (servers, ',');
+				g_string_append_len (servers, s->name.str, s->name.len);
+			}
+		}
+
+		pan_config_set_string (SERVER_NAME_KEY, servers->str);
+		g_string_free (servers, TRUE);
 	}
 
-	pan_config_set_string ("/Pan/Servers/Names", servers->str);
-	g_string_free (servers, TRUE);
 	pan_config_sync ();
 }
 
