@@ -106,7 +106,7 @@ status_item_new (StatusItemDescribeFunc describe)
 }
 
 StatusItem*
-status_item_new_with_description (const gchar * description)
+status_item_new_with_description (const char * description)
 {
 	StatusItem *item = g_new (StatusItem, 1);
         debug1 (DEBUG_PAN_OBJECT, "status_item_new: %p", item);
