@@ -97,7 +97,7 @@ pan_file_normalize (const char * filename, const char * fallback_absolute_path)
 	g_return_val_if_fail (is_nonempty_string(filename), g_strdup(filename));
 
 	/* take care of paths beginning with ~ */
-	pch = filename[0]=='~' && filename[1]=='/'
+	pch = filename[0]=='~' && filename[1]==G_DIR_SEPARATOR
 		? g_build_filename (pan_get_home_dir(), filename+2, NULL)
 		: g_strdup (filename);
 
