@@ -56,7 +56,7 @@ dialog_about (GtkWidget * window)
 
 	pan_about_add_author (about, "Charles Kerr", "charles@rebelbase.com",
 	                      _("Programmer"));
-	pan_about_add_author (about, "Christophe Lambin", "clambin@easynet.be",
+	pan_about_add_author (about, "Christophe Lambin", "chris@rebelbase.com",
 	                      _("Programmer"));
 	pan_about_add_author (about, "Matt Eagleson", "e.Messiah@superpimp.org",
 	                      _("Project Originator, Programmer"));
@@ -101,6 +101,7 @@ get_credits_translators (void)
 		"\n\tpt - Duarte Loreto"
 		"\n\tpt_BR - Evandro Fernandes Giovanini"
 		"\n\tsk - Marcel Telka"
+		"\n\tsr - Aleksandar Urosevic (Prevod.org)"
 		"\n\tsv - Christian Rose"
 		"\n\ttr - Arman Aksoy (Armish)";
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
