@@ -28,7 +28,15 @@
 
 #include <ctype.h>
 #include <unistd.h>
+
+#if defined(HAVE_GNET_1)
 #include <gnet/gnet.h>
+#define gnet_inetaddr_get_host_name gnet_inetaddr_gethostname
+#elif defined(HAVE_GNET_2)
+#include <gnet.h>
+#else
+#error Must have GNet 1 or GNet 2
+#endif
 
 #include <pan/base/debug.h>
 #include <pan/base/pan-config.h>
@@ -83,6 +91,8 @@ typedef struct
 }
 Druid;
 
+static GtkTooltips * ttips = NULL;
+
 /***
 ****
 ****  HOSTNAME
@@ -97,7 +107,7 @@ get_fqdn (void)
 	char * retval = NULL;
 	debug_enter ("get_fqdn");
 
-	retval = gnet_inetaddr_gethostname ();
+	retval = gnet_inetaddr_get_host_name ();
 
 	debug_exit ("get_fqdn");
 	return retval;
@@ -110,7 +120,7 @@ get_default_author_address (void)
 	debug_enter ("get_default_author_address");
 	
 	if (!email) {
-		email = pan_config_get_string ("/Pan/User/Email");
+		email = pan_config_get_string ("/Pan/User/Email", NULL);
 		if (email != NULL)
 			g_strstrip (email);
 		if (!is_nonempty_string (email))
@@ -163,15 +173,15 @@ druid_finish (Druid * druid)
 
 	if (profile_name && server_address) {
 		server = server_new ();
-                server->name = g_strdup (profile_name);
-		server->address = g_strdup (server_address);
+                pstring_set (&server->name, profile_name, -1);
+		pstring_set (&server->address, server_address, -1);
                 server->port = server_port;
                 server->need_auth = auth_toggle;
-                server->username = auth_toggle ? g_strdup (server_username) : NULL;
-		server->password = auth_toggle ? g_strdup (server_password) : NULL;
+                pstring_set (&server->username, auth_toggle?server_username:NULL, -1);
+		pstring_set (&server->password, auth_toggle?server_password:NULL, -1);
 		server->max_connections = 4;
 		server->idle_secs_before_timeout = 180;
-		serverlist_add_server (server);
+		serverlist_add_server (server, NULL);
 		pan_config_set_string ("/Pan/State/Server", profile_name);
 	}
 
@@ -182,7 +192,10 @@ druid_finish (Druid * druid)
 
 	/* add a new identity */
 	id = identity_new ();
-	pch = g_strdup_printf ("%s <%s>", full_name, email);
+	if (is_nonempty_string (full_name))
+		pch = g_strdup_printf ("%s", full_name);
+	else
+		pch = g_strdup_printf ("Default");
 	identity_set_name (id, pch);
 	replace_gstr (&pch, NULL);
 	identity_set_author_real (id, full_name);
@@ -257,118 +270,45 @@ druid_user_page (Druid * druid)
 {
 	GtkWidget * t;
 	GtkWidget * w;
-	GtkWidget * l;
 	int row = 0;
-	int items_in_section;
 	char * pch;
-	char buf[1024];
 	debug_enter ("druid_user_page");
 
-	t = gtk_table_new (4, 100, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	t = pan_hig_workarea_create ();
 
-	/* title for profile info section */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("How should you appear when posting articles?"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_title (t, &row, _("How should you appear when posting articles?"));
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Required Information"));
 
-	/* section spacer */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u);
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+		pan_hig_workarea_add_section_spacer (t, row, 2);
 
-	/* title for profile info section */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Required Information"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-	++row;
-
-		items_in_section = 2;
-	
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
-
-		/* real name label */
-		l = gtk_label_new_with_mnemonic (_("Your _Name:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* real name control */
+		/* real name */
 		w = druid->full_name = gtk_entry_new();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
 		pan_gtk_entry_set_text (w, g_get_real_name());
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("Your _Name:"), w, NULL);
 
-		/* email label */
-		l = gtk_label_new_with_mnemonic (_("Email _Address:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* email control */
+		/* email */
 		w = druid->email = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
 		pch = get_default_author_address ();
 		pan_gtk_entry_set_text (w, pch);
 		g_free (pch);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
+		pan_hig_workarea_add_row (t, &row, _("Email _Address:"), w, NULL);
 
-	/* section spacer */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u);
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Optional Information"));
 
-	/* title for profile info section */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Optional Information"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-	++row;
+		pan_hig_workarea_add_section_spacer (t, row, 2);
 
-		items_in_section = 2;
-
-		/* organization label */
-		l = gtk_label_new_with_mnemonic (_("_Organization:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* organization control */
+		/* organization */
 		w = druid->organization = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("_Organization:"), w, NULL);
 
-		/* signature label */
-		l = gtk_label_new_with_mnemonic (_("_Signature File:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* signature file */
+		/* signature */
 		w = druid->signature_entry = pan_file_entry_new (_("Select Signature File"));
 		pch = g_build_filename (pan_get_home_dir(), ".signature", NULL);
-		if (pan_file_exists (pch))
-			pan_file_entry_set (w, pch);
-		else
-			pan_file_entry_set (w, "");
+		pan_file_entry_set (w, pan_file_exists(pch) ? pch :  "");
 		g_free (pch);
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), pan_file_entry_gtk_entry(w));
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("_Signature File:"), w, pan_file_entry_gtk_entry(w));
 
 	debug_exit ("druid_user_page");
 	return t;
@@ -453,61 +393,27 @@ druid_smtp_page (Druid *druid)
 {
 	GtkWidget * t;
 	GtkWidget * w;
-	GtkWidget * l;
 	int row = 0;
-	int items_in_section;
 	char * pch;
-	char buf[1024];
 	debug_enter ("druid_smtp_page");
 
-	t = gtk_table_new (4, 100, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	t = pan_hig_workarea_create ();
 
-	/* title for profile info section */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("What mail server should Pan use?"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_title (t, &row, _("What mail server should Pan use?"));
 
-		items_in_section = 2;
-	
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
+		pan_hig_workarea_add_section_spacer (t, row, 2);
 
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
-
-		/* smtp server address label */
-		l = gtk_label_new_with_mnemonic (_("_Address:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* smtp server address control */
+		/* smtp server address */
 		w = druid->smtp_server = gtk_entry_new();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
 		pch = get_fqdn ();
 		pan_gtk_entry_set_text (w, pch);
 		g_free (pch);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("_Address:"), w, NULL);
 
-		/* smtp server port label */
-		l = gtk_label_new_with_mnemonic (_("Por_t:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* smtp server port control */
+		/* smtp server port */
 		w = druid->smtp_port = gtk_entry_new();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
 		pan_gtk_entry_set_text (w, "25");
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("Por_t:"), w, NULL);
 
 	debug_exit ("druid_smtp_page");
 	return t;
@@ -518,128 +424,52 @@ druid_nntp_page (Druid *druid)
 {
 	GtkWidget * t;
 	GtkWidget * w;
-	GtkWidget * l;
 	int row = 0;
-	int items_in_section;
 	char * pch;
-	char buf[1024];
 	debug_enter ("druid_nntp_page");
 
-	t = gtk_table_new (4, 100, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	t = pan_hig_workarea_create ();
 
-	/* title for profile info section */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("What news server should Pan use?"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_title (t, &row, _("What news server should Pan use?"));
 
-		items_in_section = 3 + 2;
-	
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
+		pan_hig_workarea_add_section_spacer (t, row, 3);
 
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
-
-		/* news server address label */
-		l = gtk_label_new_with_mnemonic (_("_Address:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* news server address control */
+		/* news server address */
 		w = druid->server_address = gtk_entry_new();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The news server's actual address, e.g. \"news.mynewsserver.com\""), NULL);
 		pch = get_default_nntp_server ();
-		if (is_nonempty_string (pch))
-			pan_gtk_entry_set_text (w, pch);
-		else
-			pan_gtk_entry_set_text (w, "news");
+		pan_gtk_entry_set_text (w, is_nonempty_string(pch) ? pch : "news");
 		g_free (pch);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("_Address:"), w, NULL);
 
-		/* news server port label */
-		l = gtk_label_new_with_mnemonic (_("Por_t:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* news server port control */
+		/* news server port */
 		w = druid->server_port = gtk_entry_new();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The news server's port.  This is almost always 119."), NULL);
 		pan_gtk_entry_set_text (w, "119");
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("Por_t:"), w, NULL);
 
-		/* news server mnemonic label */
-		l = gtk_label_new_with_mnemonic (_("Short _name for Pan to show:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
-
-		/* news server mnemonic control */
+		/* news server mnemonic */
 		w = druid->profile_name = gtk_entry_new();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
+                gtk_tooltips_set_tip (ttips, w, _("Short name for Pan to show, e.g. \"mynewsserver\""), NULL);
 		pch = get_default_profile_name ();
 		pan_gtk_entry_set_text (w, pch);
 		g_free (pch);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("Short _name for Pan to show:"), w, NULL);
 
-	/* section spacer */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u);
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Login"));
 
-	/* title for profile info section */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Login"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-	++row;
+		pan_hig_workarea_add_section_spacer (t, row, 3);
 
-		items_in_section = 2;
-	
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
-
-		w = druid->auth_toggle = gtk_check_button_new_with_mnemonic (_("Server requires authori_zation"));
+		w = druid->auth_toggle = pan_hig_workarea_add_wide_checkbutton (t, &row, _("Server requires authori_zation"), FALSE);
 		gtk_signal_connect (GTK_OBJECT(w), "toggled", GTK_SIGNAL_FUNC(password_needed_cb), druid);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
-		++row;
-
-		l = druid->server_username_label = gtk_label_new_with_mnemonic (_("_Username:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
 
 		w = druid->server_username = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = druid->server_password_label = gtk_label_new_with_mnemonic (_("_Password:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		druid->server_username_label = pan_hig_workarea_add_row (t, &row, _("_Username:"), w, NULL);
 
 		w = druid->server_password = gtk_entry_new ();
 		gtk_entry_set_visibility (GTK_ENTRY(w), FALSE);
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		druid->server_password_label = pan_hig_workarea_add_row (t, &row, _("_Password:"), w, NULL);
 
 	password_set (druid, FALSE);
 
@@ -720,6 +550,8 @@ dialog_newuser (void)
 	Druid * druid;
 	debug_enter ("dialog_newuser");
 
+	ttips = GTK_TOOLTIPS (gtk_tooltips_new ());
+
 	/* create the dialog */
 	druid = g_new (Druid, 1);
 	dialog = gtk_dialog_new_with_buttons (_("Pan Setup Wizard"),
