@@ -95,7 +95,7 @@ pan_about_new (GtkWindow     * window,
 	dialog = gtk_dialog_new_with_buttons (pch,
 	                                      window,
 	                                      GTK_DIALOG_DESTROY_WITH_PARENT,
-	                                      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+	                                      GTK_STOCK_OK, GTK_RESPONSE_OK,
 	                                      NULL);
 	g_free (pch);
 	                                      
