@@ -221,7 +221,6 @@ get_parent (FilterEditDialog * d, Filter * f)
 	g_return_val_if_fail (f!=NULL, NULL);
 
 	/* find the selected filter */
-        pan_lock ();
 	clist = GTK_CLIST(d->view_clist);
 	for (i=gtk_clist_find_row_from_data(clist, f)-1; retval==NULL && i>=0; --i) {
 		Filter * test = FILTER(gtk_clist_get_row_data (clist, i));
@@ -229,7 +228,6 @@ get_parent (FilterEditDialog * d, Filter * f)
 		    filter_aggregate_has_child(FILTER_AGGREGATE(test), f))
 			retval = test;
 	}
-	pan_unlock ();
 
 	return retval;
 }
@@ -241,12 +239,10 @@ get_selected_index (FilterEditDialog * d)
 	GtkCList * clist;
 
 	/* find the selected filter */
-        pan_lock ();
 	clist = GTK_CLIST(d->view_clist);
 	sel = -1;
         if (clist->selection != NULL)
                 sel = GPOINTER_TO_INT(clist->selection->data);
-	pan_unlock ();
 
 	return sel;
 }
@@ -254,14 +250,7 @@ get_selected_index (FilterEditDialog * d)
 static Filter*
 get_filter_at (FilterEditDialog * d, int index)
 {
-	Filter * retval = NULL;
-
-	/* get the filter at the specified index */
-        pan_lock ();
-	retval = FILTER(gtk_clist_get_row_data (GTK_CLIST(d->view_clist), index));
-	pan_unlock ();
-
-	return retval;
+	return FILTER(gtk_clist_get_row_data (GTK_CLIST(d->view_clist), index));
 }
 	
 static Filter*
@@ -298,7 +287,6 @@ update_clist_move_sel (FilterEditDialog * d, int move_sel)
 	a = get_filter_view (f);
 
 	/* update the UI */
-        pan_lock ();
 	clist = GTK_CLIST(d->view_clist);
 	gtk_clist_freeze (clist);
 	sel = -1;
@@ -315,7 +303,6 @@ update_clist_move_sel (FilterEditDialog * d, int move_sel)
 	}
 	gtk_clist_select_row (clist, sel+move_sel, 0);
 	gtk_clist_thaw (clist);
-	pan_unlock ();
 
 	g_ptr_array_free (a, TRUE);
 }
