@@ -1,14 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Pan - A Newsreader for X
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,19 +15,17 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtkwindow.h>
 #include <pan/filters/filter-top.h>
 
-GtkWidget*          filter_edit_dialog_new          (const FilterTop  * filter);
+GtkWidget*          filter_edit_dialog_new          (GtkWindow        * parent,
+                                                     const FilterTop  * filter);
 
+/**
+ * GTK_RESPONSE_CANCEL
+ * GTK_RESPONSE_APPLY
+ * GTK_RESPONSE_OK
+ */
 const FilterTop*    filter_edit_dialog_get_filter   (GtkWidget        * w);
-
-enum {
-	FILTER_EDIT_DIALOG_CLICKED_DONE = 0,
-	FILTER_EDIT_DIALOG_CLICKED_APPLY = 1,
-	FILTER_EDIT_DIALOG_CLICKED_CANCEL = 2
-};
-
