@@ -32,10 +32,6 @@
 
 static GPtrArray * _filters = NULL;
 
-const char * bozo_filter_name = NULL;
-
-const char * spam_filter_name = NULL;
-
 /************
 *************  PRIVATE
 ************/
@@ -46,70 +42,16 @@ get_filter_filename (void)
 	return g_build_filename (get_data_dir(), "filters.xml", NULL);
 }
 
-static gboolean
-filter_manager_has_named_filter (const char * name)
-{
-	guint i;
-
-	for (i=0; i!=_filters->len; ++i)
-		if (!pan_strcmp (FILTER_TOP(g_ptr_array_index(_filters,i))->name, name))
-			return TRUE;
-
-	return FALSE;
-}
-
-static void
-add_any_needed_canned_filters (GPtrArray * filters)
-{
-	if (!filter_manager_has_named_filter (bozo_filter_name))
-	{
-		Filter * f;
-		Filter * bozo = filter_top_new ();
-		filter_top_set_visible (FILTER_TOP(bozo), FALSE);
-		filter_top_set_name (FILTER_TOP(bozo), bozo_filter_name);
-		filter_aggregate_set_type (FILTER_AGGREGATE(bozo), AGGREGATE_TYPE_OR);
-		f = filter_phrase_new ();
-		filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_CONTAINS, PHRASE_KEY_AUTHOR, "Dave Rhodes");
-		filter_aggregate_add (FILTER_AGGREGATE(bozo), &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-		g_ptr_array_add (filters, bozo);
-
-
-	}
-
-	if (!filter_manager_has_named_filter (spam_filter_name))
-	{
-		Filter * f;
-		Filter * spam = filter_top_new ();
-		filter_top_set_visible (FILTER_TOP(spam), FALSE);
-		filter_top_set_name (FILTER_TOP(spam), spam_filter_name);
-		filter_aggregate_set_type (FILTER_AGGREGATE(spam), AGGREGATE_TYPE_OR);
-		f = filter_phrase_new ();
-		filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_CONTAINS, PHRASE_KEY_SUBJECT, "MAKE MONEY FAST");
-		filter_aggregate_add (FILTER_AGGREGATE(spam), &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-		g_ptr_array_add (filters, spam);
-
-
-	}
-}
-
 void
 filter_manager_init (void)
 {
-
 	char * fname;
 
-	bozo_filter_name = _("Bozos");
-	spam_filter_name = _("Spam");
-
 	/* populate _filters from the array */
 	fname = get_filter_filename ();
 	_filters = g_ptr_array_new ();
 	filter_xml_read (fname, _filters);
 	g_free (fname);
-
-	add_any_needed_canned_filters (_filters);
 }
 
 /************
