@@ -63,7 +63,7 @@ filter_mine_test_articles (Filter            * filter,
 	{
 		const Article * a  = articles[i];
 		Identity      * id = identity_manager_get_identity_by_author
-					(a->author_real, a->author_addr);
+					(a->author_real.str, a->author_addr.str);
 
 		passfail[i] = id != NULL;
 
