@@ -22,6 +22,8 @@
 #include <ctype.h>
 #include <string.h>
 
+#include <pcre.h>
+
 #include <glib.h>
 
 #include <pan/base/debug.h>
@@ -78,6 +80,131 @@ static char _metacharacters[UCHAR_MAX] = {
 *************  PROTECTED
 ************/
 
+/****
+*****  PcreInfo loosely based on Anjuta's search-replace.c
+****/
+
+struct _PcreInfo
+{
+        int ovector_bufsize;
+        int * ovector;
+        pcre * re;
+        pcre_extra * extra;
+};
+
+static void
+pcre_info_free (PcreInfo *re)
+{
+	if (re)
+	{
+		if (re->re)
+			pcre_free (re->re);
+		if (re->extra)
+			pcre_free (re->extra);
+		if (re->ovector)
+			g_free (re->ovector);
+		g_free (re);
+	}
+}
+
+static PcreInfo *
+pcre_info_new (const char * pattern, gboolean case_sensitive, char ** ret_err)
+{
+	PcreInfo *re;
+	int options;
+	const char *err;
+	int err_offset;
+	int status;
+	int capturecount;
+	static int default_options = 0;
+	static gboolean checked_pcre_for_utf8_support = FALSE;
+
+	g_return_val_if_fail (pattern!=NULL, NULL);
+
+	re = g_new0 (PcreInfo, 1);
+
+	if (!checked_pcre_for_utf8_support)
+	{
+		int i = 0;
+		pcre_config (PCRE_CONFIG_UTF8, &i);
+		if (i)
+			default_options |= PCRE_UTF8;
+		else
+			log_add (LOG_ERROR, _("Your system's copy of libpcre doesn't support UTF-8.  UTF-8 regular expressions may fail."));
+
+		checked_pcre_for_utf8_support = TRUE;
+	}
+
+	options = default_options;
+	if (!case_sensitive)
+		options |= PCRE_CASELESS;
+
+	re->re = pcre_compile (pattern, options, &err, &err_offset, NULL);
+	if (NULL == re->re) {
+		char * pch = g_strdup_printf (_("Can't use regular expression \"%s\": %s at position %d"), pattern, err, err_offset);
+		if (ret_err != NULL)
+			*ret_err = g_strdup (pch);
+		log_add (LOG_ERROR, pch);
+		pcre_info_free (re);
+		g_free (pch);
+		return NULL;
+	}
+
+	re->extra = pcre_study (re->re, 0, &err);
+	if (err != NULL) {
+		char * pch = g_strdup_printf (_("Can't use regular expression \"%s\": %s"), pattern, err);
+		if (ret_err != NULL)
+			*ret_err = g_strdup (pch);
+		log_add (LOG_ERROR, pch);
+		pcre_info_free (re);
+		g_free (pch);
+		return NULL;
+	}
+
+	capturecount = 0;
+	status = pcre_fullinfo (re->re, re->extra, PCRE_INFO_CAPTURECOUNT, &capturecount);
+        re->ovector_bufsize = 3 * (capturecount + 1); /* +1 for safety; * 3 for pcre_exec's tuples */
+	re->ovector = g_new (int, re->ovector_bufsize);
+	if (ret_err != NULL)
+		*ret_err = NULL;
+
+	return re;
+}
+
+static int
+my_regexec (FilterPhrase * filter, const char * text_str, int text_len)
+{
+	int retval;
+
+	if (filter->pcre_state == REGEX_NEED_COMPILE)
+	{
+		filter->pcre_info = pcre_info_new (filter->private_key, filter->case_sensitive, NULL);
+
+		if (filter->pcre_info != NULL)
+			filter->pcre_state = REGEX_COMPILED;
+		else
+			filter->pcre_state = REGEX_ERR;
+	}
+
+	if (filter->pcre_state != REGEX_COMPILED)
+		retval = PCRE_ERROR_NOMATCH;
+	else
+		retval = pcre_exec (filter->pcre_info->re,
+		                    filter->pcre_info->extra,
+		                    text_str,
+		                    text_len,
+		                    0,
+		                    PCRE_NOTEMPTY,
+		                    filter->pcre_info->ovector,
+		                    filter->pcre_info->ovector_bufsize);
+
+	return retval;
+}
+
+/****
+*****
+****/
+
 static Filter *
 filter_phrase_dup (const Filter * f_old)
 {
@@ -129,54 +256,6 @@ filter_phrase_to_string (const Filter * filter)
 	return g_strdup_printf ("%s %s \"%s\"", key_str, match_str, phrase->public_key);
 }
 
-static gboolean
-my_regcomp (regex_t * preg, const char * key, gboolean case_sensitive)
-{
-	int errcode;
-	gboolean retval;
-
-	int flags = REG_EXTENDED;
-	if (!case_sensitive)
-		flags |= REG_ICASE;
-
-	errcode = regcomp (preg, key, flags);
-
-	if (!errcode)
-		retval = TRUE;
-	else
-	{
-		char buf[2048];
-		regerror (errcode, preg, buf, sizeof(buf));
-		log_add_va (LOG_ERROR, _("Can't use regular expression \"%s\": %s"), key, buf);
-
-		retval = FALSE;
-	}
-
-	return retval;
-}
-
-
-static int
-my_regexec (FilterPhrase * filter, const char * text_str)
-{
-	int retval;
-
-	if (filter->regex_state == REGEX_NEED_COMPILE)
-	{
-		if (my_regcomp (&filter->key_regex, filter->private_key, filter->case_sensitive))
-			filter->regex_state = REGEX_COMPILED;
-		else
-			filter->regex_state = REGEX_ERR;
-	}
-
-	if (filter->regex_state == REGEX_COMPILED)
-		retval = regexec (&filter->key_regex, text_str, 0, NULL, 0);
-	else
-		retval = REG_NOMATCH;
-
-	return retval;
-}
-
 /**
  * Boyer-Moore-Horspool-Sunday search algorithm.
  * Returns position of match, or -1 if no match.
@@ -258,6 +337,63 @@ bmhs_search (const unsigned char * text, int text_len,
 	return -1;
 }
 
+gboolean
+filter_phrase_does_match   (FilterPhrase    * phrase,
+                            const char      * text,
+                            int               text_len)
+{
+	gboolean retval;
+
+	if (!is_nonempty_string (text))
+	{
+		retval = FALSE;
+	}
+	else
+	{
+		const char * pat = phrase->private_key;
+		const int pat_len = phrase->private_key_len;
+
+		switch (phrase->match_type)
+		{
+			case PHRASE_MATCH_REGEX:
+				retval = my_regexec (phrase, text, text_len)>0;
+				break;
+
+			case PHRASE_MATCH_ENDS_WITH:
+				if (text_len < pat_len) {
+					retval = FALSE;
+					break;
+				}
+				text += text_len - pat_len;
+				text_len -= pat_len;
+				/* fall through to "is" */
+
+			case PHRASE_MATCH_IS:
+				if (phrase->case_sensitive)
+					retval = !pan_strcmp (pat, text);
+				else
+					retval = !g_ascii_strcasecmp (pat, text);
+				break;
+
+			case PHRASE_MATCH_STARTS_WITH:
+				if (phrase->case_sensitive)
+					retval = !strncmp (text, pat, pat_len);
+				else
+					retval = !g_ascii_strncasecmp (text, pat, pat_len);
+				break;
+
+			case PHRASE_MATCH_CONTAINS:
+				if (phrase->case_sensitive)
+					retval = bmhs_search ((const guchar*)text, text_len, phrase->private_key, phrase->private_key_len, phrase->bmhs_skip) != -1;
+				else
+					retval = bmhs_isearch ((const guchar*)text, text_len, phrase->private_key, phrase->private_key_len, phrase->bmhs_skip) != -1;
+				break;
+		}
+	}
+
+	return retval;
+}
+
 static void
 filter_phrase_test_articles (Filter          * filter,
                              const Article  ** articles,
@@ -310,52 +446,7 @@ filter_phrase_test_articles (Filter          * filter,
 				break;
 		}
 
-		if (!is_nonempty_string (text))
-		{
-			does_match[i] = FALSE;
-		}
-		else
-		{
-			const char * pat = phrase->private_key;
-			const int pat_len = phrase->private_key_len;
-
-			switch (phrase->match_type)
-			{
-				case PHRASE_MATCH_REGEX:
-					does_match[i] = !my_regexec (phrase, text);
-					break;
-
-				case PHRASE_MATCH_ENDS_WITH:
-					if (text_len < pat_len) {
-						does_match[i] = FALSE;
-						break;
-					}
-					text += text_len - pat_len;
-					text_len -= pat_len;
-					/* fall through to "is" */
-
-				case PHRASE_MATCH_IS:
-					if (phrase->case_sensitive)
-						does_match[i] = !pan_strcmp (pat, text);
-					else
-						does_match[i] = !g_ascii_strcasecmp (pat, text);
-					break;
-
-				case PHRASE_MATCH_STARTS_WITH:
-					if (phrase->case_sensitive)
-						does_match[i] = !strncmp (text, pat, pat_len);
-					else
-						does_match[i] = !g_ascii_strncasecmp (text, pat, pat_len);
-					break;
-
-				case PHRASE_MATCH_CONTAINS:
-					if (phrase->case_sensitive)
-						does_match[i] = bmhs_search ((const guchar*)text, text_len, phrase->private_key, phrase->private_key_len, phrase->bmhs_skip) != -1;
-					else
-						does_match[i] = bmhs_isearch ((const guchar*)text, text_len, phrase->private_key, phrase->private_key_len, phrase->bmhs_skip) != -1;
-					break;
-			}
-		}
+		does_match[i] = filter_phrase_does_match (phrase, text, text_len);
 	}
 }
 
@@ -385,7 +476,7 @@ filter_phrase_constructor (FilterPhrase * f)
 	f->private_key = NULL;
 	f->public_key = NULL;
 	f->case_sensitive = FALSE;
-	f->regex_state = REGEX_NEED_COMPILE;
+	f->pcre_state = REGEX_NEED_COMPILE;
 
 	debug_exit ("filter_phase_constructor");
 }
@@ -467,22 +558,14 @@ filter_phrase_new (void)
 }
 
 char*
-filter_phrase_validate_regex   (const char * regex_str)
+filter_phrase_validate_pattern   (const char * pattern)
 {
-	char* retval = NULL;
-	const int flags = REG_EXTENDED;
-	int errcode;
-	regex_t reg;
-	
-	errcode = regcomp (&reg, regex_str, flags);
+	char * retval = NULL;
+	PcreInfo * pcre_info;
 
-	if (!errcode)
-		regfree (&reg);
-	else {
-		char buf[2048];
-		regerror (errcode, &reg, buf, sizeof(buf));
-		retval = g_strdup (buf);
-	}
+	pcre_info = pcre_info_new (pattern, FALSE, &retval);
+	if (pcre_info != NULL)
+		pcre_info_free (pcre_info);
 
 	return retval;
 }
@@ -502,9 +585,10 @@ filter_phrase_set (FilterPhrase        * filter,
 	/* free the previous phrase */
 	replace_gstr (&filter->public_key, NULL);
 	replace_gstr (&filter->private_key, NULL);
-	if (filter->regex_state == REGEX_COMPILED) {
-		regfree (&filter->key_regex);
-		filter->regex_state = REGEX_NEED_COMPILE;
+	if (filter->pcre_state == REGEX_COMPILED) {
+		pcre_info_free (filter->pcre_info);
+		filter->pcre_info = NULL;
+		filter->pcre_state = REGEX_NEED_COMPILE;
 	}
 
 	/* repopulate the filter */
@@ -521,9 +605,10 @@ filter_phrase_set (FilterPhrase        * filter,
 			? g_strdup (key)
 			: regexp_unescape (key);
 		if (!filter->case_sensitive) {
-			char * pch;
-			for (pch=filter->private_key; *pch; ++pch)
-				*pch = tolower (*pch);
+			char * pch = g_utf8_strdown (filter->private_key, -1);
+			if (pch) {
+				replace_gstr (&filter->private_key, pch);
+			}
 		}
 		filter->private_key_len = strlen (filter->private_key);
 
