@@ -27,22 +27,36 @@ extern const gchar * FILTER_SCORE_CLASS_ID;
 
 typedef struct _FilterScore FilterScore;
 
+typedef enum
+{
+	SCORE_WATCHED     = (1<<0),
+        SCORE_HIGH        = (1<<1),
+        SCORE_MEDIUM      = (1<<2),
+        SCORE_ZERO        = (1<<4),
+        SCORE_LOW         = (1<<5),
+        SCORE_IGNORED     = (1<<6),
+	SCORE_ALL         = (SCORE_WATCHED | SCORE_HIGH | SCORE_MEDIUM | SCORE_ZERO | SCORE_LOW | SCORE_IGNORED)
+}
+FilterScoreMode;
+
 struct _FilterScore
 {
 	/* Parent */
 	Filter parent;
 
 	/* Fields */
-	gint16 minimum_score;
+	FilterScoreMode mode;
 };
 
 /***
 ****  PUBLIC
 ***/
 
-Filter*      filter_score_new           (void);
+FilterScoreMode  filter_score_get_score_mode    (int                  score);
 
-void         filter_score_set_score     (FilterScore        * filter,
-                                         gint16               minimum_score);
+Filter*          filter_score_new               (FilterScoreMode      mode);
+
+void             filter_score_set_mode          (FilterScore        * filter,
+                                                 FilterScoreMode      mode);
 
 #endif
