@@ -41,6 +41,7 @@
 
 #include <pan/filters/filter-aggregate.h> 
 #include <pan/filters/filter-binary.h> 
+#include <pan/filters/filter-bytes.h> 
 #include <pan/filters/filter-cached.h> 
 #include <pan/filters/filter-crosspost.h> 
 #include <pan/filters/filter-date.h> 
@@ -48,9 +49,7 @@
 #include <pan/filters/filter-linecount.h> 
 #include <pan/filters/filter-new.h> 
 #include <pan/filters/filter-phrase.h> 
-#include <pan/filters/filter-saved.h> 
 #include <pan/filters/filter-score.h> 
-#include <pan/filters/filter-thread.h> 
 #include <pan/filters/filter-top.h> 
 #include <pan/filters/filter.h> 
 #include <pan/filters/filter-xml.h>
@@ -155,7 +154,16 @@ static void
 add_score_arguments (GString * appendme, const FilterScore * f)
 {
 	add_filter_arguments (appendme, FILTER(f));
-	g_string_append_printf (appendme, " min_score=\"%d\"", (int)f->minimum_score);
+
+	g_string_append (appendme, " mode=\"");
+	if (f->mode & SCORE_WATCHED)    g_string_append (appendme, "watched,");
+	if (f->mode & SCORE_HIGH)       g_string_append (appendme, "high,");
+	if (f->mode & SCORE_MEDIUM)     g_string_append (appendme, "medium,");
+	if (f->mode & SCORE_ZERO)       g_string_append (appendme, "zero,");
+	if (f->mode & SCORE_LOW)        g_string_append (appendme, "low,");
+	if (f->mode & SCORE_IGNORED)    g_string_append (appendme, "ignored,");
+	g_string_truncate (appendme, appendme->len-1);
+	g_string_append_c (appendme, '"');
 }
 static void
 add_crosspost_arguments (GString * appendme, const FilterCrosspost * f)
@@ -181,20 +189,10 @@ add_linecount_arguments (GString * appendme, const FilterLineCount * f)
 	g_string_append_printf (appendme, " min_lines=\"%d\"", f->minimum_line_count);
 }
 static void
-add_thread_arguments (GString * appendme, const FilterThread * f)
+add_bytes_arguments (GString * appendme, const FilterBytes * f)
 {
 	add_filter_arguments (appendme, FILTER(f));
-
-	switch (f->state) {
-		case FILTER_THREAD_WATCHED:
-			g_string_append (appendme, " type=\"watched\"");
-			break;
-		case FILTER_THREAD_IGNORED:
-			g_string_append (appendme, " type=\"ignored\"");
-			break;
-		case FILTER_THREAD_NOTHING:
-			g_string_append (appendme, " type=\"nothing\"");
-	}
+	g_string_append_printf (appendme, " minimum=\"%lu\"", f->minimum_bytes);
 }
 static void
 add_new_arguments (GString * appendme, const FilterNew * f)
@@ -241,37 +239,14 @@ add_phrase_arguments (GString * appendme, const FilterPhrase * f)
 		case PHRASE_KEY_XREF:         str = "xref"; break;
 	}
 	g_string_append_printf (appendme, " key_type=\"%s\"", str);
+	g_string_append_printf (appendme, " case_sensitive=\"%c\"", f->case_sensitive ? 't' : 'f');
 
 	/* key */
 	g_string_append (appendme, " key=\"");
-	pan_g_string_append_escaped (appendme, f->key);
+	pan_g_string_append_escaped (appendme, f->public_key);
 	g_string_append (appendme, "\"");
 }
 static void
-add_saved_arguments (GString * appendme, const FilterSaved * f)
-{
-	g_return_if_fail (appendme!=NULL);
-	g_return_if_fail (f!=NULL);
-
-	/* superclass */
-	add_filter_arguments (appendme, FILTER(f));
-
-	/* saved */
-	g_string_append (appendme, " state=\"");
-	switch (f->state) {
-		case FILTER_SAVED_SAVED:
-			g_string_append (appendme, "saved");
-			break;
-		case FILTER_SAVED_QUEUED:
-			g_string_append (appendme, "queued");
-			break;
-		case FILTER_SAVED_NOT_SAVED:
-			g_string_append (appendme, "not_saved");
-			break;
-	}
-	g_string_append_c (appendme, '"');
-}
-static void
 add_binary_arguments (GString * appendme, const FilterBinary * f)
 {
 	g_return_if_fail (appendme!=NULL);
@@ -300,14 +275,6 @@ add_binary_arguments (GString * appendme, const FilterBinary * f)
 ***
 **/
 
-static void
-write_saved (GString * appendme, const FilterSaved * f, gint depth)
-{
-	newline_depth (appendme, depth);
-	g_string_append (appendme, "<saved");
-	add_saved_arguments (appendme, f);
-	g_string_append (appendme, "/>");
-}
 static void
 write_binary (GString * appendme, const FilterBinary * f, gint depth)
 {
@@ -365,6 +332,14 @@ write_linecount (GString * appendme, const FilterLineCount * f, gint depth)
 	g_string_append (appendme, "/>");
 }
 static void
+write_bytes (GString * appendme, const FilterBytes * f, int depth)
+{
+	newline_depth (appendme, depth);
+	g_string_append (appendme, "<bytes");
+	add_bytes_arguments (appendme, f);
+	g_string_append (appendme, "/>");
+}
+static void
 write_new (GString * appendme, const FilterNew * f, gint depth)
 {
 	newline_depth (appendme, depth);
@@ -381,14 +356,6 @@ write_phrase (GString * appendme, const FilterPhrase * f, gint depth)
 	g_string_append (appendme, "/>");
 }
 static void
-write_thread (GString * appendme, const FilterThread * f, gint depth)
-{
-	newline_depth (appendme, depth);
-	g_string_append (appendme, "<thread");
-	add_thread_arguments (appendme, f);
-	g_string_append (appendme, "/>");
-}
-static void
 write_top (GString * appendme, const FilterTop * f, gint depth)
 {
 	newline_depth (appendme, depth);
@@ -425,9 +392,6 @@ write_filter (GString * appendme, const Filter * f, gint depth)
 	else if (!strcmp (f->class_id, FILTER_AGGREGATE_CLASS_ID))
 		write_aggregate (appendme, FILTER_AGGREGATE(f), depth);
 
-	else if (!strcmp (f->class_id, FILTER_SAVED_CLASS_ID))
-		write_saved (appendme, FILTER_SAVED(f), depth);
-
 	else if (!strcmp (f->class_id, FILTER_BINARY_CLASS_ID))
 		write_binary (appendme, FILTER_BINARY(f), depth);
 
@@ -449,15 +413,15 @@ write_filter (GString * appendme, const Filter * f, gint depth)
 	else if (!strcmp (f->class_id, FILTER_LINE_COUNT_CLASS_ID))
 		write_linecount (appendme, FILTER_LINE_COUNT(f), depth);
 
+	else if (!strcmp (f->class_id, FILTER_BYTES_CLASS_ID))
+		write_bytes (appendme, FILTER_BYTES(f), depth);
+
 	else if (!strcmp (f->class_id, FILTER_NEW_CLASS_ID))
 		write_new (appendme, FILTER_NEW(f), depth);
 
 	else if (!strcmp (f->class_id, FILTER_PHRASE_CLASS_ID))
 		write_phrase (appendme, FILTER_PHRASE(f), depth);
 
-	else if (!strcmp (f->class_id, FILTER_THREAD_CLASS_ID))
-		write_thread (appendme, FILTER_THREAD(f), depth);
-
 	else pan_warn_if_reached ();
 }
 
@@ -519,53 +483,29 @@ filter_xml_write (const char         * filename,
 	g_return_if_fail (filter_qty>=0);
 
 	/* write the file */
-
-	if (filter_qty == 0) {
+	if (filter_qty == 0)
 		unlink (filename);
-	}
 	else {
-		FILE     * fp;
-		gchar    * tmp_path = pan_file_make_temp (&fp);
+		FILE * fp;
+		char * filename_tmp = pan_file_make_temp (&fp);
 		gboolean   ok = fp ? TRUE : FALSE;
 
 		if (ok) {
 			ok = filter_xml_fwrite (fp, filters, filter_qty);
 			fclose (fp);
 		}
+		if (ok && pan_file_swap_datafile (filename_tmp, filename))
+			log_add_va (LOG_INFO, _("Wrote filters to \"%s\""), filename);
 
-		if (ok) {
-			ok = pan_file_rename (tmp_path, filename);
-		}
-
-		if (ok) {
-			log_add_va (LOG_INFO, _("Wrote filters to \"%s\""),
-				filename);
-		}
-	       	else {
-			log_add_va (LOG_ERROR,
-				_("Can't write to \"%s\": %s"),
-				filename, g_strerror(errno));
-		}
-
-		g_free (tmp_path);
+		g_free (filename_tmp);
 	}
 }
 
-
-
-
-
-
-
-
-
-/*****
-******  
-******  
-******    READING
-******  
-******  
-*****/
+/***
+****  
+****    READING
+****  
+***/
 
 static void
 populate_filter (Filter * f, xmlNodePtr node)
@@ -585,30 +525,21 @@ populate_score (FilterScore * f, xmlNodePtr node)
 
 	populate_filter (FILTER(f), node);
 
-	if ((val = xmlGetProp (node, "min_score")) != NULL) {
-		f->minimum_score = atoi (val);
-		xmlFree (val);
-	}
-}
-
-static void
-populate_saved (FilterSaved * f, xmlNodePtr node)
-{
-	xmlChar * val;
-	FilterSavedState state = FILTER_SAVED_SAVED;
-
-	populate_filter (FILTER(f), node);
-
-	if ((val = xmlGetProp (node, "state")) != NULL)
+	if ((val = xmlGetProp (node, "mode")) != NULL)
 	{
-		if (!pan_strcmp(val,"saved")) state = FILTER_SAVED_SAVED;
-		else if (!pan_strcmp(val,"queued")) state = FILTER_SAVED_QUEUED;
-		else if (!pan_strcmp(val,"not_saved")) state = FILTER_SAVED_NOT_SAVED;
+		FilterScoreMode mode = 0;
+
+                if (strstr (val, "watched") != NULL)  mode |= SCORE_WATCHED;
+                if (strstr (val, "high") != NULL)     mode |= SCORE_HIGH;
+                if (strstr (val, "medium") != NULL)   mode |= SCORE_MEDIUM;
+                if (strstr (val, "zero") != NULL)     mode |= SCORE_ZERO;
+                if (strstr (val, "low") != NULL)      mode |= SCORE_LOW;
+                if (strstr (val, "ignored") != NULL)  mode |= SCORE_IGNORED;
+
+		filter_score_set_mode (f, mode);
 
 		xmlFree (val);
 	}
-
-	filter_saved_set_state (f, state);
 }
 
 static void
@@ -657,27 +588,6 @@ populate_new (FilterNew * f, xmlNodePtr node)
 	filter_new_set_state (f, state);
 }
 
-static void
-populate_thread (FilterThread * f, xmlNodePtr node)
-{
-	xmlChar * val;
-	FilterThreadState state = FILTER_THREAD_WATCHED;
-
-	populate_filter (FILTER(f), node);
-
-	if ((val = xmlGetProp (node, "type")) != NULL)
-	{
-		if (!pan_strcmp(val,"watched")) state = FILTER_THREAD_WATCHED;
-		else if (!pan_strcmp(val,"ignored")) state = FILTER_THREAD_IGNORED;
-		else if (!pan_strcmp(val,"nothing")) state = FILTER_THREAD_NOTHING;
-		else pan_warn_if_reached ();
-
-		xmlFree (val);
-	}
-
-	filter_thread_set_state (f, state);
-}
-
 static void
 populate_linecount (FilterLineCount * f, xmlNodePtr node)
 {
@@ -691,6 +601,19 @@ populate_linecount (FilterLineCount * f, xmlNodePtr node)
 	}
 }
 
+static void
+populate_bytes (FilterBytes * f, xmlNodePtr node)
+{
+	xmlChar * val;
+
+	populate_filter (FILTER(f), node);
+
+	if ((val = xmlGetProp (node, "minimum")) != NULL) {
+		f->minimum_bytes = strtoul (val, NULL, 10);
+		xmlFree (val);
+	}
+}
+
 static void
 populate_crosspost (FilterCrosspost * f, xmlNodePtr node)
 {
@@ -726,6 +649,7 @@ static void
 populate_phrase (FilterPhrase * f, xmlNodePtr node)
 {
 	xmlChar * val;
+	gboolean case_sensitive = FALSE;
 	gboolean pre_0_13_3_91 = FALSE;
 	char * key = NULL;
 	PhraseKeyType key_type = PHRASE_KEY_SUBJECT;
@@ -789,8 +713,16 @@ populate_phrase (FilterPhrase * f, xmlNodePtr node)
 			xmlFree (val);
 		}
 	}
+	else
+	{
+		/* check for case-sensitive */
+		if ((val = xmlGetProp (node, "case_sensitive")) != NULL) {
+			case_sensitive = *val == 't';
+			xmlFree (val);
+		}
+	}
 
-	filter_phrase_set (f, match_type, key_type, key);
+	filter_phrase_set (f, match_type, key_type, key, case_sensitive);
 	g_free (key);
 }
 
@@ -855,6 +787,11 @@ create_filter (xmlDoc * doc, xmlNodePtr node)
 		f = filter_line_count_new ();
 		populate_linecount (FILTER_LINE_COUNT(f), node);
 	}
+	else if (!pan_strcmp(tag,"bytes"))
+	{
+		f = filter_bytes_new ();
+		populate_bytes (FILTER_BYTES(f), node);
+	}
 	else if (!pan_strcmp(tag,"crosspost"))
 	{
 		f = filter_crosspost_new ();
@@ -870,11 +807,6 @@ create_filter (xmlDoc * doc, xmlNodePtr node)
 		f = filter_mine_new ();
 		populate_mine (FILTER_MINE(f), node);
 	}
-	else if (!pan_strcmp(tag,"thread"))
-	{
-		f = filter_thread_new ();
-		populate_thread (FILTER_THREAD(f), node);
-	}
 	else if (!pan_strcmp(tag,"read"))
 	{
 		f = filter_new_article_new (FILTER_NEW_READ);
@@ -885,14 +817,9 @@ create_filter (xmlDoc * doc, xmlNodePtr node)
 		f = filter_binary_new ();
 		populate_binary (FILTER_BINARY(f), node);
 	}
-	else if (!pan_strcmp(tag,"saved"))
-	{
-		f = filter_saved_new ();
-		populate_saved (FILTER_SAVED(f), node);
-	}
 	else if (!pan_strcmp(tag,"score"))
 	{
-		f = filter_score_new ();
+		f = filter_score_new (0);
 		populate_score (FILTER_SCORE(f), node);
 	}
 	else if (!pan_strcmp(tag,"cached"))
