@@ -48,7 +48,6 @@
 #include <pan/prefs.h>
 
 #include <sys/types.h>
-#include <regex.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -65,8 +64,8 @@
 typedef struct
 {
 	const char * name;
-	regex_t * regex;
-	int regex_qty;
+	FilterPhrase ** filters;
+	int filter_qty;
 	gboolean section_name_negate;
 	GSList * scores;
 }
@@ -288,12 +287,12 @@ get_section (const char * name, const char * filename, int line_number)
 		const gboolean negate = *name == '~';
 
 		/* make a new section */
-		retval = g_new0 (Section, 1);
+		retval = g_new (Section, 1);
 		retval->name = g_string_chunk_insert_const (_string_chunk, name);
 		retval->section_name_negate = negate;
 		retval->scores = NULL;
-		retval->regex = NULL;
-		retval->regex_qty = 0;
+		retval->filters = NULL;
+		retval->filter_qty = 0;
 
 		/* break the groups apart by whitespace and compile each one as a regular expression */
 		if (negate)
@@ -309,34 +308,44 @@ get_section (const char * name, const char * filename, int line_number)
 		/* count the number of names */
 		march = gname->str;
 		while ((get_next_token_pstring (march, ' ', &march, &token)))
-			++retval->regex_qty;
+			++retval->filter_qty;
 
 		/* compile the names into regex_t's */
 		prep = g_string_new (NULL);
-		if (retval->regex_qty)
+		if (retval->filter_qty)
 		{
 			int real_qty = 0;
-			retval->regex = g_new (regex_t, retval->regex_qty);
+			retval->filters = g_new (FilterPhrase*, retval->filter_qty);
 			march = gname->str;
 
 			while ((get_next_token_pstring (march, ' ', &march, &token)))
 			{
-				int errcode;
+				char * errmsg;
 
 				prep_section_regex (&token, prep);
 
-				errcode = regcomp (retval->regex + real_qty, prep->str, REG_EXTENDED|REG_ICASE);
-				if (!errcode)
-					++real_qty;
-				else {
-					char errbuf[1024];
-					regerror (errcode, retval->regex + real_qty, errbuf, sizeof(errbuf));
+				errmsg = filter_phrase_validate_pattern (prep->str);
+				if (errmsg != NULL)
+				{
 					log_add_va (LOG_ERROR, _("Can't use regular expression \"%s\" from file \"%s\", line %d: %s"),
-					                       prep->str, filename, line_number, errbuf);
+					            prep->str, filename, line_number, errmsg);
+					g_free (errmsg);
+				}
+				else
+				{
+					FilterPhrase * phrase = FILTER_PHRASE (filter_phrase_new ());
+
+					filter_phrase_set (phrase,
+					                   PHRASE_MATCH_REGEX,
+					                   PHRASE_KEY_SUBJECT,
+					                   prep->str,
+					                   TRUE);
+
+					retval->filters[real_qty++] = phrase;
 				}
 			}
 
-			retval->regex_qty = real_qty;
+			retval->filter_qty = real_qty;
 		}
 
 		/* remember this section */
@@ -459,7 +468,7 @@ parse_score_file (ScorefileContext   * context,
 			}
 
 			/* new score */
-			score = g_new0 (Score, 1);
+			score = g_new (Score, 1);
 			score->expired = FALSE;
 			score->name = score_name ? g_string_chunk_insert_const (_string_chunk, score_name) : NULL;
 			score->filename = g_string_chunk_insert_const (_string_chunk, filename);
@@ -808,9 +817,9 @@ unload_scores (void)
 		g_slist_free (section->scores);
 
 		/* free the section regex */
-		for (i=0; i<section->regex_qty; ++i)
-			regfree (&section->regex[i]);
-		g_free (section->regex);
+		for (i=0; i<section->filter_qty; ++i)
+			pan_object_unref (PAN_OBJECT(section->filters[i]));
+		g_free (section->filters);
 
 		/* free the section */
 		g_free (section);
@@ -972,9 +981,10 @@ make_scorefile_entry (const char           * score_name,
 		const char * value = item->value ? item->value : "";
 		if (is_nonempty_string(item->value) && (item->on || slrn_adds_these_even_if_turned_off(item->key)))
 			g_string_append_printf (str, "%c\t%s%s: %s" EOL,
-						(item->on?' ':'%'),
-						(item->negate?"~":""),
-						item->key, value);
+						(item->on ? ' ' : '%'),
+						(item->negate ? "~" : ""),
+						item->key,
+						value);
 	}
 	g_string_append_printf (str, "%%EOS" EOL EOL);
 
@@ -1060,17 +1070,14 @@ does_section_match (const Section * section, const PString * group_name)
 {
 	int i;
 	gboolean match = FALSE;
-	char buf[1024];
 
 	/* sanity clause */
 	g_return_val_if_fail (section!=NULL, FALSE);
 	g_return_val_if_fail (pstring_is_set (group_name), FALSE);
 
-	pan_strncpy_len (buf, sizeof(buf), group_name->str, group_name->len);
-
 	/* does it match? */
-	for (i=0; !match && i<section->regex_qty; ++i) {
-		match = !regexec (&section->regex[i], buf, 0, NULL, 0);
+	for (i=0; !match && i<section->filter_qty; ++i) {
+		match = filter_phrase_does_match (section->filters[i], group_name->str, group_name->len);
 		if (section->section_name_negate)
 			match = !match;
 	}
@@ -1168,7 +1175,7 @@ ensure_articles_scored  (Group          * group,
 			Article ** articles;
 
 			/* make a temporary list of articles not scored yet */
-			articles = g_new0 (Article*, article_in_qty);
+			articles = g_new (Article*, article_in_qty);
 			for (article_qty=i=0; i<article_in_qty; ++i)
 				if (articles_in[i]->score_date < scorefile_time)
 					articles[article_qty++] = articles_in[i];
@@ -1274,7 +1281,7 @@ view_article_func (const Section * section,
 {
 	ScoreViewItem * item;
        
-	item = g_new0 (ScoreViewItem, 1);
+	item = g_new (ScoreViewItem, 1);
 	item->filename = g_strdup (score->filename);
 	item->begin_line = score->begin_line;
 	item->end_line = score->end_line;
@@ -1320,9 +1327,8 @@ score_view_article (const Article * a)
 /**
  * Remove a score from the scorefile.
  *
- * It's easy to imagine how a dumb program could do damage to
- * a finely-tuned scorefile, so instead of removing *anything*
- * we just comment out the relevant lines.
+ * It's easy to imagine how much damage we could do to a finely-tuned
+ * scorefile, so let's comment lines out instead of deleting them.
  */
 void
 score_remove_score  (const char * filename,
@@ -1443,10 +1449,10 @@ score_remove_score  (const char * filename,
 ***/
 
 static void
-add_group_name_to_section (const PString * server_name,
-                           const PString * group_name,
-                           gulong number,
-                           gpointer gstring_gpointer)
+add_group_name_to_section (const PString    * server_name,
+                           const PString    * group_name,
+                           gulong             number,
+                           gpointer           gstring_gpointer)
 {
 	GString * gstr = (GString*) gstring_gpointer;
 
