@@ -26,7 +26,7 @@
 
 void          flagset_init            (void);
 
-gboolean      flagset_has_message     (const char          * message_id);
+gboolean      flagset_has_article     (const Article       * article);
 
 void          flagset_add_articles    (const Article      ** articles,
                                        int                   qty);
