@@ -58,6 +58,7 @@ static GtkItemFactoryEntry group_popup_entries[] =
 {
 	/* h */ {N_("/Get New _Headers"), NULL, group_action_selected_download_new, 0, NULL},
 	/* b */ {N_("/Get New Headers and _Bodies"), NULL, group_action_selected_download_new_and_bodies, 0, NULL},
+	/*   */ {N_("/Get New Headers in Subscribed Groups"), NULL, group_action_subscribed_download_new, 0, NULL},
 	/* o */ {N_("/More Download _Options..."), NULL, group_action_selected_download_dialog, 0, NULL},
 	/* c */ {N_("/Refresh Article _Counts"), NULL, group_action_selected_update_count_info, 0, "<StockItem>", GTK_STOCK_REFRESH},
 	/*   */ {N_("/---"), NULL, NULL, 0, "<Separator>"},
@@ -525,6 +526,7 @@ grouplist_button_press (GtkWidget *widget, GdkEventButton * event)
 			menu_set_sensitive (gif, "/Delete Group's Articles", have_group);
 			menu_set_sensitive (gif, "/Get New Headers", have_group);
 			menu_set_sensitive (gif, "/Get New Headers and Bodies", have_group);
+			menu_set_sensitive (gif, "/Get New Headers in Subscribed Groups", TRUE);
 			menu_set_sensitive (gif, "/More Download Options...", have_group);
 			menu_set_sensitive (gif, "/Refresh Article Counts", have_group);
 			menu_set_sensitive (gif, "/Subscribe", have_group);
