@@ -131,7 +131,7 @@ add_header_nolock (GtkWidget      * table,
 		GtkWidget * hbox = gtk_hbox_new (FALSE, 0);
 		GtkWidget * l = NULL;
 		GtkWidget * l2 = gtk_label_new (_("<More>"));
-		char * full = NULL;
+		GString * lines;
 		char * pch = NULL;
 
 		/* create the truncated label */
@@ -140,12 +140,13 @@ add_header_nolock (GtkWidget      * table,
 		l = gtk_label_new (pch);
 
 		/* add them both to the button */
-		gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
-		gtk_box_pack_end(GTK_BOX(hbox), l2, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX(hbox), l, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX(hbox), l2, FALSE, FALSE, 0);
 
-		/* break the line into easier-to-read pieces
-		   for the popup dialog */
-		pch = full = g_strdup(utf8_str2);
+		/* break the line into easier-to-read
+		 * pieces for the popup dialog */
+		lines = g_string_new (utf8_str2);
+		pch = lines->str;
 		if (delimiter != ' ') {
 			for (; *pch; pch = g_utf8_next_char (pch))
 				if (*pch == delimiter)
@@ -153,9 +154,14 @@ add_header_nolock (GtkWidget      * table,
 		}
 		else for (;;) {
 			const int index = find_breakpoint (pch, delimiter, max_width);
-			if (index == -1) break;
-			pch[index] = '\n';
-			pch += (index + 1);
+			if (index == -1)
+				break;
+			else {
+				const int lines_offset = pch - lines->str;
+				const int lines_index = index + lines_offset;
+				g_string_insert_c (lines, lines_index, '\n');
+				pch += (index + 2); /* 1 for the delimiter, 1 for the linefeed */
+			}
 		}
 
 		/* create a button */
@@ -165,7 +171,8 @@ add_header_nolock (GtkWidget      * table,
 		gtk_widget_show (l);
 		gtk_widget_show (l2);
 		gtk_widget_show (hbox);
-		gtk_object_set_data_full (GTK_OBJECT(w2), "text", full, g_free);
+		g_object_set_data_full (G_OBJECT(w2), "text",
+		                        g_string_free(lines,FALSE), g_free);
 		g_signal_connect (GTK_OBJECT(w2), "clicked",
 		                  G_CALLBACK(showmore_cb), NULL);
 		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w2, utf8_str2, NULL);
