@@ -30,11 +30,7 @@
  * GTK_RESPONSE_CANCEL
  */
 GtkWidget*  identity_edit_dialog_new          (GtkWindow        * window,
-                                               const Identity   * id,
-                                               gboolean           default_news,
-                                               gboolean           default_mail);
+                                               const Identity   * id);
 
-Identity*   identity_edit_dialog_get_identity (GtkWidget        * w,
-                                               gboolean         * default_news,
-                                               gboolean         * default_mail);
+Identity*   identity_edit_dialog_get_identity (GtkWidget        * w);
 
