@@ -1535,7 +1535,10 @@ apply_identity (Compose * compose, const char * new_id_name)
 
 		/* replace the signature */
 		sig_removed = pan_remove_signature (body);
-		if (new_id!=NULL && is_nonempty_string(new_id->signature))
+
+		if (new_id==NULL || !is_nonempty_string(new_id->signature))
+			insert_pos = strlen(body) - 1;
+		else
 		{
 			char * sig = load_signature (new_id->signature);
 
@@ -1568,6 +1571,9 @@ apply_identity (Compose * compose, const char * new_id_name)
 			gtk_text_buffer_get_iter_at_offset (compose->body_buffer, &iter, insert_pos);
 			gtk_text_buffer_move_mark_by_name (compose->body_buffer, "insert", &iter);
 			gtk_text_buffer_move_mark_by_name (compose->body_buffer, "selection_bound", &iter);
+			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW(compose->body_view), 
+			                              gtk_text_buffer_get_mark(compose->body_buffer, "insert"),
+			                              0.0, FALSE, 0.0, 0.0);
 		}
 
 		g_free (body);
