@@ -991,9 +991,13 @@ nntp_download_headers (StatusItem       * status,
 		if (a->subject.len>3 && !strncmp (a->subject.str, "Re:", 3) && a->linecount<100)
 			part = parts = 0;
 
+		/* Subjects containing (0/N) aren't part of an N-part binary;
+		   they're text description articles that accompany the binary. */
+		if (part == 0)
+			parts = 0;
+
 		/* Verify Multipart info */
 		if ((parts>=1) && (part<=parts)) {
-			/* yay, we found one! */
 			a->parts = parts;
 			a->part = part;
 		}
