@@ -52,6 +52,7 @@ picker_clicked_cb (GtkWidget * picker, gpointer user_data)
 	GtkWidget * top;
 	GtkWidget * w;
 	GtkWidget * colorsel;
+	GdkColor  * color;
 
 	/* create the dialog */
        	w = gtk_color_selection_dialog_new (_("Select Color"));
@@ -63,7 +64,8 @@ picker_clicked_cb (GtkWidget * picker, gpointer user_data)
 
 	/* set its state and listen for changes */
        	colorsel = GTK_COLOR_SELECTION_DIALOG(w)->colorsel;
-	gtk_color_selection_set_current_color (GTK_COLOR_SELECTION(colorsel), &picker->style->bg[GTK_STATE_NORMAL]);
+	color = g_object_get_data (G_OBJECT(picker), "pixel");
+	gtk_color_selection_set_current_color (GTK_COLOR_SELECTION(colorsel), color);
 	g_signal_connect (w, "response", G_CALLBACK(color_picker_response_cb), picker);
 	gtk_widget_show_all (w);
 }
