@@ -76,7 +76,8 @@
 #define DEFAULT_VALUE_BODY_PANE_SMOOTH_SCROLLING_SPEED      10
 #define DEFAULT_VALUE_BREAK_THREAD_WHEN_SUBJECT_CHANGES     FALSE
 #define DEFAULT_VALUE_FETCH_NEW_AND_BODIES_ON_STARTUP       FALSE
-#define DEFAULT_VALUE_FETCH_NEW_ON_GROUP_LOAD               TRUE
+#define DEFAULT_VALUE_FETCH_NEW_ON_GROUP_ENTER              TRUE
+#define DEFAULT_VALUE_MARK_READ_ON_GROUP_EXIT               FALSE
 #define DEFAULT_VALUE_FETCH_NEW_FROM_SUBSCRIBED_ON_STARTUP  FALSE
 #define DEFAULT_VALUE_SINGLE_CLICK_SELECTS_GROUPS           FALSE
 #define DEFAULT_VALUE_SINGLE_CLICK_SELECTS_HEADERS          FALSE
@@ -131,7 +132,8 @@ typedef struct
 	GtkWidget * single_click_selects_groups_cbutton;
 	GtkWidget * single_click_selects_headers_cbutton;
 	GtkWidget * remove_failed_tasks_cbutton;
-	GtkWidget * fetch_new_on_group_load_cbutton;
+	GtkWidget * fetch_new_on_group_enter_cbutton;
+	GtkWidget * mark_read_on_group_exit_cbutton;
 	GtkWidget * fetch_new_on_startup_cbutton;
 	GtkWidget * fetch_new_and_bodies_on_startup_cbutton;
 	GtkWidget * break_thread_when_subject_changes_cbutton;
@@ -205,7 +207,8 @@ int mail_server_port                          = DEFAULT_VALUE_SMTP_PORT;
 gboolean expand_all_threads_by_default        = DEFAULT_VALUE_HEADER_PANE_THREADS_EXPANDED ;
 gboolean hide_mpart_child_nodes               = DEFAULT_VALUE_HEADER_PANE_HIDE_MULTIPARTS;
 gboolean pan_mute                             = FALSE;
-gboolean fetch_new_on_group_load              = DEFAULT_VALUE_FETCH_NEW_ON_GROUP_LOAD;
+gboolean fetch_new_on_group_enter             = DEFAULT_VALUE_FETCH_NEW_ON_GROUP_ENTER;
+gboolean mark_read_on_group_exit              = DEFAULT_VALUE_MARK_READ_ON_GROUP_EXIT;
 gboolean fetch_new_on_startup                 = DEFAULT_VALUE_FETCH_NEW_FROM_SUBSCRIBED_ON_STARTUP;
 gboolean fetch_new_and_bodies_on_startup      = DEFAULT_VALUE_FETCH_NEW_AND_BODIES_ON_STARTUP;
 gboolean header_pane_is_threaded              = DEFAULT_VALUE_HEADER_PANE_IS_THREADED;
@@ -656,10 +659,15 @@ prefs_apply (void)
 	***  General
 	**/
 
-	update_entry_and_bool_from_toggle_button (&fetch_new_on_group_load,
-	                                          DEFAULT_VALUE_FETCH_NEW_ON_GROUP_LOAD,
-	                                          KEY_FETCH_NEW_ON_GROUP_LOAD,
-	                                          win->fetch_new_on_group_load_cbutton);
+	update_entry_and_bool_from_toggle_button (&fetch_new_on_group_enter,
+	                                          DEFAULT_VALUE_FETCH_NEW_ON_GROUP_ENTER,
+	                                          KEY_FETCH_NEW_ON_GROUP_ENTER,
+	                                          win->fetch_new_on_group_enter_cbutton);
+
+	update_entry_and_bool_from_toggle_button (&mark_read_on_group_exit,
+	                                          DEFAULT_VALUE_MARK_READ_ON_GROUP_EXIT,
+	                                          KEY_MARK_READ_ON_GROUP_EXIT,
+	                                          win->mark_read_on_group_exit_cbutton);
 
 	update_entry_and_bool_from_toggle_button (&fetch_new_on_startup,
 	                                          DEFAULT_VALUE_FETCH_NEW_FROM_SUBSCRIBED_ON_STARTUP,
@@ -841,7 +849,8 @@ prefs_init (void)
 	expand_all_threads_by_default      = pan_config_get_bool (KEY_HEADER_PANE_THREADS_EXPANDED,          DEFAULT_VALUE_HEADER_PANE_THREADS_EXPANDED);
 	break_thread_when_subject_changes  = pan_config_get_bool (KEY_BREAK_THREAD_WHEN_SUBJECT_CHANGES,     DEFAULT_VALUE_BREAK_THREAD_WHEN_SUBJECT_CHANGES);
 	fetch_new_and_bodies_on_startup    = pan_config_get_bool (KEY_FETCH_NEW_AND_BODIES_ON_STARTUP,       DEFAULT_VALUE_FETCH_NEW_AND_BODIES_ON_STARTUP);
-	fetch_new_on_group_load            = pan_config_get_bool (KEY_FETCH_NEW_ON_GROUP_LOAD,               DEFAULT_VALUE_FETCH_NEW_ON_GROUP_LOAD);
+	fetch_new_on_group_enter           = pan_config_get_bool (KEY_FETCH_NEW_ON_GROUP_ENTER,              DEFAULT_VALUE_FETCH_NEW_ON_GROUP_ENTER);
+	mark_read_on_group_exit            = pan_config_get_bool (KEY_MARK_READ_ON_GROUP_EXIT,               DEFAULT_VALUE_MARK_READ_ON_GROUP_EXIT);
 	fetch_new_on_startup               = pan_config_get_bool (KEY_FETCH_NEW_FROM_SUBSCRIBED_ON_STARTUP,  DEFAULT_VALUE_FETCH_NEW_FROM_SUBSCRIBED_ON_STARTUP);
 	single_click_selects_groups        = pan_config_get_bool (KEY_SINGLE_CLICK_SELECTS_GROUPS,           DEFAULT_VALUE_SINGLE_CLICK_SELECTS_GROUPS);
 	single_click_selects_headers       = pan_config_get_bool (KEY_SINGLE_CLICK_SELECTS_HEADERS,          DEFAULT_VALUE_SINGLE_CLICK_SELECTS_HEADERS);
@@ -1518,7 +1527,8 @@ header_pane_page (void)
 		/* containing widget */
 		w = gtk_scrolled_window_new (NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(w), win->column_tree_view);
+		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(w), GTK_SHADOW_IN);
+		gtk_container_add (GTK_CONTAINER(w), win->column_tree_view);
 		gtk_widget_set_usize (w, 150u, 150u);
 		gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 
@@ -1702,11 +1712,15 @@ behavior_page (void)
 	pan_hig_workarea_add_section_divider (t, &row);
 	pan_hig_workarea_add_section_title (t, &row, _("Automatic Downloads"));
 
-		pan_hig_workarea_add_section_spacer (t, row, 3);
+		pan_hig_workarea_add_section_spacer (t, row, 4);
 
-		add_wide_check_button (t, &row, _("Download new headers when loading a _group"),
-		                       fetch_new_on_group_load,
-		                       &win->fetch_new_on_group_load_cbutton);
+		add_wide_check_button (t, &row, _("Download new headers when entering a _group"),
+		                       fetch_new_on_group_enter,
+		                       &win->fetch_new_on_group_enter_cbutton);
+
+		add_wide_check_button (t, &row, _("_Mark group's articles read when leaving a group"),
+		                       mark_read_on_group_exit,
+		                       &win->mark_read_on_group_exit_cbutton);
 
 		add_wide_check_button (t, &row, _("Download new headers from s_ubscribed groups when starting Pan"),
 		                       fetch_new_on_startup,
