@@ -89,7 +89,8 @@
 
 /* BEHAVIOR */
 #define PREFIX_BEHAVIOR "/Pan/behavior/"
-#define KEY_FETCH_NEW_ON_GROUP_LOAD                PREFIX_BEHAVIOR  "fetch_new_on_group_load"
+#define KEY_FETCH_NEW_ON_GROUP_ENTER               PREFIX_BEHAVIOR  "fetch_new_on_group_load"
+#define KEY_MARK_READ_ON_GROUP_EXIT                PREFIX_BEHAVIOR  "mark_read_on_group_exit"
 #define KEY_FETCH_NEW_FROM_SUBSCRIBED_ON_STARTUP   PREFIX_BEHAVIOR  "fetch_new_from_subscribed_on_start"
 #define KEY_FETCH_NEW_AND_BODIES_ON_STARTUP        PREFIX_BEHAVIOR  "fetch_new_and_bodies_from_described_on_startup"
 #define KEY_SINGLE_CLICK_SELECTS_GROUPS            PREFIX_BEHAVIOR  "single_click_selects_groups"
@@ -139,7 +140,8 @@ extern gboolean                text_window_smooth_scrolling;
 extern int                     text_window_smooth_scrolling_speed;
 extern gboolean                fetch_new_on_startup;
 extern gboolean                fetch_new_and_bodies_on_startup;
-extern gboolean                fetch_new_on_group_load;
+extern gboolean                fetch_new_on_group_enter;
+extern gboolean                mark_read_on_group_exit;
 extern gboolean                expand_all_threads_by_default;
 extern gboolean                hide_mpart_child_nodes;
 extern gboolean                pan_mute;
