@@ -88,7 +88,7 @@ QueueTodo;
 static void fire_queue_size_changed (int running_qty, int total_qty);
 static void fire_connection_size_changed (int increment);
 static void fire_online_status_changed (gboolean online);
-static void fire_message_id_status_changed (const char ** message_ids, int message_id_qty);
+static void fire_message_id_status_changed (const PString ** mids, int qty);
 
 static void queue_run_thread (gpointer data, gpointer user_data);
 
@@ -138,7 +138,6 @@ static void queue_do_todo (void);
 PanCallback * queue_tasks_added           = NULL;
 PanCallback * queue_tasks_removed         = NULL;
 PanCallback * queue_tasks_moved           = NULL;
-PanCallback * queue_task_status_changed   = NULL;
 
 /***********
 ************  Private
@@ -194,14 +193,13 @@ queue_init (gboolean online, gboolean remove_failed_tasks)
 
 	server_to_pool = g_hash_table_new (g_direct_hash, g_direct_equal);
 	task_to_status = g_hash_table_new (g_direct_hash, g_direct_equal);
-	message_id_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	message_id_hash = g_hash_table_new_full (pstring_hash, pstring_equal, (GDestroyNotify)pstring_free, NULL);
 
 	todo_queue = g_queue_new ();
 
 	queue_tasks_added = pan_callback_new ();
 	queue_tasks_removed = pan_callback_new ();
 	queue_tasks_moved = pan_callback_new ();
-	queue_task_status_changed = pan_callback_new ();
 
 	task_lock = g_mutex_new ();
 	cond_mutex = g_mutex_new ();
@@ -262,58 +260,58 @@ get_socket_pool_from_server (Server * server)
 ***/
 
 int
-queue_get_message_id_status (const char * message_id)
+queue_get_message_id_status (const PString * message_id)
 {
 	return GPOINTER_TO_INT (g_hash_table_lookup (message_id_hash, message_id));
 }
 static void
-queue_add_message_id_state (const char   ** message_ids,
-                            int             message_id_qty,
-                            int             state)
+queue_add_message_id_state (const PString    ** mids,
+                            int                 mid_qty,
+                            int                 state)
 {
 	int i;
 
 	/* sanity clause */
 	g_return_if_fail (state);
-	g_return_if_fail (message_ids!=NULL);
-	g_return_if_fail (message_id_qty>0);
-	g_return_if_fail (is_nonempty_string(message_ids[0]));
+	g_return_if_fail (mids!=NULL);
+	g_return_if_fail (mid_qty>0);
+	g_return_if_fail (pstring_is_set(mids[0]));
 
 	/* add the state */
-	for (i=0; i<message_id_qty; ++i)
+	for (i=0; i<mid_qty; ++i)
 	{
-		const char * id = message_ids[i];
-		const int new_state = state | queue_get_message_id_status(id);
-		g_hash_table_insert (message_id_hash, g_strdup(id), GINT_TO_POINTER(new_state));
+		const int new_state = state | queue_get_message_id_status(mids[i]);
+		g_hash_table_insert (message_id_hash, pstring_dup(mids[i]), GINT_TO_POINTER(new_state));
 	}
 
-	fire_message_id_status_changed (message_ids, message_id_qty);
+	fire_message_id_status_changed (mids, mid_qty);
 }
+
 static void
-queue_remove_message_id_state (const char **   message_ids,
-                               int             message_id_qty,
-                               int             state)
+queue_remove_message_id_state (const PString   ** mids,
+                               int                mid_qty,
+                               int                state)
 {
 	int i;
 
 	/* sanity clause */
 	g_return_if_fail (state);
-	g_return_if_fail (message_ids!=NULL);
-	g_return_if_fail (message_id_qty>0);
-	g_return_if_fail (is_nonempty_string(message_ids[0]));
+	g_return_if_fail (mids!=NULL);
+	g_return_if_fail (mid_qty>0);
+	g_return_if_fail (pstring_is_set(mids[0]));
 
 	/* remove the state */
-	for (i=0; i<message_id_qty; ++i)
+	for (i=0; i<mid_qty; ++i)
 	{
-		const char * id = message_ids[i];
+		const PString * id = mids[i];
 		const int new_state = queue_get_message_id_status(id) & ~state;
 		if (!new_state)
 			g_hash_table_remove (message_id_hash, id);
 		else
-			g_hash_table_insert (message_id_hash, g_strdup(id), GINT_TO_POINTER(new_state));
+			g_hash_table_insert (message_id_hash, pstring_dup(id), GINT_TO_POINTER(new_state));
 	}
 
-	fire_message_id_status_changed (message_ids, message_id_qty);
+	fire_message_id_status_changed (mids, mid_qty);
 }
 
 static int
@@ -350,15 +348,14 @@ queue_add_message_ids (Task * task)
 	if (state)
 	{
 		int i;
-		GPtrArray * message_ids = g_ptr_array_new ();
+		PString ** mids = g_newa (PString*, task->identifiers->len);
+
 		for (i=0; i<task->identifiers->len; ++i) {
 			MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (task->identifiers, i));
-			g_ptr_array_add (message_ids, mid->message_id);
+			mids[i] = &mid->message_id;
 		}
 
-		queue_add_message_id_state ((const char **)message_ids->pdata, message_ids->len, state);
-
-		g_ptr_array_free (message_ids, TRUE);
+		queue_add_message_id_state ((const PString**)mids, task->identifiers->len, state);
 	}
 }
 
@@ -370,16 +367,14 @@ queue_remove_message_ids (Task * task)
 	if (state)
 	{
 		int i;
-		GPtrArray * message_ids = g_ptr_array_new ();
-		for (i=0; i<task->identifiers->len; ++i)
-		{
+		PString ** mids = g_newa (PString*, task->identifiers->len);
+
+		for (i=0; i<task->identifiers->len; ++i) {
 			MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (task->identifiers, i));
-			g_ptr_array_add (message_ids, mid->message_id);
+			mids[i] = &mid->message_id;
 		}
 
-		queue_remove_message_id_state ((const char **)message_ids->pdata, message_ids->len, state);
-
-		g_ptr_array_free (message_ids, TRUE);
+		queue_remove_message_id_state ((const PString**)mids, task->identifiers->len, state);
 	}
 }
 
@@ -719,6 +714,8 @@ real_queue_remove_tasks (GSList * tasks, gboolean * do_wakeup)
 		for (l=removed; l!=NULL; l=l->next)
 		{
 			Task * task = TASK(l->data);
+
+			pan_callback_call (task->task_ran_callback, task, GINT_TO_POINTER(task->state.health));
 			g_hash_table_remove (task_to_status, task);
 			queue_remove_message_ids (task);
 			pan_object_unref (PAN_OBJECT(task));
@@ -824,6 +821,7 @@ real_requeue_failed_tasks (GSList * tasks, gboolean * do_wakeup)
 		for (l=tasks; l!=NULL; l=l->next) {
 			Task * task = TASK(l->data);
 			task->hint_abort = FALSE;
+			task_state_set_health (&task->state, TASK_OK);
 			queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
 		}
 
@@ -871,10 +869,8 @@ queue_set_task_status (Task* task, QueueTaskStatus status)
 	g_return_if_fail (task!=NULL);
 	g_return_if_fail (g_slist_index(task_list, task) != -1);
 
-	g_hash_table_insert (task_to_status, task,
-	                     GINT_TO_POINTER(status));
-	pan_callback_call (queue_task_status_changed, task,
-	                   GINT_TO_POINTER(status)); 
+	if (task_to_status != NULL)
+		g_hash_table_insert (task_to_status, task, GINT_TO_POINTER(status));
 }
 
 /************
@@ -976,7 +972,7 @@ queue_run_thread (gpointer gp_data, gpointer user_data)
 	{
 		/* if the task aborted, the read buffer may still have contents,
 		 * so throw away the socket */
-		if (task->hint_abort)
+		if ((task->state.health != TASK_OK) || task->hint_abort)
 			pan_socket_set_error_flag (sock, TRUE);
 
 		socket_pool_checkin (get_socket_pool_from_server (data->server), sock);
@@ -997,13 +993,13 @@ queue_run (Task * task, TaskFunc func, PanSocket * sock, Server * server)
 
 	g_return_if_fail (task!=NULL);
 
-	/* set the task to TASK_WORKING here in the queue's thread so that
+	/* set the task to `working' here in the queue's thread so that
 	 * queue_run_what_we_can() doesn't kick off two workprocs at exactly
 	 * the same time, causing threading issues.  All workprocs can safely
 	 * assume that the state is TASK_WORKING when they begin, and it's up
 	 * to them to tell the queue if another concurrent workproc would be good.
 	 * (See task-save.c's task_save_run_download()). */
-	task_state_set (&task->state, TASK_WORKING);
+	task_state_set_work_working (&task->state);
 
 	/* This is mostly for the benefit of the GUI */
 	queue_set_task_status (task, QUEUE_TASK_STATUS_RUNNING);
@@ -1032,53 +1028,71 @@ queue_run_what_we_can (void)
 	l = task_list;
 	while (l != NULL)
 	{
-		SocketPool * pool;
-		PanSocket * sock;
 		Task * task = TASK(l->data);
 		const TaskState state = task->state;
 		const QueueTaskStatus qts = queue_get_task_status (task); 
-		const gboolean runnable = qts==QUEUE_TASK_STATUS_QUEUED || qts==QUEUE_TASK_STATUS_RUNNING;
 
 		debug5 (DEBUG_QUEUE,
 			"queue_run_what_we_can: #%d task %p (%s) task-state %d queue-task-state %d",
-			n++, task, status_item_describe(STATUS_ITEM(task)), (int)state.state, (int)qts);
+			n++, task, status_item_describe(STATUS_ITEM(task)), (int)state.health, (int)qts);
 
-		switch (state.state)
+		switch (state.health)
 		{
-			case TASK_NEED_SOCKET:
-				if (runnable) {
-					pool = get_socket_pool_from_server (state.server);
-					socket_pool_request_connection (pool);
-					sock = socket_pool_checkout (pool);
-					if (sock != NULL)
-						queue_run (task, state.func, sock, state.server);
+			case TASK_FAIL:
+			{
+				if (qts != QUEUE_TASK_STATUS_STOPPED)
+				{
+					if (_remove_failed_tasks)
+						queue_remove_tasks (g_slist_append (NULL, task));
+					else
+						queue_set_task_status (task, QUEUE_TASK_STATUS_STOPPED);
 				}
 				break;
+			}
+			case TASK_FAIL_NETWORK:
+			{
+				/* for now, just retry.
+				   maybe at some point in the future we can have the task sleep a bit first.
+				   note the intentional fall-through. */
+				task_state_set_health (&task->state, TASK_OK);
+			}
+			case TASK_OK:
+			{
+				const gboolean runnable = qts==QUEUE_TASK_STATUS_QUEUED || qts==QUEUE_TASK_STATUS_RUNNING;
 
-			case TASK_NEED_WORK:
-				if (runnable)
-					queue_run (task, state.func, NULL, state.server);
-				break;
+				switch (state.work)
+				{
+					case TASK_NEED_SOCKET:
+						if (runnable) {
+							PanSocket * sock;
+							SocketPool * pool = get_socket_pool_from_server (state.server);
+							socket_pool_request_connection (pool);
+							sock = socket_pool_checkout (pool);
+							if (sock != NULL)
+								queue_run (task, state.func, sock, state.server);
+						}
+						break;
 
-			case TASK_WORKING:
-				/* don't do anything -- the task is busy */
-				break;
+					case TASK_NEED_WORK:
+						if (runnable)
+							queue_run (task, state.func, NULL, state.server);
+						break;
 
-			case TASK_SUCCESS:
-			case TASK_FAIL_HOPELESS:
-				if (state.state==TASK_SUCCESS || _remove_failed_tasks)
-					queue_remove_tasks (g_slist_append (NULL, task));
-				else
-					queue_set_task_status (task, QUEUE_TASK_STATUS_STOPPED);
-				pan_callback_call (task->task_ran_callback, task, GINT_TO_POINTER(state.state));
-				break;
+					case TASK_WORKING:
+						/* don't do anything -- the task is busy */
+						break;
 
-			default:
-				g_warning ("unhandled state");
-				break;
+					case TASK_COMPLETED:
+						queue_remove_tasks (g_slist_append (NULL, task));
+						break;
+
+					default:
+						g_warning ("unhandled state");
+						break;
+				}
+			}
 		}
 
-
 		/* do we need to do some actions & rebuild the task list? */
 		if (g_queue_is_empty (todo_queue))
 			l = l->next;
@@ -1220,7 +1234,7 @@ queue_get_message_id_status_changed (void)
 }
 
 static void
-fire_message_id_status_changed (const char ** message_ids, int message_id_qty)
+fire_message_id_status_changed (const PString ** message_ids, int message_id_qty)
 {
 	pan_callback_call (queue_get_message_id_status_changed(),
 	                   message_ids,
