@@ -34,6 +34,7 @@
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
 
+#include <pan/article-actions.h>
 #include <pan/queue.h>
 #include <pan/flagset.h>
 #include <pan/task-bodies.h>
@@ -218,8 +219,7 @@ rule_action_apply (const RuleAction * action,
 
 	if (action->flags & RULE_ACTION_DISCARD)
 	{
-		Group * g = ARTICLE(g_ptr_array_index(articles,0))->group;
-		group_remove_articles (g, (Article**)articles->pdata, articles->len);
+		article_action_delete_articles ((Article**)articles->pdata, articles->len, NULL);
 		*deleted = TRUE;
 		return;
 	}
