@@ -37,9 +37,9 @@ typedef enum
 	RULE_ACTION_DECODE		= 1<<10,
 	RULE_ACTION_TAG_FOR_RETRIEVAL	= 1<<11,
 	RULE_ACTION_RETREIVE_BODY	= 1<<12,
-	RULE_ACTION_WATCH		= 1<<13,
+	RULE_ACTION_WATCH               = 1<<13,
 	RULE_ACTION_DISCARD		= 1<<15,
-	RULE_ACTION_IGNORE		= 1<<16
+	RULE_ACTION_IGNORE              = 1<<16,
 }
 RuleActionFlags;
 
