@@ -658,7 +658,7 @@ populate_newsgroup_tab (RuleEditDialog * d, const Rule * r)
 	for (i=0; i!=groups->len; ++i) {
 		Group * g = GROUP(g_ptr_array_index(groups,i));
 		char * name = (char*) group_get_name (g);
-		GtkCList * c = r!=NULL && r->group_type==RULE_GROUP_LIST && rule_test_group(r,g->name) ? app : sub;
+		GtkCList * c = r!=NULL && r->group_type==RULE_GROUP_LIST && rule_test_group(r,&g->name) ? app : sub;
 		gint row = gtk_clist_insert (c, -1, &name);
 		gtk_clist_set_row_data (c, row, g);
 	}
@@ -695,9 +695,8 @@ rule_from_newsgroup_tab (RuleEditDialog * d, Rule * r)
 		r->group_list = g_ptr_array_new ();
 		for (i=0; i!=apply->rows; ++i) {
 			Group * g = GROUP(gtk_clist_get_row_data (apply, i));
-			if (g) {
-				g_ptr_array_add (r->group_list, g_strdup(g->name));
-			}
+			if (g)
+				g_ptr_array_add (r->group_list, g_strndup(g->name.str, g->name.len));
 		}
 	}
 }
