@@ -198,13 +198,13 @@ rule_dup (const Rule * rule)
 *****/
 
 gboolean
-rule_test_group (const Rule * rule,
-                 const gchar * groupname)
+rule_test_group (const Rule    * rule,
+                 const PString * groupname)
 {
 	gboolean retval = FALSE;
 
 	g_return_val_if_fail (rule!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string(groupname), FALSE);
+	g_return_val_if_fail (pstring_is_set (groupname), FALSE);
 
 	switch (rule->group_type)
 	{
@@ -213,15 +213,15 @@ rule_test_group (const Rule * rule,
 			break;
 
 		case RULE_GROUP_WILDCARD:
-			retval = g_pattern_match_simple (rule->group_wildcard, groupname);
+			retval = g_pattern_match_simple (rule->group_wildcard, groupname->str);
 			break;
 
 		case RULE_GROUP_LIST:
 		{
 			guint i;
 			for (i=0; rule->group_list!=NULL && i!=rule->group_list->len; ++i) {
-				const gchar * name = (const gchar*) g_ptr_array_index (rule->group_list,i);
-				if (!g_strcasecmp(groupname,name)) {
+				const gchar * name = (const char*) g_ptr_array_index (rule->group_list,i);
+				if (!g_strcasecmp(groupname->str,name)) {
 					retval = TRUE;
 					break;
 				}
@@ -250,7 +250,7 @@ rule_remove_group_failures (const Rule * rule,
 	for (i=0; i!=articles->len; )
 	{
 		Group * group = ARTICLE(g_ptr_array_index(articles,i))->group;
-		gboolean pass = rule_test_group (rule, group->name);
+		gboolean pass = rule_test_group (rule, &group->name);
 
 		if (pass)
 			++i;
