@@ -33,5 +33,11 @@
 extern void save_attachment_as (GSList      * gslist_of_article_gslists,
                                 const char  * optional_download_dir);
 
+/**
+ * Identical to save_attachment_as, except it doesn't preprocess
+ * the articles by weeding them.
+ */
+extern void manual_decode_attachment_as (GSList      * gslist_of_article_gslists,
+                                         const char  * optional_download_dir);
 
 #endif /* __QUEUE_ITEM_BODY__H__ */
