@@ -53,19 +53,18 @@ sort_servers_by_name (gconstpointer va, gconstpointer vb, gpointer unused)
 {
 	const Server * a = *(const Server **)va;
 	const Server * b = *(const Server **)vb;
-	return g_strcasecmp (a->name, b->name);
+	return g_strcasecmp (a->name.str, b->name.str);
 }
 
 static void
 server_selected_cb (gpointer user_data, int unused, GtkWidget * w)
 {
-	const char * sel_name;
 	Server * cur_server;
 	Server * sel_server;
+	const PString sel_name = pstring_shallow ((char*) g_object_get_data (G_OBJECT(w), "server_name"), -1);
 
 	/* get the selected server */
-	sel_name = (const char*) g_object_get_data (G_OBJECT(w), "server_name");
-	sel_server = serverlist_get_named_server (sel_name);
+	sel_server = serverlist_get_named_server (&sel_name);
 
 	/* get the cur server */
 	cur_server = serverlist_get_active_server ();
@@ -82,8 +81,10 @@ server_activated_cb (gpointer server_gp, gpointer foo, gpointer bar)
 	GtkWidget * w = NULL;
 	Server * server = (Server*) server_gp;
 
-	if (server != NULL)
+	if (server != NULL) {
 		path = g_strdup_printf ("/Servers/%s", server_get_name (server));
+		g_strdelimit (path, "_", ' ');
+	}
 	if (path != NULL)
 		w = gtk_item_factory_get_widget (_main_menu_factory, path);
 	if (w != NULL)
@@ -145,8 +146,7 @@ server_menu_update (void)
 		g_ptr_array_sort_with_data (servers, sort_servers_by_name, NULL);
 		for (i=0; i<servers->len; ) {
 			const Server * server = SERVER(g_ptr_array_index(servers,i));
-			const char * name = server_get_name (server);
-			if (!strncmp (name, INTERNAL_SERVER_NAME, strlen(INTERNAL_SERVER_NAME)))
+			if (pstring_equal (&INTERNAL_SERVER_NAME, &server->name))
 				g_ptr_array_remove_index (servers, i);
 			else
 				++i;
@@ -156,10 +156,11 @@ server_menu_update (void)
 		entries = g_new0 (GtkItemFactoryEntry, alloc_qty);
 		for (qty=0; qty<servers->len; ++qty)
 		{
-			const Server * server = SERVER(g_ptr_array_index(servers,qty));
-
 			/* build an entry: the callback arg is the index into the servers array */
-			entries[qty].path = g_strdup_printf ("/Servers/%s", server_get_name(server));
+			const Server * server = SERVER(g_ptr_array_index(servers,qty));
+			char * path = g_strdup_printf ("/Servers/%s", server_get_name (server));
+			g_strdelimit (path, "_", ' ');
+			entries[qty].path = path;
 			entries[qty].item_type = qty==0 ? "<RadioItem>" : radio_path;
 			entries[qty].callback = server_selected_cb;
 			if (qty==0)
