@@ -609,7 +609,8 @@ server_dialog_new (GtkWindow * parent)
 
         w = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(w), d->server_tree_view);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(w), GTK_SHADOW_IN);
+	gtk_container_add (GTK_CONTAINER(w), d->server_tree_view);
 	gtk_box_pack_start (GTK_BOX (hbox), w, TRUE, TRUE, 0);
 	gtk_widget_set_usize (w, 300, 300);
 
