@@ -24,7 +24,15 @@
 #include <unistd.h>
 
 #include <glib.h>
+
+#if defined(HAVE_GNET_1)
 #include <gnet/gnet.h>
+#define gnet_inetaddr_get_host_name gnet_inetaddr_gethostname 
+#elif defined (HAVE_GNET_2)
+#include <gnet.h>
+#else
+#error must have GNet 1 or GNet 2
+#endif
 
 #include <gmime/gmime-utils.h>
 
@@ -90,24 +98,25 @@ smtp_get_response (StatusItem   * item,
  * @param string a string to write.
  * @return zero on success (no write errors), nonzero on failure.
  */
-static int
+static TaskStateEnum
 smtp_send_line (StatusItem   * item,
                 PanSocket    * sock,
                 const char   * str)
 {
-	int retval = 0;
+	TaskStateEnum val;
+
 	if (pan_mute)
 	{
 		g_message ("(MUTE) to smtp: [%s]", str);
-		retval = 0;
+		val = TASK_OK;
 	}
 	else
 	{
-		const int retval = pan_socket_putline (sock, str);
-		if (retval)
+		val = pan_socket_putline (sock, str);
+		if (val != TASK_OK)
 			status_item_emit_error (item, sockwrite_err_msg);
 	}
-	return retval;
+	return val;
 }
 
 
@@ -249,17 +258,11 @@ smtp_send_message (StatusItem    * item,
 	{
 		char * pch = build_nntp_message (message);
 		const char * march = pch;
-		int line_len = 0;
-		const char * line_start = NULL;
+		PString line = PSTRING_INIT;
 
-		while ((get_next_token_run (march, '\n', &march, &line_start, &line_len)))
-		{
-			char * line = g_newa (char, line_len+1);
-			memcpy (line, line_start, line_len);
-			line[line_len] = '\0';
+		while (get_next_token_pstring (march, '\n', &march, &line))
+			okay = okay && !smtp_send_line_va (item, sock, "%*.*s\n", line.len, line.len, line.str);
 
-			okay = okay && !smtp_send_line_va (item, sock, "%s\n", line);
-		}
 		g_free (pch);
 	}
 
@@ -279,7 +282,7 @@ static int
 smtp_handshake (StatusItem   * item,
                 PanSocket    * sock)
 {
-	char * my_name = gnet_inetaddr_gethostname ();
+	char * my_name = gnet_inetaddr_get_host_name ();
 
 	status_item_emit_status (item, _("Handshaking with mail server"));
 
@@ -317,21 +320,23 @@ smtp_disconnect (StatusItem   * item,
 static PanSocket*
 smtp_get_connection (StatusItem *item)
 {
+	const PString mail_server = pstring_shallow ("mail_server", -1);
+	const PString mail_address = pstring_shallow (mail_server_address, -1);
 	PanSocket *sock = NULL;
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(mail_server_address), NULL);
+	g_return_val_if_fail (pstring_is_set (&mail_server), NULL);
 	g_return_val_if_fail (mail_server_port>0, NULL);
 
 	/* get smtp address/port */
 	status_item_emit_status (item, _("Connecting to mail server"));
 
 	/* get the socket... */
-	sock = pan_socket_new ("mail_server", mail_server_address, mail_server_port);
+	sock = pan_socket_new (&mail_server, &mail_address, mail_server_port);
 	if (sock == NULL) {
 		status_item_emit_error_va (item,
-			_("Unable to connect to mail server \"%s\""),
-			mail_server_address);
+			_("Unable to connect to mail server \"%*.*s\""),
+			mail_address.len, mail_address.len, mail_address.str);
 		pan_object_unref (PAN_OBJECT(sock));
 		sock = NULL;
 	}
