@@ -23,6 +23,8 @@
 
 #include <config.h>
 
+#include <stdio.h>
+
 #include <glib.h>
 
 #include <pan/base/debug.h>
@@ -30,6 +32,7 @@
 #include <pan/base/log.h>
 
 #include <pan/nntp.h>
+#include <pan/queue.h>
 #include <pan/sockets.h>
 #include <pan/socket-pool.h>
 #include <pan/task.h>
@@ -123,21 +126,29 @@ connect_thread_func (gpointer data)
 
 		/* create the socket */
 		debug1 (DEBUG_QUEUE, "connecting to server %s", server->name);
-		sock = pan_socket_new (server->name, server->address, server->port);
-		pan_socket_set_nntp_auth (sock, server->need_auth, server->username, server->password);
+		sock = pan_socket_new (&server->name, &server->address, server->port);
+		pan_socket_set_nntp_auth (sock, server->need_auth, &server->username, &server->password);
 
 		/* handshake */
 		debug2 (DEBUG_QUEUE, "handshake to server %s sock %p", server->name, sock);
-		if (sock != NULL) {
+		if (sock != NULL)
+		{
 			gboolean posting_ok = FALSE;
 			int val;
 			StatusItem * status = status_item_new_with_description (_("Connecting"));
 			status_item_set_active (status, TRUE);
+
 			val = nntp_handshake (status, sock, &posting_ok);
-			if (val != TASK_SUCCESS) {
+			if (val != TASK_OK) {
 				pan_object_unref (PAN_OBJECT(sock));
 				sock = NULL;
 			}
+
+			if (val == TASK_FAIL) { /* the news server doesn't love us anymore! */
+				queue_set_online (FALSE);
+				break;
+			}
+
 			status_item_set_active (status, FALSE);
 			pan_object_unref (PAN_OBJECT(status));
 		}
@@ -281,7 +292,7 @@ socket_pool_refresh (SocketPool * pool)
 				status_item_set_active (status, TRUE);
 
 				debug2 (DEBUG_QUEUE, "sending keepalive pool %p sock %p", pool, sock);
-				destroy = nntp_noop (status, sock) != TASK_SUCCESS;
+				destroy = nntp_noop (status, sock) != TASK_OK;
 				pan_socket_set_last_action_time (sock, last_action_time);
 
 				status_item_set_active (status, FALSE);
