@@ -62,16 +62,16 @@
 
 static void task_headers_run_download (Task * task, PanSocket * sock);
 
-static gchar* task_headers_describe (const StatusItem *item);
+static char* task_headers_describe (const StatusItem *item);
 
-static gint task_get_header_range (TaskHeaders   * task,
-                                   PanSocket     * sock,
-                                   gulong        * setme_lo_article_in_group,
-                                   gulong        * setme_hi_article_in_group,
-                                   gulong        * setme_lo_article_to_fetch,
-                                   gulong        * setme_hi_article_to_fetch,
-                                   gulong        * setme_total_in_group,
-                                   const char   ** setme_progress_fmt);
+static TaskStateEnum task_get_header_range (TaskHeaders   * task,
+                                            PanSocket     * sock,
+                                            gulong        * setme_lo_article_in_group,
+                                            gulong        * setme_hi_article_in_group,
+                                            gulong        * setme_lo_article_to_fetch,
+                                            gulong        * setme_hi_article_to_fetch,
+                                            gulong        * setme_total_in_group,
+                                            const char   ** setme_progress_fmt);
 
 /*********************
 **********************  Variables
@@ -128,7 +128,7 @@ real_task_headers_new (Group                * group,
 	                  task_headers_describe,
 	                  group->server,
 	                  TRUE); /* high priority */
-	task_state_set_need_socket (&TASK(item)->state, group->server, task_headers_run_download);
+	task_state_set_work_need_socket (&TASK(item)->state, group->server, task_headers_run_download);
 	TASK(item)->type = TASK_TYPE_HEADERS;
 	item->group = group;
 	item->download_type = dl_type;
@@ -175,6 +175,33 @@ task_headers_new_sample (Group     * group,
 ******
 *****/
 
+static void
+mark_all_articles_old (Group * group, Article ** articles, guint article_qty, gpointer user_data)
+{
+	if (article_qty > 0u)
+		articles_set_new (articles, article_qty, FALSE);
+}
+
+static void
+mark_read_articles_old (Group * group, Article ** articles, guint article_qty, gpointer user_data)
+{
+	guint i;
+	GPtrArray * read;
+
+	/* get a list of read articles */
+	read = g_ptr_array_new ();
+	for (i=0; i<article_qty; ++i)
+		if (article_is_read (articles[i]))
+			g_ptr_array_add (read, articles[i]);
+
+	/* mark those articles as old */
+	if (read->len > 0)
+		articles_set_new ((Article**)read->pdata, read->len, FALSE);
+
+	/* cleanup */
+	g_ptr_array_free (read, TRUE);
+}
+
 static void
 task_headers_run_download (Task * task, PanSocket * sock)
 {
@@ -182,27 +209,28 @@ task_headers_run_download (Task * task, PanSocket * sock)
 	TaskHeaders * task_h = TASK_HEADERS(task);
 	gulong low_in_group, high_in_group;
 	gulong low_to_fetch, high_to_fetch;
+	gulong high_reached = 0ul;
 	gulong total_in_group;
 	const char * progress_fmt;
 	gboolean reffed = FALSE;
 	gboolean state_set = FALSE;
-	TaskStateEnum state = TASK_SUCCESS;
-	int result;
+	TaskStateEnum val;
+	TaskStateEnum state = TASK_OK;
 	debug_enter ("task_headers_run");
 
 	/* get up-to-date information about the group */
 	progress_fmt = NULL;
 	low_in_group = high_in_group = low_to_fetch = high_to_fetch = 0;
-	result = task_get_header_range (task_h, sock,
-	                                &low_in_group,
-	                                &high_in_group,
-	                                &low_to_fetch,
-	                                &high_to_fetch,
-	                                &total_in_group,
-	                                &progress_fmt);
-	if (result != TASK_SUCCESS) {
+	val = task_get_header_range (task_h, sock,
+	                             &low_in_group,
+	                             &high_in_group,
+	                             &low_to_fetch,
+	                             &high_to_fetch,
+	                             &total_in_group,
+	                             &progress_fmt);
+	if (val != TASK_OK) {
 		state_set = TRUE;
-		state = result;
+		state = val;
 	}
 
 	/* where to start downloading? */
@@ -212,14 +240,14 @@ task_headers_run_download (Task * task, PanSocket * sock)
 	/* no articles to fetch */
 	if (!state_set && (high_to_fetch==0 || low_to_fetch>high_to_fetch)) {
 		state_set = TRUE;
-		state = TASK_SUCCESS;
+		state = TASK_OK;
 	}
 
 	/* if articles to fetch, try fetching them */
 	if (!state_set)
 	{
+		int steps;
 		GPtrArray * articles;
-		gint steps;
 
 		/* load the existing headers */
 		if (!reffed) {
@@ -247,50 +275,37 @@ task_headers_run_download (Task * task, PanSocket * sock)
 
 		/* get the article headers... */
 		articles = g_ptr_array_new ();
-		result = nntp_download_headers (status,
-		                                sock,
-		                                task_h->group,
-		                                low_to_fetch,
-		                                high_to_fetch,
-		                                &task->hint_abort,
-		                                progress_fmt,
-		                                articles);
-		if (result != TASK_SUCCESS) {
+		val = nntp_download_headers (status,
+		                             sock,
+		                             task_h->group,
+		                             low_to_fetch,
+		                             high_to_fetch,
+		                             &task->hint_abort,
+		                             progress_fmt,
+		                             &high_reached,
+		                             articles);
+
+		if (val != TASK_OK) {
 			state_set = TRUE;
-			state = result;
+			state = val;
 		}
 
-		if (1)
+		/* if we've read the group since the last fetch, then mark some
+		   articles as old.  If there are new articles in this fetch,
+		   then mark all of them as old.  Otherwise just mark the read
+		   ones as old. */
+		if (task_h->group->loaded_since_last_fetch)
 		{
-			GPtrArray * a = group_get_article_array (task_h->group);
-
-			if (a->len && task_h->group->loaded_since_last_fetch)
-			{
-				if (a->len > 0)
-				{
-					if (articles->len != 0)
-						articles_set_new ((Article**)a->pdata, a->len, FALSE);
-					else {
-						GPtrArray * read = g_ptr_array_new ();
-						guint i;
-						for (i=0; i<a->len; ++i) {
-							Article * article = ARTICLE(g_ptr_array_index (a, i));
-							if (article_is_read (article))
-								g_ptr_array_add (read, article);
-						}
-						if (read->len > 0)
-							articles_set_new ((Article**)read->pdata, read->len, FALSE);
-						g_ptr_array_free (read, TRUE);
-					}
-				}
-			}
-
-			g_ptr_array_free (a, TRUE);
 			task_h->group->loaded_since_last_fetch = FALSE;
+
+			if (articles->len)
+				group_article_forall (task_h->group, mark_all_articles_old, NULL);
+			else
+				group_article_forall (task_h->group, mark_read_articles_old, NULL);
 		}
 
 		/* if we got new articles, process them */
-		if (articles->len != 0)
+		if (articles->len)
 		{
 			/* log */
 			log_add_va (LOG_INFO, _("Fetched %u headers for \"%s\""),
@@ -300,24 +315,18 @@ task_headers_run_download (Task * task, PanSocket * sock)
 			/* add the articles to the group */
 			group_mark_new_article_number (task_h->group, task_h->group->article_high);
 			group_add_articles_remove_unused (task_h->group, articles, status);
-			group_set_article_range (task_h->group, low_in_group, high_in_group);
+			group_set_article_range (task_h->group, low_in_group, high_reached);
 
 			/* apply the rules to the new headers */
 			rule_manager_process_incoming_articles (articles);
 
 			/* do we need to download these? */
-			if (task_h->download_bodies) {
-				int         i;
-				GPtrArray * articles_to_get;
+			if (task_h->download_bodies)
+			{
+				guint i;
 
-				if (task_h->download_type == HEADERS_ALL)
-					articles_to_get = group_get_article_array (task_h->group);
-				else
-					articles_to_get = articles;
-
-
-				for (i=0; i<articles_to_get->len; ++i) {
-					Article * a = ARTICLE(g_ptr_array_index(articles_to_get,i));
+				for (i=0; i<articles->len; ++i) {
+					Article * a = ARTICLE(g_ptr_array_index(articles,i));
 					MessageIdentifier * mid = message_identifier_new_from_article (a);
 					task_add_identifiers (task, &mid, 1);
 					g_object_unref (mid);
@@ -337,35 +346,36 @@ task_headers_run_download (Task * task, PanSocket * sock)
 		status_item_emit_set_step (status, task_h->body_index);
 
 		/* try downloading the bodies */
-		result = nntp_download_bodies (status,
-		                               sock,
-		                               &task->hint_abort,
-		                               (MessageIdentifier**) task->identifiers->pdata,
-		                               task->identifiers->len,
-		                               &task_h->body_index,
-		                               FALSE);
-		if (result != TASK_SUCCESS) {
+		val = nntp_download_bodies (status,
+		                            sock,
+		                            &task->hint_abort,
+		                            (MessageIdentifier**) task->identifiers->pdata,
+		                            task->identifiers->len,
+		                            &task_h->body_index,
+		                            FALSE);
+		if (val != TASK_OK) {
 			state_set = TRUE;
-			state = result;
+			state = val;
 		}
 	}
 
 	/* cleanup */
 	status_item_emit_progress (status, 0);
 	if (reffed) {
-		server_save_grouplist_if_dirty (task_h->group->server, NULL);
 		file_headers_save (task_h->group, status);
 		group_unref_articles (task_h->group, status);
 	}
 
 	/* if we made it this far without fail, return success. */
 	if (!state_set)
-		state = TASK_SUCCESS;
+		state = TASK_OK;
 
-	if (state==TASK_SUCCESS || state==TASK_FAIL_HOPELESS)
-		task_state_set (&task->state, state);
+	task_state_set_health (&task->state, state);
+
+	if (state==TASK_OK)
+		task_state_set_work_completed (&task->state);
 	else
-		task_state_set_need_socket (&task->state, task->server, task_headers_run_download);
+		task_state_set_work_need_socket (&task->state, task->server, task_headers_run_download);
 
 	debug_exit ("task_headers_run");
 }
@@ -374,7 +384,7 @@ task_headers_run_download (Task * task, PanSocket * sock)
 ******
 *****/
 
-static int
+static TaskStateEnum
 task_get_header_range (TaskHeaders   * task,
                        PanSocket     * sock,
 		       gulong        * setme_lo_article_in_group,
@@ -393,26 +403,26 @@ task_get_header_range (TaskHeaders   * task,
 	gulong lo_article_in_group;
 	gulong hi_article_in_group;
 	gulong total_in_group;
-	gint result;
+	TaskStateEnum val;
 	debug_enter ("task_get_header_range");
 
 	/* sanity checks */
-	g_return_val_if_fail (sock!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL);
 
 	/* change to the right group */
 	lo_article_in_group = 0;
 	hi_article_in_group = 0;
 	total_in_group = 0;
-	result = nntp_get_group_info (STATUS_ITEM(task),
-	                              sock,
-	                              group->name,
-	                              &total_in_group,
-	                              &lo_article_in_group,
-	                              &hi_article_in_group,
-	                              abort);
-	if (result != TASK_SUCCESS)
-		return result;
+	val = nntp_get_group_info (STATUS_ITEM(task),
+	                           sock,
+	                           &group->name,
+	                           &total_in_group,
+	                           &lo_article_in_group,
+	                           &hi_article_in_group,
+	                           abort);
+	if (val != TASK_OK)
+		return val;
 
 	/* figure out which articles to ask for, based on the first/last
 	 * returned by the server and the dl_type of this task.
@@ -463,7 +473,7 @@ task_get_header_range (TaskHeaders   * task,
 	*setme_progress_fmt        = progress_fmt;
 
 	debug_exit ("task_get_header_range");
-	return TASK_SUCCESS;
+	return TASK_OK;
 }
 
 static char*
