@@ -847,10 +847,11 @@ task_manager_build_ui (GtkWidget * window)
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
 	gui_restore_column_widths_tree_view (ui->task_manager_tree_view, "task_manager_2");
-		
+
 	w = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(w), ui->task_manager_tree_view);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(w), GTK_SHADOW_IN);
+	gtk_container_add (GTK_CONTAINER(w), ui->task_manager_tree_view);
 	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
 
 	pan_callback_add (queue_tasks_added, tasks_added_cb, ui);
