@@ -98,7 +98,8 @@ task_constructor (Task                    * task,
 	task->identifiers = g_ptr_array_new ();
 	task->thread_qty = 0;
 	task->sockets = NULL;
-	task_state_set (&task->state, TASK_SUCCESS);
+	task_state_set_health (&task->state, TASK_OK);
+	task_state_set_work_completed (&task->state);
 
         debug1 (DEBUG_PAN_OBJECT, "task constructor: %p", task);
 }
