@@ -166,10 +166,9 @@ append_text_buffer_nolock (const TextMassager  * text_massager,
                            const char          * body,
 		           gboolean              mute_quotes)
 {
-	int line_len;
+	PString line;
 	char * freeme1 = NULL;
 	char * freeme2 = NULL;
-	const char * line;
 	const char * pch;
 	const char * last_quote_begin = NULL;
 	const char * quote_tag = NULL;
@@ -200,29 +199,29 @@ append_text_buffer_nolock (const TextMassager  * text_massager,
 	gtk_text_buffer_delete_mark (text_buffer, mark);
 
 	/* markup quotes */
-	line = NULL;
-	line_len = 0;
+	line = PSTRING_INIT;
 	pch = last_quote_begin = body;
 	mark_start = start;
-	while (get_next_token_run (pch, '\n', &pch, &line, &line_len)) {
-		if (!line_len || (line_len==1 && *line=='\n'))
+	while (get_next_token_pstring (pch, '\n', &pch, &line))
+	{
+		if (!line.len || (line.len==1 && *line.str=='\n'))
 			continue;
 		
 		if (!is_sig)
-			is_sig = pan_is_signature_delimiter (line, line_len) != SIG_NONE;
+			is_sig = pan_is_signature_delimiter (line.str, line.len) != SIG_NONE;
 
-		quote_tag = get_quote_tag (text_massager, line, line_len);
+		quote_tag = get_quote_tag (text_massager, line.str, line.len);
 		if (strcmp (quote_tag, "quote_0"))
 			is_sig = FALSE;
 		
 		quote_tag = is_sig?"signature":quote_tag;
 		if (last_quote_tag!=NULL && strcmp (quote_tag, last_quote_tag)) {
 			mark_end = mark_start;
-			gtk_text_iter_forward_chars (&mark_end, g_utf8_strlen(last_quote_begin,line-1-last_quote_begin));
+			gtk_text_iter_forward_chars (&mark_end, g_utf8_strlen(last_quote_begin,line.str-1-last_quote_begin));
 			gtk_text_buffer_apply_tag_by_name (text_buffer, last_quote_tag, &mark_start, &mark_end);
 			mark_start = mark_end;
 			gtk_text_iter_forward_chars (&mark_start, 1);
-			last_quote_begin = line;
+			last_quote_begin = line.str;
 		}
 		last_quote_tag = quote_tag;
 	}
@@ -344,19 +343,19 @@ static void
 text_read_next (gboolean more)
 {
 	if (more)
-		articlelist_read_next ();
+		header_pane_read_next ();
 	else
-		articlelist_read_prev ();
+		header_pane_read_prev ();
 }
 
 static void
-text_read_more_impl (gboolean more)
+text_read_more_with_article (Article * a, gpointer more_gpointer)
 {
+	gboolean more = more_gpointer != NULL;
 	GMimeMessage * message = get_current_message ();
 	const int arbitrary_font_height_pixels_hack = 18;
 	Group * grouplist_group = grouplist_get_selected_group ();
 	const Group * articlelist_group = articlelist_get_group ();
-	const Article * a = articlelist_get_selected_article_nolock ();
 	const char * art_msgid = a ? article_get_message_id (a) : NULL;
 	const char * cur_msgid = message ? g_mime_message_get_message_id (message) : NULL;
 	GtkAdjustment * v = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(scrolled_window));
@@ -378,7 +377,7 @@ text_read_more_impl (gboolean more)
 	}
 	else if (pan_strcmp (art_msgid, cur_msgid)) {
 		/* if the user has selected a different article, jump to that. */
-		articlelist_read_selected ();
+		articlelist_activate_selected ();
 	}
 	else {
 		/* just go to the next article */
@@ -389,6 +388,18 @@ text_read_more_impl (gboolean more)
 		g_object_unref (message);
 }
 
+static void
+text_read_more_impl (gboolean more)
+{
+	const gpointer more_gpointer = GINT_TO_POINTER(more);
+
+	if (header_pane_has_selection ())
+		header_pane_first_selected (text_read_more_with_article, more_gpointer);
+	else
+		text_read_more_with_article (NULL, more_gpointer);
+}
+
+
 void
 text_read_more (void)
 {
@@ -777,6 +788,12 @@ get_current_message (void)
 	return retval;
 }
 
+gboolean
+text_pane_has_message (void)
+{
+	return _current_message != NULL;
+}
+
 static void
 set_current_message_impl (GMimeMessage * message)
 {
@@ -875,19 +892,18 @@ static void
 task_bodies_ran_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
 	Task * task = TASK (call_obj);
+	char * default_charset = (char*) PAN_OBJECT(task)->user_data;
 	const int status = GPOINTER_TO_INT (call_arg);
 	const char * acache_key = (const char*) user_data;
 
-	if (status == TASK_SUCCESS)
+	if (status == TASK_OK)
 	{
-		GMimeMessage * message = acache_get_message (
-			acache_key,
-			(MessageIdentifier**)task->identifiers->pdata,
-			task->identifiers->len);
+		const PString ** ids = message_identifiers_get_id_array ((const MessageIdentifier**)task->identifiers->pdata, task->identifiers->len);
+		GMimeMessage * message = acache_get_message (acache_key, ids, task->identifiers->len);
+		g_free (ids);
 
 		if (message != NULL)
 		{
-			char * default_charset = (char*) PAN_OBJECT(task)->user_data;
 			const char * charset = pan_g_mime_message_get_charset (message);
 
 			if (!is_nonempty_string (charset))
@@ -896,9 +912,10 @@ task_bodies_ran_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 			g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(charset), g_free);
 			set_current_message (message);
 			g_object_unref (message);
-			g_free (default_charset);
 		}
 	}
+
+	g_free (default_charset);
 }
 
 void
@@ -915,13 +932,16 @@ text_set_from_identifiers (Server             * server,
 	else
 	{
 		GMimeMessage * message;
+		const PString ** ids;
 
 		g_return_if_fail (is_nonempty_string(acache_key));
 		g_return_if_fail (server != NULL);
 		g_return_if_fail (mids != NULL);
 		g_return_if_fail (mid_qty > 0);
 
-		message = acache_get_message (acache_key, mids, mid_qty);
+		ids = message_identifiers_get_id_array ((const MessageIdentifier**)mids, mid_qty);
+		message = acache_get_message (acache_key, ids, mid_qty);
+		g_free (ids);
 
 		if (message)
 		{
@@ -954,6 +974,12 @@ text_set_from_identifiers (Server             * server,
 *****
 ****/
 
+static void
+header_pane_group_changed_cb (gpointer call_obj, gpointer call_arg, gpointer client_data)
+{
+	set_current_message (NULL);
+}
+
 static int
 compare_pgchar_pparticle_msgid (const void * a, const void *b)
 {
@@ -1090,16 +1116,11 @@ get_url_from_location (GtkWidget * w, int x, int y)
 		GtkTextTagTable * tag_table = gtk_text_buffer_get_tag_table (text_buffer);
 		url_tag = gtk_text_tag_table_lookup (tag_table, "url");
 		if (url_tag == NULL)
-		{
-			g_assert (url_tag != NULL );
 			return NULL;
-		}
 	}
 	
 	/* ASSERTION FAILED */
-	g_assert( GTK_IS_TEXT_TAG(url_tag));
 	clicked_on_url = gtk_text_iter_has_tag (&pos, url_tag);
-
 	if (clicked_on_url)
 	{
 		GtkTextIter begin;
@@ -1195,52 +1216,6 @@ motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 	return FALSE;
 }
 
-/*
- * The following code was "stolen" from the gtk-devel archives
- */
-static void 
-gtk_text_tag_copy_values (GtkTextTag *dest, GtkTextTag *src)
-{
-	g_return_if_fail (dest);
-	g_return_if_fail (src);
-	g_return_if_fail (GTK_IS_TEXT_TAG (dest));
-	g_return_if_fail (GTK_IS_TEXT_TAG (src));
-
-	gtk_text_attributes_copy_values (src->values, dest->values);
-			      
-#define _COPY_FLAG(flag) dest->flag = src->flag
-	_COPY_FLAG (bg_color_set);
-	_COPY_FLAG (bg_color_set);
-	_COPY_FLAG (bg_stipple_set);
-	_COPY_FLAG (fg_color_set);
-#if 0
-  	_COPY_FLAG (family_set);
-   	_COPY_FLAG (style_set);
-   	_COPY_FLAG (variant_set);
- 	_COPY_FLAG (weight_set);
- 	_COPY_FLAG (stretch_set);
- 	_COPY_FLAG (size_set);
-#endif
-	_COPY_FLAG (fg_stipple_set);
-	_COPY_FLAG (justification_set);
-	_COPY_FLAG (left_margin_set);
-	_COPY_FLAG (indent_set);
-	_COPY_FLAG (rise_set);
-	_COPY_FLAG (strikethrough_set);
-	_COPY_FLAG (right_margin_set);
-	_COPY_FLAG (pixels_above_lines_set);
-	_COPY_FLAG (pixels_below_lines_set);
-	_COPY_FLAG (pixels_inside_wrap_set);
-	_COPY_FLAG (tabs_set);
-	_COPY_FLAG (underline_set);
-	_COPY_FLAG (wrap_mode_set);
-	_COPY_FLAG (bg_full_height_set);
-	_COPY_FLAG (invisible_set);
-	_COPY_FLAG (editable_set);
-	_COPY_FLAG (language_set);
-#undef _COPY_FLAG
-}
-
 
 /**
  ** gtk_text_tag_table_install:
@@ -1251,11 +1226,10 @@ gtk_text_tag_copy_values (GtkTextTag *dest, GtkTextTag *src)
  ** already exists in @table, update the existing tag in the table with
  ** the values in @tag. If the tag is anonymous, it is simply added to
  ** the table.
- ***/
+ **/
 static void
 gtk_text_tag_table_install (GtkTextTagTable *table, GtkTextTag *tag)
 {
-	GtkTextTag *existing_tag;
 	g_return_if_fail (table);
 	g_return_if_fail (GTK_IS_TEXT_TAG_TABLE (table));
 	g_return_if_fail (tag);
@@ -1264,13 +1238,12 @@ gtk_text_tag_table_install (GtkTextTagTable *table, GtkTextTag *tag)
 	if (!tag->name)
 		gtk_text_tag_table_add(table, tag);
 	else {
-		existing_tag = gtk_text_tag_table_lookup(table, tag->name);
-		if (existing_tag) {
-			gtk_text_tag_copy_values(existing_tag, tag);
-			g_object_unref (G_OBJECT(tag));
-		}
-		else
-			gtk_text_tag_table_add(table, tag);
+		GtkTextTag * existing_tag = gtk_text_tag_table_lookup (table, tag->name);
+
+		if (existing_tag)
+			gtk_text_tag_table_remove (table, existing_tag);
+
+		gtk_text_tag_table_add (table, tag);
 	}
 }
 
@@ -1280,42 +1253,69 @@ text_set_text_buffer_tags (GtkTextBuffer * buffer)
 {
 	GtkTextTag * tag;
 	GtkTextTagTable * table = gtk_text_buffer_get_tag_table (buffer);
+	char * header_key_font = NULL;
+	char * header_val_font = NULL;
+	PangoFontDescription * pfd = pango_font_description_from_string (body_pane_monospace_font);
+
+	if (pfd) {
+		header_val_font = pango_font_description_to_string (pfd);
+		pango_font_description_set_weight (pfd, PANGO_WEIGHT_BOLD);
+		header_key_font = pango_font_description_to_string (pfd);
+		pango_font_description_free (pfd);
+	}
+
+	if (!is_nonempty_string (header_key_font))
+		header_key_font = g_strdup ("courier bold");
+	if (!is_nonempty_string (header_val_font))
+		header_val_font = g_strdup ("courier");
 
 	tag = gtk_text_tag_new ("header_key");
-	g_object_set (tag, "font", "courier bold", NULL);
+	g_object_set (tag, "font", header_key_font, NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 
 	tag = gtk_text_tag_new ("header_val");
-	g_object_set (tag, "font", "courier", NULL);
+	g_object_set (tag, "font", header_val_font, NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 
 	tag = gtk_text_tag_new ("center");
 	g_object_set (tag, "justification", GTK_JUSTIFY_CENTER, NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 	
 	tag = gtk_text_tag_new ("url");
 	g_object_set (tag, "underline", PANGO_UNDERLINE_SINGLE, "foreground_gdk", &text_url_color, NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 	
 	tag = gtk_text_tag_new ("quote_0");
 	g_object_set (tag, NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 	
 	tag = gtk_text_tag_new ("quote_1");
 	g_object_set (tag, "foreground_gdk", &text_quoted_color[0], NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 	
 	tag = gtk_text_tag_new ("quote_2");
 	g_object_set (tag, "foreground_gdk", &text_quoted_color[1], NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 
 	tag = gtk_text_tag_new ("quote_3");
 	g_object_set (tag, "foreground_gdk", &text_quoted_color[2], NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
 
 	tag = gtk_text_tag_new ("signature");
 	g_object_set (tag, "foreground_gdk", &signature_color, NULL);
 	gtk_text_tag_table_install (table, tag);
+	g_object_unref (tag);
+
+	g_free (header_key_font);
+	g_free (header_val_font);
 }
 
 	
@@ -1401,6 +1401,8 @@ text_create (void)
 
 	pan_callback_add (group_get_articles_removed_callback(),
 	                  group_articles_removed_cb, NULL);
+	pan_callback_add (articlelist_get_group_changed_callback(),
+	                  header_pane_group_changed_cb, NULL);
 
 
 	Pan.text = text_view;
