@@ -47,6 +47,8 @@ extern PanCallback * current_article_changed;
  */
 extern GMimeMessage * get_current_message (void);
 
+extern gboolean text_pane_has_message (void);
+
 /**
 ***  Wrapping
 **/
@@ -84,10 +86,6 @@ void          text_rot13_selected_text_nolock                (void);
 ***  Colors
 **/
 
-#if 0
-extern GdkColor text_fg_color;
-extern GdkColor text_bg_color;
-#endif
 extern GdkColor text_url_color;
 extern GdkColor text_quoted_color[3];
 extern GdkColor signature_color;
