@@ -171,10 +171,15 @@ pan_timeout_wrapper (gpointer data)
 		argset_free (argset);
 	return retval;
 }
+
 guint
 pan_timeout_add (guint32 interval, GSourceFunc func, gpointer arg)
 {
-	return g_timeout_add (interval, pan_timeout_wrapper, argset_new2 (func, arg));
+	return g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
+	                           interval,
+	                           pan_timeout_wrapper,
+	                           argset_new2 (func, arg),
+	                           (GDestroyNotify)argset_free);
 }
 
 /***
@@ -187,8 +192,10 @@ static GSList * gui_queue_slist = NULL;
 
 typedef struct
 {
-	GSourceFunc func;
+	GSourceFunc run_func;
 	gpointer user_data;
+	GDestroyNotify remove_func;
+	guint item_id;
 }
 GuiQueueItem;
 
@@ -215,7 +222,7 @@ gui_queue_timer_cb (gpointer user_data)
 		for (l=list; l!=NULL; l=l->next)
 		{
 			GuiQueueItem * queue_item = (GuiQueueItem*) l->data;
-			(*queue_item->func)(queue_item->user_data);
+			(*queue_item->run_func)(queue_item->user_data);
 			g_free (queue_item);
 		}
 		g_slist_free (list);
@@ -227,7 +234,7 @@ gui_queue_timer_cb (gpointer user_data)
 void
 gui_queue_init (void)
 {
-	gui_queue_timer_id = pan_timeout_add (100, gui_queue_timer_cb, NULL);
+	gui_queue_timer_id = pan_timeout_add (200, gui_queue_timer_cb, NULL);
 }
 
 void
@@ -237,22 +244,93 @@ gui_queue_shutdown (void)
 	gui_queue_timer_id = 0;
 }
 
-void
-gui_queue_add (GSourceFunc func, gpointer user_data)
+static gboolean
+find_node_from_item_id (gconstpointer item_gpointer, gconstpointer id_key_gpointer)
 {
+	register const GuiQueueItem * item = (const GuiQueueItem *) item_gpointer;
+	register const guint id_key = GPOINTER_TO_UINT (id_key_gpointer);
+
+	/* return 0 when the desired element is found */
+	return id_key == item->item_id ? 0 : 1;
+}
+
+static void
+gui_queue_remove_gpointer (gpointer item_id_gpointer)
+{
+	gui_queue_remove (GPOINTER_TO_UINT (item_id_gpointer));
+}
+
+void
+gui_queue_remove (guint item_id)
+{
+	GSList * l = NULL;
+
+	/* remove the item from the list */
+	g_static_mutex_lock (&gui_queue_mutex);
+	{
+		l = g_slist_find_custom (gui_queue_slist, GUINT_TO_POINTER(item_id), find_node_from_item_id);
+		if (l != NULL)
+			gui_queue_slist = g_slist_remove_link (gui_queue_slist, l);
+	}
+	g_static_mutex_unlock (&gui_queue_mutex);
+
+	/* run the remove function */
+	if (l != NULL)
+	{
+		GuiQueueItem * queue_item = (GuiQueueItem*) l->data;
+		if (queue_item->remove_func != NULL)
+			(*queue_item->remove_func)(queue_item->user_data);
+		g_free (queue_item);
+		g_slist_free_1 (l);
+	}
+}
+
+guint
+gui_queue_add (GSourceFunc run_func, gpointer user_data)
+{
+	return gui_queue_add_full_to_g_object (run_func, user_data, NULL, NULL);
+}
+
+guint
+gui_queue_add_full (GSourceFunc run_func, gpointer user_data, GDestroyNotify remove_func)
+{
+	return gui_queue_add_full_to_g_object (run_func, user_data, remove_func, NULL);
+}
+
+guint
+gui_queue_add_full_to_g_object (GSourceFunc run_func, gpointer user_data, GDestroyNotify remove_func, GObject * object)
+{
+	guint retval;
 	GuiQueueItem * item;
 
 	/* create the new queue item */	
 	item = g_new (GuiQueueItem, 1);
-	item->func = func;
+	item->run_func = run_func;
 	item->user_data = user_data;
+	item->remove_func = remove_func;
 
 	/* add it to the queue */
 	g_static_mutex_lock (&gui_queue_mutex);
-	gui_queue_slist = g_slist_prepend (gui_queue_slist, item);
+	{
+		static guint item_id = 0;
+		retval = item->item_id = item_id++;
+		gui_queue_slist = g_slist_prepend (gui_queue_slist, item);
+
+		/* are we attaching it to a GObject? */
+		if (object!=NULL && G_IS_OBJECT(object))
+		{
+			char str [64];
+			g_snprintf (str, sizeof(str), "gui_queue_item_%u", retval); /* make sure the GObject's data key is unique */
+			g_object_set_data_full (object, str, GUINT_TO_POINTER(retval), gui_queue_remove_gpointer);
+		}
+	}
 	g_static_mutex_unlock (&gui_queue_mutex);
+
+	return retval;
 }
 
+
+
 /***
 ****
 ****  LOCKING
@@ -330,22 +408,28 @@ pan_unlock_from (const gchar* file, const gchar * func, int line)
 ****
 ***/
 
-void
-get_date_display_string (time_t date, const char * fmt, char * fillme, int len)
+char *
+get_date_display_string (time_t date, const char * fmt)
 {
+	char buf[512];
+	char *freeme;
 	struct tm tm_date;
+	const char *cpch;
 
 	/* sanity clause */
-	g_return_if_fail (is_nonempty_string(fmt));
-	g_return_if_fail (fillme != NULL);
-	*fillme = '\0';
+	g_return_val_if_fail (is_nonempty_string(fmt), NULL);
 
 	/* get the date string */
 	pan_localtime_r (&date, &tm_date);
 
 	/* try to put the date in the buffer */
-	if (!strftime (fillme, len, fmt, &tm_date))
-		*fillme = '\0';
+	if (!strftime (buf, sizeof buf, fmt, &tm_date))
+		*buf = '\0';
+
+	cpch = pan_utf8ize (buf, -1, &freeme);
+	replace_gstr (&freeme, g_strdup (cpch));
+
+	return freeme;
 }
 
 static gboolean
@@ -355,17 +439,16 @@ try_to_execute_url (const char * template, const char * url)
 	int argc = 0;
 	char * freeme;
 	char ** argv = NULL;
-	gboolean retval = FALSE;
 	gboolean ok = TRUE;
 
 	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(template), FALSE);
 	g_return_val_if_fail (is_nonempty_string(url), FALSE);
-	if (template == NULL)
-		template = "netscape %s";
 
 	/* make sure the backslashes are escaped */
 	freeme = pan_substitute (template, "\\", "\\\\");
 	template = freeme;
+	debug1 (DEBUG_TRACE, "about to parse the command [%s]", template);
 
 	/* parse the command line */
 	if (ok) {
@@ -379,9 +462,8 @@ try_to_execute_url (const char * template, const char * url)
 		}
 	}
 
-	/* make sure the URL is substitued _after_ the command-line
-	 * parser; those path slashes and backslashes make Windows
-	 * very sad */
+	/* substitute in the URL _after_ parsing the command line:
+	   path slashes and backslashes in the URL confuse g_shell_parse_argv. */
 	for (i=0; i<argc; ++i)
 		if (strstr (argv[i], "%s") != NULL) /* filename */
 			replace_gstr (&argv[i], pan_substitute (argv[i], "%s", url));
@@ -399,40 +481,65 @@ try_to_execute_url (const char * template, const char * url)
 
 	/* cleanup */
 	g_free (freeme);
-	return retval;
+	g_strfreev (argv);
+	return ok;
 }
 
 
+/**
+ * http://www.catb.org/~esr/BROWSER/
+ *
+ * The value of BROWSER may consist of a colon-separated series of browser
+ * command parts.  These should be tried in order until one succeeds. Each
+ * command part may optionally contain the string "%s"; if it does, the URL
+ * to be viewed is substituted there. If a command part does not contain %s,
+ * the browser is to be launched as if the URL had been supplied as its first
+ * argument. The string %% must be substituted as a single %.
+ *
+ * Pan issues: on Windows, ':' is typically embedded in the pathname, so it's
+ * a poor choice for browser command delimiters there.  On Windows, use ';'
+ * instead.
+ */
+
+#ifdef G_OS_WIN32
+#define BROWSER_DELIMITER ';'
+#else
+#define BROWSER_DELIMITER ':'
+#endif
+
 void
 pan_url_show (const char * url)
 {
-	const char * cmd;
+	const char * browser_string;
+	const char * march;
+	GString * gstr;
 
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string(url));
 
-	if (is_nonempty_string (external_web_browser))
-		cmd = external_web_browser;
-	else
-		cmd = g_getenv ("BROWSER");
+	/* if the user has a browser defined, use it; "BROWSER" is fallback #1; "mozilla %s" is fallback #2 */
+	browser_string = external_web_browser;
+	if (!is_nonempty_string (browser_string))
+		browser_string = g_getenv ("BROWSER");
+	if (!is_nonempty_string (browser_string))
+		browser_string = "mozilla %s";
+	debug1 (DEBUG_TRACE, "browser_string is [%s]", browser_string);
 
-	if (is_nonempty_string (cmd))
+	/* walk through the string, trying different browser commands until one works */
+	march = browser_string;
+	gstr = g_string_new (NULL);
+	while (get_next_token_g_str (march, BROWSER_DELIMITER, &march, gstr))
 	{
-		int i;
-		char ** tries;
-		gboolean result = FALSE;
-	
-#ifdef G_OS_WIN32
-#define DELIMITER ";"	
-#else
-#define DELIMITER ":"
-#endif
-		tries = g_strsplit (external_web_browser, DELIMITER, -1);
-		for (i=0; !result && tries[i]!=NULL; ++i)
-			result = try_to_execute_url (tries[i], url);
+		debug1 (DEBUG_TRACE, "parsed [%s] out of browser string", gstr->str);
 
-		g_strfreev (tries);
+		if (strstr (gstr->str, "%s") == NULL)
+			g_string_append (gstr, " %s");
+		
+		if (try_to_execute_url (gstr->str, url))
+			break;
 	}
+
+	g_string_free (gstr, TRUE);
 }
 
 
@@ -513,19 +620,6 @@ menu_set_checked (GtkItemFactory   * ifactory,
 		gtk_check_menu_item_set_active (w, active);
 }
 
-gboolean
-menu_is_checked (GtkItemFactory   * ifactory,
-                 const char       * path)
-{
-	GtkWidget * w;
-
-	g_return_val_if_fail (ifactory!=NULL, FALSE);
-	w = gtk_item_factory_get_item (ifactory, path);
-	g_return_val_if_fail (GTK_IS_CHECK_MENU_ITEM(w), FALSE);
-
-	return gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w));
-}
-
 void
 pan_gtk_entry_set_text (GtkWidget            * w,
                         const char           * text)
@@ -608,6 +702,8 @@ gui_save_window_size (GtkWidget     * widget,
 	char buf[1024];
 	int x, y, w, h;
 	GtkWindow * window;
+	GdkWindow *toplevel;
+	GdkWindowState state = 0;
 
 	/* sanity clause */
 	g_return_if_fail (GTK_IS_WINDOW(widget));
@@ -615,8 +711,10 @@ gui_save_window_size (GtkWidget     * widget,
 	g_return_if_fail (is_nonempty_string(key));
 
 	window = GTK_WINDOW(widget);
+	toplevel = widget->window;
+	state = toplevel ? gdk_window_get_state (toplevel) : 0;
 
-	if (window->maximize_initially)
+	if (state & GDK_WINDOW_STATE_MAXIMIZED)
 	{
 		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
 		pan_config_set_bool (buf, TRUE);
@@ -626,6 +724,8 @@ gui_save_window_size (GtkWidget     * widget,
 		gtk_window_get_position (window, &x, &y);
 		gtk_window_get_size (window, &w, &h);
 
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
+		pan_config_set_bool (buf, FALSE);
 		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
 		pan_config_set_int (buf, x);
 		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
@@ -653,13 +753,13 @@ gui_restore_column_widths (GtkWidget     * clist,
 		/* get width from config... */
 		for (i=0; i!=cols; ++i) {
 			g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
-			widths[i] = pan_config_get_int (buf);
+			widths[i] = pan_config_get_int (buf, 0);
 		}
 
 		/* set ui.. */
 		pan_lock();
 		for (i=0; i!=cols; ++i)
-			if (widths[i] != 0)
+			if (widths[i] > 0)
 				gtk_clist_set_column_width (list, i, widths[i]);
 		pan_unlock();
 
@@ -687,7 +787,7 @@ gui_restore_column_widths_tree_view (GtkWidget     * tree_view,
 	/* get width from config... */
 	for (i = 0; i != length; ++i) {
 		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
-		widths[i] = pan_config_get_int (buf);
+		widths[i] = pan_config_get_int (buf, 0);
 	}
 
 	/* set ui.. */
@@ -724,13 +824,13 @@ gui_restore_window_size (GtkWidget     * widget,
 
 	/* get the dimensions */
 	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
-	x = pan_config_get_int (buf);
+	x = pan_config_get_int (buf, -1);
 	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
-	y = pan_config_get_int (buf);
+	y = pan_config_get_int (buf, -1);
 	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
-	w = pan_config_get_int (buf);
+	w = pan_config_get_int (buf, -1);
 	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
-	h = pan_config_get_int (buf);
+	h = pan_config_get_int (buf, -1);
 
 	/* move & resize */
 	if ((0<=x && x<screen_w-fuzz) && (0<=y && y<screen_h-fuzz))
@@ -744,7 +844,7 @@ gui_restore_window_size (GtkWidget     * widget,
 	{
 		gboolean maximize;
 		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
-		maximize = pan_config_get_bool (buf);
+		maximize = pan_config_get_bool (buf, FALSE);
 
 		if (maximize)
 			gtk_window_maximize (GTK_WINDOW(widget));
@@ -813,3 +913,147 @@ launch_external_editor (const char    * edit_command,
 	g_strfreev (argv);
 	gui_queue_add (finished_callback, finished_user_data);
 }
+
+/***
+****
+***/
+
+GtkWidget*
+pan_gtk_image_new_from_inline_text (const guint8 * inline_text, GtkIconSize icon_size)
+{
+	GdkPixbuf * pixbuf;
+	GtkIconSet * icon_set;
+	GtkWidget * image;
+
+	pixbuf = gdk_pixbuf_new_from_inline (-1, inline_text, FALSE, NULL);
+	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
+	g_object_unref (G_OBJECT(pixbuf));
+	image = gtk_image_new_from_icon_set (icon_set, icon_size);
+	gtk_icon_set_unref (icon_set);
+
+	return image;
+}
+
+/***
+****
+***/
+
+GtkWidget*
+pan_hig_workarea_create (void)
+{
+	GtkWidget * t = gtk_table_new (4, 100, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	gtk_container_set_border_width (GTK_CONTAINER(t), 12);
+	return t;
+}
+
+void
+pan_hig_workarea_add_section_divider  (GtkWidget   * table,
+                                       int         * row)
+{
+	GtkWidget * w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+	gtk_widget_set_usize (w, 0u, 6u);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 4, *row, *row+1, 0, 0, 0, 0);
+	++*row;
+}
+
+void
+pan_hig_workarea_add_section_title    (GtkWidget   * table,
+                                       int         * row,
+                                       const char  * section_title)
+{
+	char buf[512];
+	GtkWidget * l;
+
+	g_snprintf (buf, sizeof(buf), "<b>%s</b>", section_title);
+	l = gtk_label_new (buf);
+	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
+	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
+	gtk_table_attach (GTK_TABLE(table), l, 0, 4, *row, *row+1, GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
+	++*row;
+}
+
+void
+pan_hig_workarea_add_section_spacer  (GtkWidget   * table,
+                                      int           row,
+                                      int           items_in_section)
+{
+	GtkWidget * w;
+
+	/* spacer to move the fields a little to the right of the name header */
+	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+	gtk_widget_set_usize (w, 18u, 0u);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
+
+	/* spacer between the controls and their labels */
+	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+	gtk_widget_set_usize (w, 12u, 0u);
+	gtk_table_attach (GTK_TABLE(table), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
+}
+
+GtkWidget *
+pan_hig_workarea_add_wide_checkbutton   (GtkWidget   * table,
+                                         int         * row,
+                                         const char  * mnemonic_string,
+                                         gboolean      is_active)
+{
+	GtkWidget * w = gtk_check_button_new_with_mnemonic (mnemonic_string);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), is_active);
+	gtk_table_attach_defaults (GTK_TABLE(table), w, 1, 4, *row, *row+1);
+
+	++*row;
+
+	return w;
+}
+
+
+GtkWidget*
+pan_hig_workarea_add_label   (GtkWidget   * table,
+                              int           row,
+                              const char  * mnemonic_string)
+{
+	GtkWidget * l;
+
+	l = gtk_label_new_with_mnemonic (mnemonic_string);
+	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
+	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
+	gtk_table_attach (GTK_TABLE(table), l, 1, 2, row, row+1, GTK_FILL, 0, 0, 0);
+
+	return l;
+}
+
+                                                                                                                                                             
+void
+pan_hig_workarea_add_control (GtkWidget   * table,
+                              int           row,
+                              GtkWidget   * control)
+{
+	gtk_table_attach_defaults (GTK_TABLE(table), control, 3, 4, row, row+1);
+}
+
+                                                                                                                                                             
+GtkWidget*
+pan_hig_workarea_add_row  (GtkWidget   * table,
+                           int         * row,
+                           const char  * mnemonic_string,
+                           GtkWidget   * control,
+                           GtkWidget   * mnemonic_or_null_if_control_is_mnemonic)
+{
+	GtkWidget * l;
+	GtkWidget * mnemonic;
+
+	l = pan_hig_workarea_add_label (table, *row, mnemonic_string);
+	pan_hig_workarea_add_control (table, *row, control);
+
+	if (mnemonic_or_null_if_control_is_mnemonic == NULL)
+		mnemonic = control;
+	else
+		mnemonic = mnemonic_or_null_if_control_is_mnemonic;
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL(l), mnemonic);
+
+	++*row;
+
+	return l;
+}
+
