@@ -121,6 +121,21 @@ test_phrases (Server * server, Group * group)
 	test ("illyjonk", FALSE)
 	test ("fillyjonk ", FALSE)
 
+	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_REGEX, PHRASE_KEY_SUBJECT, "(filly|jonk)", FALSE);
+	test ("illyonking", FALSE)
+	test ("filly at the front", TRUE)
+	test ("Filly at the front, in caps", TRUE)
+	test ("at the end, filly", TRUE)
+	test ("in the middle fillyjonk without caps", TRUE)
+	test ("in the middle Fillyjonk with caps", TRUE)
+
+	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_REGEX, PHRASE_KEY_SUBJECT, "\\bfillyjonk\\b", FALSE);
+	test ("fillyyonking at the front", FALSE)
+	test ("fillyjonk at the front", TRUE)
+	test ("at the end, fillyjonk", TRUE)
+	test ("in the middle fillyjonk without caps", TRUE)
+	test ("in the middle Fillyjonk with caps", TRUE)
+
 	pan_object_unref (PAN_OBJECT (f));
 
 #undef filter
