@@ -655,7 +655,7 @@ int main (void)
 		char * expected;
 		char * out;
 
-		in = "~/.signature";
+		in = "~"G_DIR_SEPARATOR_S".signature";
 		expected = g_build_filename (home_dir, ".signature", NULL);
 		out = pan_file_normalize (in, NULL);
 		if (pan_strcmp (expected, out))
@@ -663,7 +663,7 @@ int main (void)
 		g_free (expected);
 		g_free (out);
 
-		in = "~//.signature";
+		in = "~"G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S".signature";
 		expected = g_build_filename (home_dir, ".signature", NULL);
 		out = pan_file_normalize (in, NULL);
 		if (pan_strcmp (expected, out))
@@ -679,22 +679,28 @@ int main (void)
 		g_free (expected);
 		g_free (out);
 
-		in = "/var/tmp/.signature";
-		expected = "/var/tmp/.signature";
+		in = G_DIR_SEPARATOR_S"var"G_DIR_SEPARATOR_S"tmp"G_DIR_SEPARATOR_S".signature";
+		expected = G_DIR_SEPARATOR_S"var"G_DIR_SEPARATOR_S"tmp"G_DIR_SEPARATOR_S".signature";
 		out = pan_file_normalize (in, NULL);
 		if (pan_strcmp (expected, out))
 			return num;
 		g_free (out);
 
-		in = "///var//tmp/////.signature";
-		expected = "/var/tmp/.signature";
+		in = G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S
+		     "var"G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S"tmp"
+		     G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S
+		     ".signature";
+		expected = G_DIR_SEPARATOR_S"var"G_DIR_SEPARATOR_S"tmp"G_DIR_SEPARATOR_S".signature";
 		out = pan_file_normalize (in, NULL);
 		if (pan_strcmp (expected, out))
 			return num;
 		g_free (out);
 
-		in = "  ///var//tmp/////.signature     ";
-		expected = "/var/tmp/.signature";
+		in = "  "G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S
+		     "var"G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S"tmp"
+		     G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S
+		     ".signature     ";
+		expected = G_DIR_SEPARATOR_S"var"G_DIR_SEPARATOR_S"tmp"G_DIR_SEPARATOR_S".signature";
 		out = pan_file_normalize (in, NULL);
 		if (pan_strcmp (expected, out))
 			return num;
