@@ -197,6 +197,8 @@ int main (void)
 	if (strcmp(in,out)) return num;
 	replace_gstr (&out, NULL);
 	g_message ("PASS rot13 twice for double security");
+	g_free (in);
+	g_free (out);
 
 	/* rot13 3 */
 	++num;
@@ -205,6 +207,7 @@ int main (void)
 	expected_out = "urer vf n yvar jvgu n ,, ohg gur yvar fubhyq abg or gehapngrq.";
 	if (strcmp (in, expected_out)) return num;
 	g_message ("PASS rot13 with , , or  characters");
+	g_free (in);
 
 	/* mute quoted test 2 */
 	++num;
@@ -298,5 +301,6 @@ int main (void)
 
 	/* success */
 	g_message ("All wrap tests passed");
+	text_massager_free (tm);
 	return 0;
 }
